/* $Header: https://svn.mpia.de/gulli/TwiceAsNice/branch/unstable/ln/ltcs/test/LbtoIIFSimulator/LbtoIIFInterfaceI.cc 14502 2017-07-17 08:46:15Z mathar $
*/

#include <LbtoIIFInterfaceI.h>
#include <Nice/Log.h>
#include <Nice/Map.h>
#include <sofa.h>


/** defining this enables some static huge list installation
* perhaps needed for test/iif/Service/LbcsIIFServiceWorkerTest.cc.
* This usually takes too long to compile because the optimizer passes out. RJM 2017-01-30
#define ENABLE_LBCSIIFSERVICEWORKERTEST
*/

/** This indicates we have a newer version of the ln source code.
* It means the $INSROOT/lib contains a shared version libsofa_so.0.0.0
*/
#define HAVE_LIBSOFA_C_SO

/** @brief dtor
* Does nothing.
*/
lbto::IIFServerI::~IIFServerI()
{
}

/** @brief Check authorization for control the telescope.
* @return Result object
*/
lbto::result lbto::IIFServerI::Authorize(const Ice::Current &)
{
#if 0
	return m_errorResult;
#endif
	D_LOG(__func__) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief StepFocus moves the respective focus position, by moving the OPE
* @brief a given distance in the direction of of the telescope�~@~Ys Z axis.
* @param relPos distance to move the OPE [millimeter]
* @param ope Optical element :  M1 or M2 or M3 or M1M2
* @param side the side this command applies to.
* @return Result object.
*/
lbto::result lbto::IIFServerI::StepFocus(Ice::Double relPos, const std::string & ope,
	const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__<< " relPost " << relPos << " ope " << ope << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Moves an OPE to a new absolute position z to adjust or to define a new focus  position.
* @param absPos distance to move the OPE [millimeter]
* @param ope Optical element :  M1 or M2 or M3 or M1M2
* @param side the side this command applies to.
* @return Result object.
*/
lbto::result lbto::IIFServerI::MoveFocus(Ice::Double absPos, const std::string & ope,
	const std::string & side, const Ice::Current &)
{
	D_LOG(__func__ << " absPos " << absPos << " ope " << ope << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief OffsetPointing moves the telescope a small distance, setting the value
* @brief of the telescope pointing coordinates to the new position.
* @param rotangle Rotator angle [radian]
* @param offx        Offset in X [radian/pixels]
* @param offy Offset in Y [radian/pixels]
* @param coordsys Coordinate system: RADEC, AZALT, DETXY, GALACTIC
* @param ope Optical element : DEFAULT, M1, M2, M3, HEXAPOD, MOUNT
* @param np New position flag
* @param movetype movement type to apply: Relative / Absolute
* @param side the side this command applies to.
* @return Result object.
*/
lbto::result lbto::IIFServerI::OffsetPointing(Ice::Double _rotangle,
	Ice::Double _offx,
	Ice::Double _offy,
	const std::string & _coordsys,
	const std::string & _ope,
	const std::string & _np,
	const std::string & _movetype,
	const std::string & _side,
	const Ice::Current &)
{
	D_LOG(__func__ << " angle " << _rotangle << " offx " << _offx
	      << " offy " << _offy << " " << _coordsys << " " << _ope << " " << _np
	      << " " << _movetype << " " << _side);

#if 0
	//return success:
	res.rescode = 0;
	res.resmsg = msg;
	//  0: "OffsetPointing lbto::result status: OK"

	//return failed:
	res.rescode = 1; // error
	res.resmsg = msg;
	// 0: "OffsetPointing lbto::result status: ERROR"
	// 1-N: "Reason why it fails ..."

	res.rescode = 2; // warning
	res.resmsg = msg;
	// 0: "OffsetPointing lbto::result status: WARNING"
	// 1-N: "Warning massages ..."

	// error and warning comes from the sub-instrument
	// a complete list doesn't exist.
#endif
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief OffsetPointing moves the telescope a small distance, setting the value
* @brief of the telescope pointing coordinates to the new position.
* @param rotangle Rotator angle [radian]
* @param offx        Offset in X [radian/pixels]
* @param offy Offset in Y [radian/pixels]
* @param coordsys Coordinate system: RADEC, AZALT, DETXY, GALACTIC
* @param movetype movement type to apply: Relative / Absolute
* @param side the side this command applies to.
* @return Result object.
* @since 2016-11-17
*/
lbto::result lbto::IIFServerI::OffsetPointing2(Ice::Double _rotangle,
	Ice::Double _offx,
	Ice::Double _offy,
	const std::string & _coordsys,
	const std::string & _movetype,
	const std::string & _side,
	const Ice::Current &)
{
	D_LOG(__func__ << " " << _rotangle << " " << _offx
	      << " " << _offy << " " << _coordsys
	      << " " << _movetype << " " << _side);
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief OffsetGuiding moves the telescope a small distance without changing
* @brief the value of the pointing coordinates.
* @param rotangle Rotator angle [radian]
* @param offx Offset in X [radian/pixels]
* @param offy Offset in Y [radian/pixels]
* @param coordsys Coordinate system: AZALT
* @param side the side this command applies to.
* @return Result object.
*/
lbto::result lbto::IIFServerI::OffsetGuiding(Ice::Double, Ice::Double,
	Ice::Double, const std::string &,
	const std::string &,
	const Ice::Current &)
{
	D_LOG(__func__) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief This command allows the user to use the current sky or focal plane location achieved via the
* @brief OffsetPointing command and update the previous values; To be used in conjunction with the OffsetPointing cmd.
* @param offset_coord the offset coordinate system.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::UpdatePointingReference(const std::string & offset_coord,
	const std::string & side, const Ice::Current &)
{
	D_LOG(__func__ << " offset_coord " << offset_coord << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief TipTilt moves an OPE in tip or tilt direction, relative to the current position.
* @param xrot rotation in x [microradian]
* @param yrot rotation in y [microradian]
* @param ope Optical element: M1 or M2 or M3
* @param side the side this command applies to.
* @return Result object.
*/
lbto::result lbto::IIFServerI::TipTilt(Ice::Double xrot, Ice::Double yrot,
	const std::string & ope, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " xrot " << xrot << " yrot " << yrot << " ope " << ope << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief SetPMTerm sets a singel term in the pointing model.
* @param name name of term (only "CA" and "IE" are allowed)
* @param value value of term
* @param side the side this command applies to.
* @return Result object.
*/
lbto::result lbto::IIFServerI::SetPMTerm(const std::string & name, Ice::Double value,
	const std::string & side, const Ice::Current &)
{
	D_LOG(__func__ << " name " << name << " value " << value << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief SetPMTerm sets a singel term in the pointing model.
* @param name name of term (only "CA" and "IE" are allowed)
* @param value value of term
* @param movetype 
* @param side the side this command applies to.
* @return Result object.
* @since 2016-11-17
*/
lbto::result lbto::IIFServerI::SetPMTerm2(const std::string & name, Ice::Double value,
	const std::string & movetype, const std::string & side, const Ice::Current & c)
{
	D_LOG(__func__<< " name " << name << " value " << value << " movetype " << movetype << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief MoveXYZ moves the primary and secondary together in X, Y and Z direction.
* @param xmov movement in x [millimeter]
* @param ymov movement in y [millimeter]
* @param zmov movement in z [millimeter]
* @param side the side this command applies to.
* @return Result object.
*/
lbto::result lbto::IIFServerI::MoveXYZ(Ice::Double xmov, Ice::Double ymov, Ice::Double zmov,
	const std::string & side, const Ice::Current &)
{
	D_LOG(__func__  << " xmov " << xmov << " ymov " << ymov << " zmov " << zmov << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief MoveXY moves an OPE in X or Y direction, relative to the current position.
* @param xmov movement in x [millimeter]
* @param ymov movement in y [millimeter]
* @param ope Optical element: M1 or M2
* @param side the side this command applies to.
* @return Result object.
*/
lbto::result lbto::IIFServerI::MoveXY(Ice::Double xmov, Ice::Double ymov,
	const std::string & ope, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__  << " xmov " << xmov << " ymov " << ymov << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief This command combines the functionality of MoveXY, TipTilt, StepFocus and MoveFocus.
* @param x movement in x [millimeter]
* @param y movement in y [millimeter]
* @param z movement in z [millimeter]
* @param rx rotation in x [microradian]
* @param ry rotation in y [microradian]
* @param rz rotation in z [microradian]
* @param dflag  6 bits with a bit for each of the preceding 6 variables. Bit 0 enables X, bit 1 enables Y, bit 2 enables Z, and so on.
* @param movetype movement type: ABS, REL
* @param ope Optical element: DEFAULT, M1, M2, M3, M1M2, M1M3, M2M3, M1M2M3
* @param time lookahead time for the collimation correction [sec]
* @param side the side this command applies to.
* @return Result object.
*/
lbto::result lbto::IIFServerI::Move(Ice::Double x, Ice::Double y, Ice::Double z,
	Ice::Double rx, Ice::Double ry, Ice::Double rz,
	Ice::Int dflag, const std::string & movetype,
	const std::string & ope, Ice::Int time,
	const std::string & side, const Ice::Current &)
{
	D_LOG(__func__ << " x " << x << " y " << y << " z " << z << " rx " << rx << " ry " << ry << " rz " << rz << " dflag " << dflag) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Set the target information to be used by PresetTelescope
* @param coord1 Coordinate x [radian]
* @param coord2 Coordinate y [radian]
* @param coordsys Coordinate system: RADEC, AZALT, GALACTIC
* @param epoch
* @param wl Target wavelength
* @return Result object.
* @note This is not an IIF/TCS command and is not completed and must be completed.
*/
lbto::result lbto::IIFServerI::SetTarget(Ice::Double coord1, Ice::Double coord2,
	const std::string & coordsys, Ice::Double epoch,
	Ice::Float wl, const Ice::Current &)
{
	D_LOG(__func__ << " coord1 " << coord1 << " coord2 " << coord2 << " coordsys " << coordsys << " epoch " << epoch << " wl " << wl) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Sets the target, and eventually, the guide stars. (first element on the list is used as target)
* @param list List of objects.
* @return Result object
* @note This is not an IIF/TCS command.
*/
lbto::result lbto::IIFServerI::SetStars(const lbto::SeqPos & _stars,
	const Ice::Current &)
{
	D_LOG(__func__<<" " << _stars.size() << " stars") ;
	for (unsigned int i = 0; i < _stars.size(); i++)
	{
		D_LOG(__func__<<" " << _stars[i] ) ;
	}

#if 0
	//return success:
	res.rescode = 0;
	res.resmsg = msg;
	//  0: "OffsetPointing lbto::result status: OK"

	//return failed:
	res.rescode = 1; // error
	res.resmsg = msg;
	// 0: "OffsetPointing lbto::result status: ERROR"
	// 1-N: "Reason why it fails ..." //

	res.rescode = 2; // warning
	res.resmsg = msg;
	// 0: "OffsetPointing lbto::result status: WARNING"
	// 1-N: "Warning massages ..."
#endif
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Sets the target, and eventually, the guide stars. (first element on the list is used as target)
* @param list List of objects.
* @return Result object
* @note This is not an IIF/TCS command.
* @deprected use SetStars2
*/
lbto::result lbto::IIFServerI::SetStarsNew(const lbto::SeqNewPos & list,
        const Ice::Current & c)
{
	D_LOG(__func__) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Sets the target, and eventually, the guide stars. (first element on the list is used as target)
* @param list List of objects.
* @return Result object
* @note This is not an IIF/TCS command.
* @since 2016-11-17
*/
lbto::result lbto::IIFServerI::SetStars2(const lbto::SeqPos2 & list,
        const Ice::Current & c)
{
	D_LOG(__func__<<" " << list.size() << " stars") ;
	for (unsigned int i = 0; i < list.size(); i++)
	{
		D_LOG(__func__<<" " << list[i] ) ;
	}

	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Removes the selected target and guide stars from the system.
* @return Result object
* @note This is not an IIF/TCS command.
*/
lbto::result lbto::IIFServerI::ClearStars(const Ice::Current &)
{
	D_LOG(__func__) ;
	/* return m_errorResult; return is same as SetStars*/

	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Set the hotspot coordinates to be used by PresetTelescope command
* @param coord1 Coordinate x [pixel]
* @param coord2 Coordinate y [pixel]
* @return Result object.
* @note This is not an IIF/TCS command.
*/
lbto::result lbto::IIFServerI::SetHotspot(Ice::Double _offx, Ice::Double _offy,
        const Ice::Current &)
{
	D_LOG(__func__ << " offx " << _offx << " offy " << _offy);
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}


/** @brief Removes the hotspot from the system.
* @return Result object.
* @note This is not an IIF/TCS command.
*/
lbto::result lbto::IIFServerI::ClearHotspot(const Ice::Current &)
{
	D_LOG(__func__) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Set the offset coordinates to be used by PresetTelescope
* @param _offx Coordinate x [radian/pixel]
* @param _offy Coordinate y [radian/pixel]
* @param coordsys Coordinate system: RADEC, ALTAZ, DETXY, GALACTIC.
* @return Result object.
* @note This is not an IIF/TCS command.
*/
lbto::result lbto::IIFServerI::SetOffset(Ice::Double _offx, Ice::Double _offy,
        const std::string & _coordsys,
        const Ice::Current &)
{
	D_LOG(__func__ << " " << _offx << " " << _offy << " " << _coordsys);
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Set the offset coordinates to be used by PresetTelescope
* @param coord1 Coordinate x [radian/pixel]
* @param coord2 Coordinate y [radian/pixel]
* @param system Coordinate system: RADEC, ALTAZ, DETXY, GALACTIC.
* @param movetype Coordinate system: RADEC, ALTAZ, DETXY, GALACTIC.
* @return Result object.
* @note This is not an IIF/TCS command.
* @deprecated Use SetOffset2
* * @since 2016-11-17
*/
lbto::result lbto::IIFServerI::SetOffsetNew(Ice::Double coord1, Ice::Double coord2,
        const std::string & system,
        const std::string & movetype,
        const Ice::Current & c)
{
	D_LOG(__func__) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Set the offset coordinates to be used by PresetTelescope
* @param coord1 Coordinate x [radian/pixel]
* @param coord2 Coordinate y [radian/pixel]
* @param system Coordinate system: RADEC, ALTAZ, DETXY, GALACTIC.
* @param movetype 
* @return Result object.
* @note This is not an IIF/TCS command.
* @since 2016-11-17
*/
lbto::result lbto::IIFServerI::SetOffset2(Ice::Double coord1, Ice::Double coord2,
        const std::string & system,
        const std::string & movetype,
        const Ice::Current & c)
{
	D_LOG(__func__ << " coord1 " << coord1 << " coord2 " << coord2 << " system " << system << " movetype " << movetype);
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Removes the offset from the system.
* @return Result object.
* @note This is not an IIF/TCS command.
*/
lbto::result lbto::IIFServerI::ClearOffset(const Ice::Current &)
{
	D_LOG(__func__) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Set the non-sidereal values to be used by PresetTelescope
* @param target non-sidereal target
* @param override non-sidereal override flag
* @return Result object.
* @note This is not an IIF/TCS command.
*/
lbto::result lbto::IIFServerI::SetNonSidereal(
    const lbto::nonsidereal & target, bool override,
    const Ice::Current &)
{
	D_LOG(__func__ << " target " << target << " override " << override) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Removes the non-sidereal object from the system.
* @param override non-sidereal override flag
* @return Result object.
* @note This is not an IIF/TCS command.
*/
lbto::result lbto::IIFServerI::ClearNonSidereal(bool override,
        const Ice::Current &)
{
	D_LOG(__func__ << " override " << override) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Update the non-sidereal values
* @param RArate the new RArate
* @param DECrate the new DECrate
* @param side the side this command applies to.
* @return Result object.
*/
lbto::result lbto::IIFServerI::UpdateNonSiderealTarget(
    Ice::Double RArate, Ice::Double DECrate, const Ice::Current &)
{
	D_LOG(__func__ << " RArate "<< RArate << " DECrate " << DECrate) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief
* @param objtype the non-sidereal type
* @param target
* @param override
* @return Result object.
* @since 2016-11-17
*/
lbto::result lbto::IIFServerI::SetNonSiderealObject(
	const std::string & objtype,
	const lbto::nonsidereal & target, bool override,
	const Ice::Current & c)
{
	D_LOG(__func__ << " objtype " << objtype << " target " << target << " override " << override) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief
* @param objtype the non-sidereal type
* @param override
* @return Result object.
* @since 2016-11-17
*/
lbto::result lbto::IIFServerI::ClearNonSiderealObject(
	const std::string & objtype, bool override,
	const Ice::Current & c)
{
	D_LOG(__func__ << " objtype " << objtype << " override " << override) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Update the non-sidereal values
* @param objtype the non-sidereal type
* @param RArate the new RArate
* @param DECrate the new DECrate
* @return Result object.
* @since 2016-11-17
*/
lbto::result lbto::IIFServerI::UpdateNonSiderealObject(
	const std::string & objtype,
	Ice::Double RArate, Ice::Double DECrate, const Ice::Current & c)
{
	D_LOG(__func__ << " objtype " << objtype << " RArate "<< RArate <<" DECrate " << DECrate) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief PresetTelescope slew the telescope into position in order to begin an observation cycle.
* @param rotangle initial rotator angle [radian]
* @param rotmode rotator mode: POSITION, PARALLACTIC, NATIVE, GRAVITATIONAL, IDLE.
* @param mode Operation mode: STATIC, TRACK, ACQUIRE, GUIDE, ADAPTIVE, ACTIVE, INTERFEROMETRIC.
* @param side the side this command applies to.
* @return Result object
* @note This is an IIF/TCS command, but the parameters are different.
*/
lbto::result lbto::IIFServerI::PresetTelescope(Ice::Double rotangle,
	const std::string & rotmode,
	const std::string & mode,
	const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " rotangle " << rotangle << " rotmode "
	      << rotmode << " mode " << mode << " side " << side);
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief BinocularControl clears the target information in the PCS.
* @param flag Preset control flag: CLEARPRESET, SYNCPRESET, SYNCOFFSET, ADJUSTBALANCE.
* @return Result object
*/
lbto::result lbto::IIFServerI::BinocularControl(const std::string & flag,
	const Ice::Current &)
{
	D_LOG(__func__ << " flag "<< flag) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Reads a block of entries from the data dictionary in one shot.
* @brief If FITS information is defined here, it will be included in the FITS headers (only used by the IRTC client).
* @param list DD entry list
* @return Result object.
*/
lbto::result lbto::IIFServerI::GetParameter(const lbto::SeqDD & _dd,
	const Ice::Current &)
{
	D_LOG(__func__ << " keyword ct " << _dd.size() ) ;
	lbto::result res;
	res.rescode = 0;
	std::map<std::string, std::string> dd;
	if (m_ddit < 8)
	{
		dd = m_dd1;
		m_ddit++;
	}
	else if (m_ddit < 16)
	{
		dd = m_dd2;
		m_ddit++;
	}

	if (m_ddit > 11)
		m_ddit = 0;

	std::map<std::string, std::string>::iterator dd_it = dd.begin();
	std::vector<std::string> param;
	param.push_back("GetParameter lbto::result status: OK");
	for (unsigned int i = 0; i < _dd.size(); i++)
	{
		dd_it = dd.find(_dd[i].DDname);
		if (dd_it != dd.end())
		{
			D_LOG(__func__ << " " << dd_it->second);
			param.push_back(dd_it->second);
		}
	}
	res.resmsg = param;

#if 0
	//return success:
	res.rescode = 0;
	res.resmsg = msg;
	//  0: "GetParameter lbto::result status: OK"
	//  1-N: value the we ask for in the order that we ask for (one value per message)

	//return failed:
	res.rescode = 1; // error
	res.resmsg = msg;
	// 0: "GetParameter lbto::result status: ERROR"
	// 1-I: Restricted : Parameter_name //"the unknown name of the requested value"
	// 1-I: Offset not found : Parameter_name (Parameter_name_translate) // Parameter_name is the name the we ask for

	res.rescode = 2; // no warning available
	res.resmsg = msg;
#endif

	return res;
}

/** @brief Sets the values of the specified data dictionary entries on the TCS in one shot.
* @param list DD entry list with entry-value pairs
* @return Result object.
*/
lbto::result lbto::IIFServerI::SetParameter(const lbto::SeqDD & list,
	const Ice::Current &)
{
	D_LOG(__func__) ;
	for (unsigned int i = 0; i < list.size(); i++)
	{
		D_LOG(__func__ << " " << list[i] );
	}
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Rotates the primary and the secondary mirror around a common point.
* @param z rotation point [millimeter]
* @param angle rotation angle [radian]
* @param direction direction angle [radian]
* @param side the side this command applies to.
* @return Result object.
*/
lbto::result lbto::IIFServerI::RotateCommon(Ice::Double z, Ice::Double angle,
	Ice::Double direction, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " z " << z << " angle " << angle << " direction " << direction << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief the tertiary mirror is rotated to adjust the incoming beam angle for  the instrument.
* @param angle rotation angle [radian]
* @param movetype movement type: ABS, REL
* @param side the side this command applies to.
* @return Result object.
*/
lbto::result lbto::IIFServerI::RotateZ(Ice::Double angle, const std::string & movetype,
	const std::string & side, const Ice::Current &)
{
	D_LOG(__func__ << " angle " << angle << " movetype " << movetype << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Use this command to to rotate the primary about a point on axis.
* @param distance the distance above the mirror about which to rotate
* @param angle the rotation angle
* @param direction the direction in shich to rotate
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::RotatePrimary(Ice::Double distance, Ice::Double angle,
	Ice::Double direction, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " distance " << distance << " angle " << angle << " direction " << direction << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Sends an array of 28 Zernike polynomial coefficients to be compensated by the actuators
* @brief of the primary or secondary mirror.
* @param wfs coefficients list
* @param OPE Optical element: M1, M2
* @param side the side this command applies to.
* @return Result object.
*/
lbto::result lbto::IIFServerI::SendWavefront(const lbto::SeqWF & wfs,
	const std::string & OPE,
	const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " OPE " << OPE << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Updates the target effective wavelength without issuing a new preset.
* @param wavelength new target wavelength. Range is [0.3, 15.0] [microns]
* @param side the side this command applies to.
* @return Result object.
*/
lbto::result lbto::IIFServerI::UpdateTargetWavelength(Ice::Float wavelength,
	const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " wavelength " << wavelength << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Use this command to log a message inside the TCS's logging system.
* @param event an event name that the instrument desires to log
* @param description the descriptive text that the instrument desires to log
* @return Result object
*/
lbto::result lbto::IIFServerI::LogEvent(const std::string & event, const std::string & description,
	const Ice::Current &)
{
	D_LOG(__func__ << " " << event << " " << description ) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Use this command to get the rotator trajectory for the LBC.
* @param seconds the number of seconds in the trajectory
* @param interval the time between points
* @param start the start time in MJD
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::GetRotatorTrajectory(
	Ice::Double seconds, Ice::Double interval,
	Ice::Double start, const ::std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " seconds " << seconds << " interval " << interval << " start " << start << " side " << side) ;
	lbto::result res;
	res.rescode = 0;
#ifdef ENABLE_LBCSIIFSERVICEWORKERTEST
	res.resmsg = m_PFT01;
#else
	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	res.resmsg.push_back(msg0);
#endif
	return res;

#if 0
	//return success:
	res.rescode = 0;
	res.resmsg = msg;
	//  0: "GetRotatorTrajectory lbto::result status: OK"
	//  1-N: value the we ask for in the order that we ask for (one value per message)

	//return failed:
	res.rescode = 1; // error
	res.resmsg = msg;
	// 0: "GetRotatorTrajectory lbto::result status: ERROR"
	//if it fails we get the error message
#endif
}

/** @brief Use this command to get rotator polynomials.
* @param start the start time in MJD (UT days)
* @param count the number of points in the trajectory
* @param interval the time between points (seconds)
* @param order the desired polynomial order (0,1,2)
* @param side the side this command applies to.
* @return Result object
* @since 2017-02-01 Try to calculate vals from the current parallactic angle
* @since 2017-02-14 Flip sign of polynomial coefficients (as IIF does)
* @todo get a realistic local sidereal time  via sofa
*/
lbto::result lbto::IIFServerI::GetRotatorPolynomials(
	Ice::Double start, Ice::Int count, Ice::Double interval,
	Ice::Int order, const std::string & side, const Ice::Current &)
{
	D_LOG(__func__ << " start " << start << " count " << count 
		<< " interval " << interval << " order " << order << " side " << side) ;

	lbto::result res;
	res.rescode = 0;

	if ( order < 0 )
	{
		res.rescode = 1 ;
		order = 1 ;
	}

	if ( count < 1 )
	{
		res.rescode = 1 ;
		count = 1 ;
	}

	if ( interval <= 0. )
	{
		res.rescode = 2 ;
		interval = 1.0 ;
	}

	std::string msg0(__func__) ;
	msg0 += side + " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning, Order or count too small" ;
		break;
	case 2 :
		msg0 += "Error, negative or zero interval" ;
	}
	res.resmsg.push_back(msg0);

#ifdef ENABLE_LBCSIIFSERVICEWORKERTEST
	std::string time;
	std::string position;
	std::string velocity;
	std::string acceleration;

	if (count > (int) m_poly.size())
		count = m_poly.size() - 1;

	res.resmsg.push_back(*m_poly.begin());
	for (int i = 1; i <= count; i++)
	{
		std::stringstream poly;
		poly << *(m_poly.begin() + i);
		poly >> time >> position >> velocity >> acceleration;
		std::stringstream ss;
		// predefined m_poly date will be overwritten with a calculated date
		ss << std::fixed << (Nice::Date::fromMJD(start) + Nice::Time::seconds((i-1) * interval)).toMJD();
		ss << " " <<  position;
		ss << " " <<  velocity;
		ss << " " <<  acceleration;

		res.resmsg.push_back(ss.str());
	}
#else
	/* transform target into a SkyPoint
	* todo: add a operator() to the position2 resp. LbcsStar class
	*/
	Nice::SkyPoint sp(Nice::DMS(target.coord1/Nice::DMS::DegToRad), 
		Nice::DMS(target.coord2/Nice::DMS::DegToRad)) ;

	Nice::Date tim = Nice::Date::fromMJD(start) ;

	/* number of seconds of local taylor approximation
	*/
	const float portion = 0.2*interval ;

	/* time interval, converted to degrees.
	* 86400 =24*3600 seconds make 360 degrees. 360/86400=0.00416666
	*/
	const Nice::DMS taylIntl(portion*0.00416) ;

	/* emit polynomials that have the negative sign of the parallactic angle
	*/
	const int iifSign = -1 ;
	for (int i = 0; i < count; i++)
	{
		const double nowmjd = tim.toMJD() ;
		std::stringstream ss;
		ss << std::fixed << nowmjd;

		/* very rough first approximation here just for illustration, no TT correction
		* st has units of radians. But this nees a shared sofa library...todo..
		*/
#ifdef HAVE_LIBSOFA_C_SO
		double st = iauGmst06(DJM0,nowmjd,DJM0,nowmjd) ;
#else
		double st = 0. ;
#endif
		Nice::DMS ha ;
		ha.setRadians(st) ;
		/* get a local polynomial approximation of 3 points
		* Then we emit the polynomial coefficients all with the opposite
		* sign as the getRotatorPolynomial of the actual IIF service would do.
		*/
		Nice::DMS p[3] ;
		for(int j=0 ; j < 3 ;j++)
		{
			p[j] = sp.pang(ha,geosite.Latitude) ;
			/* move on by a tenths of the interval.
			ha += taylIntl ; // that call in newer Nice versions....
			*/
			ha = ha+taylIntl ;
		}
		ss << " " <<  iifSign*p[0].radians() ;

		if ( order >= 1)
			ss << " " <<  iifSign*(p[1].radians()-p[0].radians())/portion ;

		if ( order >= 2)
			/* approximation: second finite element derivative divided by 2
			*/
			ss << " " <<  iifSign*(p[2].radians()-2*p[1].radians()+p[0].radians())/(2*portion*portion) ;

		res.resmsg.push_back(ss.str());

		tim += Nice::Time::seconds(interval) ;
	}
#endif
	return res;
}

/** @brief Use this command to set the standby level.
* @param level the standby level
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::Standby(Ice::Int level, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " level " << level << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief This command is to provide some control over the use of the rotator cablewrap.
* @param azflag flag to control AZ wrap.
* @param rotflag flag to control rotator wrap.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::MaximizeWrapTime(bool azflag, bool rotflag, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " azflag " << azflag << " rotflag " << rotflag << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Temporarily suspends the current guiding operation.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::PauseGuiding(const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Resumes suspended guiding operation.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::ResumeGuiding(const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Sets guider hotspot.
* @param coord1 coordinate x [pixel].
* @param coord2 coordinate y [pixel].
* @param movetype the move type, REL or ABS.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::SetGuidingHotspot(
	Ice::Double coord1, Ice::Double coord2,
	const std::string & movetype, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " coord1 " << coord1 << " coord2 " << coord2 << " movetype " << movetype << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Readies the cable wrap and rotator.
* @param enable flag to ready/unready the rotator.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::RotReady(bool enable, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " enable " << enable << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief If the rotator is tracking or slewing, this command makes it stop moving and hold the position.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::RotHold(const std::string & side, const Ice::Current &)
{
	D_LOG(__func__ << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Makes rotator begin tracking according to the polynomial stream it is currently receiving from the PCS.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::RotTrack(const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Makes the rotator move to the specified angle in the specified coordinate frame and hold at that position.
* @param angle [radian]
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::RotServicePosition(Ice::Double angle,
	const std::string & side, const Ice::Current &)
{
	D_LOG(__func__ << " angle " << angle << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Temporarily suspends the current AO operation.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::PauseAO(const std::string & side, const Ice::Current &)
{
	D_LOG(__func__ << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Resumes suspended operation after a PauseAO.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::ResumeAO(const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Start AO operation.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::StartAO(const std::string & side, const Ice::Current &)
{
	D_LOG(__func__ << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Stop AO operation.
* @param reason the reason for stopping.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::StopAO(const std::string & reason, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " reason " << reason << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Acquire the reference star.
* @param repointFlag flag to request the telescope to repoint.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::AcquireRefAO(bool repointFlag, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " repoitnFlag " << repointFlag << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Check the reference star position.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::CheckRefAO(const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Refine AO.
* @param method the method used for refinement.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::RefineAO(const std::string & method, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " method " << method << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Do AcquireRefAO, StartAO.
* @param repointFlag flag to request the telescope to repoint in AcquireRefAO.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::RunAO(const std::string & repointFlag, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " repointFlag " << repointFlag << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Request flat shape for AO.
* @param flat the flat name.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::PresetFlatAO(const std::string & flat,
	const std::string &side, const Ice::Current &)
{
	D_LOG(__func__ << " flat " << flat << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Send 672 mode corrections to AO.
* @param modes modes list
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::CorrectModesAO(const lbto::SeqModes & modes,
	const std::string & side, const Ice::Current &)
{
	D_LOG(__func__ << " modes " ) ;
	for (int i=0 ; i < std::min( (int)modes.size(),5) ; i++)
	{
		D_LOG(" " << modes[i]) ;
	}
	D_LOG(__func__ << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Modify AO.
* @param NModes the number of corrected modes.
* @param Freq  the CCD integration frequency.
* @param Nbins  the CCD binning.
* @param TTMod  Tip-Tilt internal mirror modulation.
* @param F1spec the selected position of filter wheel #1.
* @param F2spec the selected position of filter wheel #2.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::ModifyAO(Ice::Int NModes, Ice::Double Freq, Ice::Int Nbins,
	Ice::Double TTmod, const std::string & F1spec,
	const std::string & F2spec, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " Nmodes " << NModes << " Freq " << Freq << " Nbins " << Nbins << " TTmod " << TTmod) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Preset AO.
* @param AOMode the AO operatin mode.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::PresetAO(const std::string & AOMode, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " AOMode " << AOMode << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief OffsetXY AO.
* @param DeltaX the X offset.
* @param DeltaY the Y offset.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::OffsetXYAO(Ice::Double DeltaX, Ice::Double DeltaY,
	const std::string & side, const Ice::Current &)
{
	D_LOG(__func__) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief OffsetZ AO.
* @param DeltaZ the Z offset.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::OffsetZAO(Ice::Double DeltaZ, const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Sets the AO reference star
* @param list List of objects.
* @return Result object
* @note This is not an IIF/TCS command.
*/
lbto::result lbto::IIFServerI::SetReference(const lbto::SeqPos & list,
	const Ice::Current &)
{
	D_LOG(__func__<<" " << list.size() << " stars") ;
	for (unsigned int i = 0; i < list.size(); i++)
	{
		D_LOG(__func__<<" " << list[i] ) ;
	}
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Sets the AO reference star
* @param list List of objects.
* @return Result object
* @note This is not an IIF/TCS command.
* @deprecated use SetReference2
*/
lbto::result lbto::IIFServerI::SetReferenceNew(const lbto::SeqNewPos & /*_stars*/,
        const Ice::Current &)
{
	D_LOG(__func__) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Sets the AO reference star
* @param list List of objects.
* @return Result object
* @note This is not an IIF/TCS command.
*/
lbto::result lbto::IIFServerI::SetReference2(const lbto::SeqPos2 & list,
        const Ice::Current & c)
{
	D_LOG(__func__<<" " << list.size() << " stars") ;
	for (unsigned int i = 0; i < list.size(); i++)
	{
		D_LOG(__func__<<" " << list[i] ) ;
	}
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Removes the AO reference star from the system.
* @return Result object
* @note This is not an IIF/TCS command.
*/
lbto::result lbto::IIFServerI::ClearReference(const Ice::Current &)
{
	D_LOG(__func__) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Use this command to get focal plane positions for LN AO reference objects.
* @param list the list of stars.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::GetKFPCoordinates(const lbto::SeqPos & list,
        const std::string & side,
        const Ice::Current &)
{
	D_LOG(__func__<<" " << list.size() << " stars ") ;
	for (unsigned int i = 0; i < list.size(); i++)
	{
		D_LOG(__func__<<" " << list[i] ) ;
	}
	D_LOG(__func__<<" side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Use this command to get focal plane positions for LN AO reference objects.
* @param list the list of stars.
* @param side the side this command applies to.
* @return Result object
* @deprecated Use GetKFPCoordinates2
*/
lbto::result lbto::IIFServerI::GetKFPCoordinatesNew(
	const lbto::SeqNewPos & _stars, const std::string & _side,
	const Ice::Current &)
{
	D_LOG(__func__) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Use this command to get focal plane positions for LN AO reference objects.
* @param list the list of stars.
* @param side the side this command applies to.
* @return Result object
* @since 2016-11-17
*/
lbto::result lbto::IIFServerI::GetKFPCoordinates2(
	const lbto::SeqPos2 & list, const std::string & side,
	const Ice::Current & c)
{
	D_LOG(__func__<<" " << list.size() << " stars ") ;
	for (unsigned int i = 0; i < list.size(); i++)
	{
		D_LOG(__func__<<" " << list[i] ) ;
	}
	D_LOG(__func__<<" side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Use this command to set the AGw filter.
* @param filterNumber the number of the filter.
* @param side the side this command applies to.
* @return Result object
*/
lbto::result lbto::IIFServerI::SetAGWFilter(int filterNumber,
	const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " filterNumber " << filterNumber << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief Use this command to set the AGw filter.
* @param factor
* @param side the side this command applies to.
* @return Result object
* @since 2016-11-17
*/
lbto::result lbto::IIFServerI::SetGuidingBinning(int factor,
	const std::string & side,
	const Ice::Current &)
{
	D_LOG(__func__ << " factor " << factor << " side " << side) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief
* @param targenName
* @param startDate
* @param startTime
* @param endDate
* @param endTime
* @param interval
* @param fileName
* @since 2016-11-17
*/
lbto::result lbto::IIFServerI::NSQuery(const std::string & targenName,
	const std::string & startDate,
	const std::string & startTime,
	const std::string & endDate,
	const std::string & endTime,
	const std::string & interval,
	const std::string & fileName,
	const Ice::Current &)
{
	D_LOG(__func__ << " targenName " << targenName << " startDate " << startDate << " startTime " << startTime << " endDate " << endDate << " endTime " << endTime) ;
	std::vector<std::string> msg;

	lbto::result res;
	res.rescode = 0;

	std::string msg0(__func__) ;
	msg0 += " result status: " ;
	switch( res.rescode)
	{
	case 0 :
		msg0 += "OK" ;
		break;
	case 1 :
		msg0 += "Warning" ;
		break;
	case 2 :
		msg0 += "Error" ;
	}
	msg.push_back(msg0);
	res.resmsg = msg;

	return res;
}

/** @brief remove the IIF object
*/
void lbto::IIFServerI::remove(const Ice::Current &)
{
}

/** @brief class parameterized constructor
* @since 2017-05-12 adapted to what's expected by the snapshot() in IIFServiceWorker.
*/
lbto::IIFServerI::IIFServerI() 
	: geosite()
{
	m_ddit = 0;
	m_PFTit = 0;

	m_error.push_back("Not implemented");

	m_errorResult.rescode = -1;
	m_errorResult.resmsg = m_error;

	/* dd1[0] created
	*/
	m_dd1.insert( std::make_pair(std::string("MountAchievedRAHMS"), std::string("13:10:50"))); // [0, 2PI]
	m_dd1.insert( std::make_pair(std::string("MountAchievedDECDMS"), std::string("-02:40:50.5"))); // [-PI/2, PI/2]
	m_dd1.insert( std::make_pair(std::string("ParAngle"), std::string("1.5707"))); // [-PI, PI]
	m_dd1.insert(std::make_pair(std::string("UTC"), std::string("02:22:52"))); // [H M S] [0-23, 0-59, 0-59]
	m_dd1.insert(std::make_pair(std::string("LST"), std::string("05:22:51"))); // [H M S] [0-23, 0-59, 0-59]
	m_dd1.insert( std::make_pair(std::string("MountAchievedALT"), std::string("1.31"))); // [-Pi/2, PI/2]
	m_dd1.insert( std::make_pair(std::string("MountAchievedAZ"), std::string("3.71"))); // [0, 2PI]
	m_dd1.insert( std::make_pair(std::string("EPOCH"), std::string("2000.0"))); // [0, 2PI]

#if 0
	m_dd1.insert( std::make_pair(std::string("L_RotCenterRA"), std::string("4.71"))); // [0, 2PI]
	m_dd1.insert( std::make_pair(std::string("L_RotCenterDEC"), std::string("0.7853"))); // [-PI, PI]
	m_dd1.insert( std::make_pair(std::string("R_RotCenterRA"), std::string("4.71"))); // [0, 2PI]
	m_dd1.insert( std::make_pair(std::string("R_RotCenterDEC"), std::string("0.7853"))); // [-PI, PI]

	//		m_dd1.insert(std::make_pair( std::string("HA"),             std::string("-02:00:01") ));
#endif

        /* create dd2[0] */
	m_dd2.insert( std::make_pair(std::string("MountAchievedRAHMS"), std::string("20:17:00.3"))); // [0, 2PI]
	m_dd2.insert( std::make_pair(std::string("MountAchievedDECDMS"), std::string("70:33:42"))); // [-PI/2, PI/2]
	m_dd2.insert( std::make_pair(std::string("ParAngle"), std::string("-2.755"))); // [-PI, PI]
	m_dd2.insert(std::make_pair(std::string("UTC"), std::string("06:53:12"))); // [H M S] [0-23, 0-59, 0-59]
	m_dd2.insert(std::make_pair(std::string("LST"), std::string("08:53:11"))); // [H M S] [0-23, 0-59, 0-59]
	m_dd2.insert( std::make_pair(std::string("MountAchievedALT"), std::string("1.31"))); // [-Pi/2, PI/2]
	m_dd2.insert( std::make_pair(std::string("MountAchievedAZ"), std::string("3.71"))); // [0, 2PI]
	m_dd2.insert( std::make_pair(std::string("EPOCH"), std::string("2000.0"))); // [0, 2PI]
#if 0
	m_dd2.insert( std::make_pair(std::string("L_RotCenterRA"), std::string("1.71"))); // [0, 2PI]
	m_dd2.insert( std::make_pair(std::string("L_RotCenterDEC"), std::string("-3.7853"))); // [-PI, PI]
	m_dd2.insert( std::make_pair(std::string("R_RotCenterRA"), std::string("1.71"))); // [0, 2PI]
	m_dd2.insert( std::make_pair(std::string("R_RotCenterDEC"), std::string("-3.7853"))); // [-PI, PI]
	//		m_dd2.insert(std::make_pair( std::string("HA"),             std::string("-03:00:01") ));
#endif


#ifdef ENABLE_LBCSIIFSERVICEWORKERTEST
	/* explicit list of timestamps used for make check RJM 2016-11-17
	*/
	m_PFT01.push_back("2455775.25 2.169512966746");
	m_PFT01.push_back("2455775.250012 2.169525135913");
	m_PFT01.push_back("2455775.250023 2.16953730799");
	m_PFT01.push_back("2455775.250035 2.169549718624");
	m_PFT01.push_back("2455775.250046 2.169561896533");
	m_PFT01.push_back("2455775.250058 2.169574234438");
	m_PFT01.push_back("2455775.250069 2.169586418175");
	m_PFT01.push_back("2455775.250081 2.169598761906");
	m_PFT01.push_back("2455775.250093 2.169610951472");
	m_PFT01.push_back("2455775.250104 2.169623301028");
	m_PFT01.push_back("2455775.250116 2.169635496422");
	m_PFT01.push_back("2455775.250127 2.169647851803");
	m_PFT01.push_back("2455775.250139 2.16966005303");
	m_PFT01.push_back(" 2455775.25015 2.169672414238");
	m_PFT01.push_back("2455775.250162 2.169684621296");
	m_PFT01.push_back("2455775.250174 2.169696988331");
	m_PFT01.push_back("2455775.250185 2.16970920122");
	m_PFT01.push_back("2455775.250197 2.169721574085");
	m_PFT01.push_back("2455775.250208 2.169733792808");
	m_PFT01.push_back(" 2455775.25022 2.1697461715");
	m_PFT01.push_back("2455775.250231 2.16975839606");
	m_PFT01.push_back("2455775.250243 2.169770780581");
	m_PFT01.push_back("2455775.250255 2.169783010975");
	m_PFT01.push_back("2455775.250266 2.16979540133");
	m_PFT01.push_back("2455775.250278 2.169807637558");
	m_PFT01.push_back("2455775.250289 2.169820033745");
	m_PFT01.push_back("2455775.250301 2.169832275812");
	m_PFT01.push_back("2455775.250313 2.169844677831");
	m_PFT01.push_back("2455775.250324 2.169856925734");
	m_PFT01.push_back("2455775.250336 2.169869333588");
	m_PFT01.push_back("2455775.250347 2.16988158733");
	m_PFT01.push_back("2455775.250359 2.169894001016");
	m_PFT01.push_back(" 2455775.25037 2.1699062606");
	m_PFT01.push_back("2455775.250382 2.169918680122");
	m_PFT01.push_back("2455775.250394 2.169930945547");
	m_PFT01.push_back("2455775.250405 2.169943370905");
	m_PFT01.push_back("2455775.250417 2.16995564217");
	m_PFT01.push_back("2455775.250428 2.169968073365");
	m_PFT01.push_back(" 2455775.25044 2.169980350474");
	m_PFT01.push_back("2455775.250451 2.169992787507");
	m_PFT01.push_back("2455775.250463 2.170005070459");
	m_PFT01.push_back("2455775.250475 2.170017513331");
	m_PFT01.push_back("2455775.250486 2.170029802126");
	m_PFT01.push_back("2455775.250498 2.170042250839");
	m_PFT01.push_back("2455775.250509 2.170054545479");
	m_PFT01.push_back("2455775.250521 2.170067000032");
	m_PFT01.push_back("2455775.250532 2.170079300519");
	m_PFT01.push_back("2455775.250544 2.170091760914");
	m_PFT01.push_back("2455775.250556 2.170104067247");
	m_PFT01.push_back("2455775.250567 2.170116533484");
	m_PFT01.push_back("2455775.250579 2.170128845665");
	m_PFT01.push_back(" 2455775.25059 2.170141317745");
	m_PFT01.push_back("2455775.250602 2.170153635775");
	m_PFT01.push_back("2455775.250613 2.1701661137");
	m_PFT01.push_back("2455775.250625 2.17017843758");
	m_PFT01.push_back("2455775.250637 2.170190921351");
	m_PFT01.push_back("2455775.250648 2.170203251079");
	m_PFT01.push_back(" 2455775.25066 2.170215740696");
	m_PFT01.push_back("2455775.250671 2.170228076277");
	m_PFT01.push_back("2455775.250683 2.170240571738");
	m_PFT01.push_back("2455775.250694 2.170252913174");
	m_PFT01.push_back("2455775.250706 2.170265414485");
	m_PFT01.push_back("2455775.250718 2.170277761771");
	m_PFT01.push_back("2455775.250729 2.170290268931");
	m_PFT01.push_back("2455775.250741 2.170302622072");
	m_PFT01.push_back("2455775.250752 2.170315135082");
	m_PFT01.push_back("2455775.250764 2.170327494077");
	m_PFT01.push_back("2455775.250775 2.170340012938");
	m_PFT01.push_back("2455775.250787 2.170352377789");
	m_PFT01.push_back("2455775.250799 2.1703649025");
	m_PFT01.push_back(" 2455775.25081 2.170377273208");
	m_PFT01.push_back("2455775.250822 2.170389803773");
	m_PFT01.push_back("2455775.250833 2.170402180338");
	m_PFT01.push_back("2455775.250845 2.170414716756");
	m_PFT01.push_back("2455775.250856 2.17042709918");
	m_PFT01.push_back("2455775.250868 2.17043964145");
	m_PFT01.push_back(" 2455775.25088 2.170452029737");
	m_PFT01.push_back("2455775.250891 2.170464577862");
	m_PFT01.push_back("2455775.250903 2.170476972005");
	m_PFT01.push_back("2455775.250914 2.170489525989");
	m_PFT01.push_back("2455775.250926 2.170501925993");
	m_PFT01.push_back("2455775.250938 2.170514485833");
	m_PFT01.push_back("2455775.250949 2.170526891701");
	m_PFT01.push_back("2455775.250961 2.170539457397");
	m_PFT01.push_back("2455775.250972 2.170551869129");
	m_PFT01.push_back("2455775.250984 2.170564440684");
	m_PFT01.push_back("2455775.250995 2.170576858278");
	m_PFT01.push_back("2455775.251007 2.170589435693");
	m_PFT01.push_back("2455775.251019 2.170601859153");
	m_PFT01.push_back(" 2455775.25103 2.170614442429");
	m_PFT01.push_back("2455775.251042 2.170626871754");
	m_PFT01.push_back("2455775.251053 2.170639460891");
	m_PFT01.push_back("2455775.251065 2.170651896083");
	m_PFT01.push_back("2455775.251076 2.170664491083");
	m_PFT01.push_back("2455775.251088 2.170676932143");
	m_PFT01.push_back("  2455775.2511 2.170689533002");
	m_PFT01.push_back("2455775.251111 2.170701979931");
	m_PFT01.push_back("2455775.251123 2.170714586657");
	m_PFT01.push_back("2455775.251134 2.170727039456");
	m_PFT01.push_back("2455775.251146 2.170739652046");
	m_PFT01.push_back("2455775.251157 2.170752110715");
	m_PFT01.push_back("2455775.251169 2.17076472917");
	m_PFT01.push_back("2455775.251181 2.170777193711");
	m_PFT01.push_back("2455775.251192 2.170789818033");
	m_PFT01.push_back("2455775.251204 2.170802288445");
	m_PFT01.push_back("2455775.251215 2.170814918636");
	m_PFT01.push_back("2455775.251227 2.170827394921");
	m_PFT01.push_back("2455775.251238 2.17084003098");
	m_PFT01.push_back(" 2455775.25125 2.170852513138");
	m_PFT01.push_back("2455775.251262 2.170865155068");
	m_PFT01.push_back("2455775.251273 2.170877643101");
	m_PFT01.push_back("2455775.251285 2.170890290902");
	m_PFT01.push_back("2455775.251296 2.17090278481");
	m_PFT01.push_back("2455775.251308 2.170915438481");
	m_PFT01.push_back("2455775.251319 2.170927938265");
	m_PFT01.push_back("2455775.251331 2.170940597808");
	m_PFT01.push_back("2455775.251343 2.170953103472");
	m_PFT01.push_back("2455775.251354 2.170965768888");
	m_PFT01.push_back("2455775.251366 2.17097828043");
	m_PFT01.push_back("2455775.251377 2.17099095172");
	m_PFT01.push_back("2455775.251389 2.171003469142");
	m_PFT01.push_back("  2455775.2514 2.171016146306");
	m_PFT01.push_back("2455775.251412 2.171028669607");
	m_PFT01.push_back("2455775.251424 2.171041352647");
	m_PFT01.push_back("2455775.251435 2.171053881832");
	m_PFT01.push_back("2455775.251447 2.171066570749");
	m_PFT01.push_back("2455775.251458 2.171079105815");
	m_PFT01.push_back(" 2455775.25147 2.171091800607");
	m_PFT01.push_back("2455775.251481 2.171104341556");
	m_PFT01.push_back("2455775.251493 2.17111704223");
	m_PFT01.push_back("2455775.251505 2.17112958906");
	m_PFT01.push_back("2455775.251516 2.171142295614");
	m_PFT01.push_back("2455775.251528 2.171154848331");
	m_PFT01.push_back("2455775.251539 2.171167560764");
	m_PFT01.push_back("2455775.251551 2.171180119368");
	m_PFT01.push_back("2455775.251563 2.171192837682");
	m_PFT01.push_back("2455775.251574 2.17120540217");
	m_PFT01.push_back("2455775.251586 2.171218126367");
	m_PFT01.push_back("2455775.251597 2.171230696745");
	m_PFT01.push_back("2455775.251609 2.171243426824");
	m_PFT01.push_back(" 2455775.25162 2.171256003089");
	m_PFT01.push_back("2455775.251632 2.171268739052");
	m_PFT01.push_back("2455775.251644 2.171281321208");
	m_PFT01.push_back("2455775.251655 2.171294063057");
	m_PFT01.push_back("2455775.251667 2.171306651101");
	m_PFT01.push_back("2455775.251678 2.171319398836");
	m_PFT01.push_back(" 2455775.25169 2.171331992771");
	m_PFT01.push_back("2455775.251701 2.171344746393");
	m_PFT01.push_back("2455775.251713 2.171357346221");
	m_PFT01.push_back("2455775.251725 2.171370105732");
	m_PFT01.push_back("2455775.251736 2.171382711453");
	m_PFT01.push_back("2455775.251748 2.17139547685");
	m_PFT01.push_back("2455775.251759 2.171408088466");
	m_PFT01.push_back("2455775.251771 2.171420859753");
	m_PFT01.push_back("2455775.251782 2.171433477264");
	m_PFT01.push_back("2455775.251794 2.171446254441");
	m_PFT01.push_back("2455775.251806 2.171458877848");
	m_PFT01.push_back("2455775.251817 2.171471660917");
	m_PFT01.push_back("2455775.251829 2.171484290218");
	m_PFT01.push_back(" 2455775.25184 2.171497079181");
	m_PFT01.push_back("2455775.251852 2.171509714381");
	m_PFT01.push_back("2455775.251863 2.171522509236");
	m_PFT01.push_back("2455775.251875 2.171535150337");
	m_PFT01.push_back("2455775.251887 2.171547951084");
	m_PFT01.push_back("2455775.251898 2.171560598083");
	m_PFT01.push_back(" 2455775.25191 2.171573404726");
	m_PFT01.push_back("2455775.251921 2.171586057626");
	m_PFT01.push_back("2455775.251933 2.171598870165");
	m_PFT01.push_back("2455775.251944 2.171611528965");
	m_PFT01.push_back("2455775.251956 2.171624347402");
	m_PFT01.push_back("2455775.251968 2.171637012105");
	m_PFT01.push_back("2455775.251979 2.171649836437");
	m_PFT01.push_back("2455775.251991 2.171662507045");
	m_PFT01.push_back("2455775.252002 2.171675337277");
	m_PFT01.push_back("2455775.252014 2.171688013787");
	m_PFT01.push_back("2455775.252025 2.17170084992");
	m_PFT01.push_back("2455775.252037 2.171713532337");
	m_PFT01.push_back("2455775.252049 2.171726374369");
	m_PFT01.push_back(" 2455775.25206 2.17173906269");
	m_PFT01.push_back("2455775.252072 2.171751910625");
	m_PFT01.push_back("2455775.252083 2.171764604854");
	m_PFT01.push_back("2455775.252095 2.171777458687");
	m_PFT01.push_back("2455775.252106 2.171790158824");
	m_PFT01.push_back("2455775.252118 2.171803018567");
	m_PFT01.push_back(" 2455775.25213 2.17181572461");
	m_PFT01.push_back("2455775.252141 2.171828590254");
	m_PFT01.push_back("2455775.252153 2.171841302208");
	m_PFT01.push_back("2455775.252164 2.17185417376");
	m_PFT01.push_back("2455775.252176 2.171866891623");
	m_PFT01.push_back("2455775.252188 2.171879769079");
	m_PFT01.push_back("2455775.252199 2.171892492856");
	m_PFT01.push_back("2455775.252211 2.171905376217");
	m_PFT01.push_back("2455775.252222 2.171918105907");
	m_PFT01.push_back("2455775.252234 2.171930995177");
	m_PFT01.push_back("2455775.252245 2.171943730779");
	m_PFT01.push_back("2455775.252257 2.171956625958");
	m_PFT01.push_back("2455775.252269 2.171969367474");
	m_PFT01.push_back(" 2455775.25228 2.171982268563");
	m_PFT01.push_back("2455775.252292 2.171995015996");
	m_PFT01.push_back("2455775.252303 2.172007922995");
	m_PFT01.push_back("2455775.252315 2.172020676344");
	m_PFT01.push_back("2455775.252326 2.172033589254");
	m_PFT01.push_back("2455775.252338 2.17204634852");
	m_PFT01.push_back(" 2455775.25235 2.172059267343");
	m_PFT01.push_back("2455775.252361 2.172072032527");
	m_PFT01.push_back("2455775.252373 2.172084957263");
	m_PFT01.push_back("2455775.252384 2.172097728366");
	m_PFT01.push_back("2455775.252396 2.172110659018");
	m_PFT01.push_back("2455775.252407 2.17212343604");
	m_PFT01.push_back("2455775.252419 2.172136372606");
	m_PFT01.push_back("2455775.252431 2.172149155549");
	m_PFT01.push_back("2455775.252442 2.172162098033");
	m_PFT01.push_back("2455775.252454 2.172174886897");
	m_PFT01.push_back("2455775.252465 2.172187835298");
	m_PFT01.push_back("2455775.252477 2.172200630085");
	m_PFT01.push_back("2455775.252488 2.172213584404");
	m_PFT01.push_back("  2455775.2525 2.172226385114");
	m_PFT01.push_back("2455775.252512 2.172239345352");
	m_PFT01.push_back("2455775.252523 2.172252151987");
	m_PFT01.push_back("2455775.252535 2.172265118144");
	m_PFT01.push_back("2455775.252546 2.172277930705");
	m_PFT01.push_back("2455775.252558 2.172290902784");
	m_PFT01.push_back("2455775.252569 2.172303721271");
	m_PFT01.push_back("2455775.252581 2.17231669927");
	m_PFT01.push_back("2455775.252593 2.172329523687");
	m_PFT01.push_back("2455775.252604 2.17234250761");
	m_PFT01.push_back("2455775.252616 2.172355337952");
	m_PFT01.push_back("2455775.252627 2.172368327799");
	m_PFT01.push_back("2455775.252639 2.172381164072");
	m_PFT01.push_back(" 2455775.25265 2.172394159841");
	m_PFT01.push_back("2455775.252662 2.172407002045");
	m_PFT01.push_back("2455775.252674 2.172420003743");
	m_PFT01.push_back("2455775.252685 2.172432851876");
	m_PFT01.push_back("2455775.252697 2.172445859498");
	m_PFT01.push_back("2455775.252708 2.172458713566");
	m_PFT01.push_back(" 2455775.25272 2.172471727115");
	m_PFT01.push_back("2455775.252731 2.172484587114");
	m_PFT01.push_back("2455775.252743 2.172497606593");
	m_PFT01.push_back("2455775.252755 2.172510472526");
	m_PFT01.push_back("2455775.252766 2.172523497935");
	m_PFT01.push_back("2455775.252778 2.172536369801");
	m_PFT01.push_back("2455775.252789 2.17254940114");
	m_PFT01.push_back("2455775.252801 2.172562278942");
	m_PFT01.push_back("2455775.252813 2.172575316213");
	m_PFT01.push_back("2455775.252824 2.172588199954");
	m_PFT01.push_back("2455775.252836 2.172601243155");
	m_PFT01.push_back("2455775.252847 2.172614132832");
	m_PFT01.push_back("2455775.252859 2.172627181966");
	m_PFT01.push_back(" 2455775.25287 2.172640077585");
	m_PFT01.push_back("2455775.252882 2.172653132652");
	m_PFT01.push_back("2455775.252894 2.172666034206");
	m_PFT01.push_back("2455775.252905 2.172679095212");
	m_PFT01.push_back("2455775.252917 2.172692002708");
	m_PFT01.push_back("2455775.252928 2.172705069647");
	m_PFT01.push_back(" 2455775.25294 2.172717983085");
	m_PFT01.push_back("2455775.252951 2.17273105596");
	m_PFT01.push_back("2455775.252963 2.172743975342");
	m_PFT01.push_back("2455775.252975 2.172757054155");
	m_PFT01.push_back("2455775.252986 2.172769979479");
	m_PFT01.push_back("2455775.252998 2.172783064232");
	m_PFT01.push_back("2455775.253009 2.172795995497");
	m_PFT01.push_back("2455775.253021 2.172809086191");
	m_PFT01.push_back("2455775.253032 2.172822023404");
	m_PFT01.push_back("2455775.253044 2.172835120037");
	m_PFT01.push_back("2455775.253056 2.172848063196");
	m_PFT01.push_back("2455775.253067 2.172861165769");
	m_PFT01.push_back("2455775.253079 2.172874114878");
	m_PFT01.push_back(" 2455775.25309 2.172887223393");
	m_PFT01.push_back("2455775.253102 2.172900178448");
	m_PFT01.push_back("2455775.253113 2.172913292907");
	m_PFT01.push_back("2455775.253125 2.172926253911");
	m_PFT01.push_back("2455775.253137 2.172939374315");
	m_PFT01.push_back("2455775.253148 2.172952341269");
	m_PFT01.push_back(" 2455775.25316 2.172965467617");
	m_PFT01.push_back("2455775.253171 2.172978440521");
	m_PFT01.push_back("2455775.253183 2.172991572818");
	m_PFT01.push_back("2455775.253194 2.173004551674");
	m_PFT01.push_back("2455775.253206 2.173017689918");
	m_PFT01.push_back("2455775.253218 2.173030674724");
	m_PFT01.push_back("2455775.253229 2.173043818915");
	m_PFT01.push_back("2455775.253241 2.173056809678");
	m_PFT01.push_back("2455775.253252 2.173069959818");
	m_PFT01.push_back("2455775.253264 2.173082956535");
	m_PFT01.push_back("2455775.253275 2.173096112624");
	m_PFT01.push_back("2455775.253287 2.1731091153");
	m_PFT01.push_back("2455775.253299 2.173122277339");
	m_PFT01.push_back(" 2455775.25331 2.173135285969");
	m_PFT01.push_back("2455775.253322 2.173148453961");
	m_PFT01.push_back("2455775.253333 2.173161468548");
	m_PFT01.push_back("2455775.253345 2.173174642493");
	m_PFT01.push_back("2455775.253356 2.173187663039");
	m_PFT01.push_back("2455775.253368 2.173200842938");
	m_PFT01.push_back(" 2455775.25338 2.173213869443");
	m_PFT01.push_back("2455775.253391 2.173227055296");
	m_PFT01.push_back("2455775.253403 2.17324008776");
	m_PFT01.push_back("2455775.253414 2.173253279571");
	m_PFT01.push_back("2455775.253426 2.173266317998");
	m_PFT01.push_back("2455775.253438 2.173279515764");
	m_PFT01.push_back("2455775.253449 2.173292560152");
	m_PFT01.push_back("2455775.253461 2.173305763876");
	m_PFT01.push_back("2455775.253472 2.173318814228");
	m_PFT01.push_back("2455775.253484 2.17333202391");
	m_PFT01.push_back("2455775.253495 2.173345080224");
	m_PFT01.push_back("2455775.253507 2.173358295868");
	m_PFT01.push_back("2455775.253519 2.173371358147");
	m_PFT01.push_back(" 2455775.25353 2.173384579751");
	m_PFT01.push_back("2455775.253542 2.173397647998");
	m_PFT01.push_back("2455775.253553 2.173410875562");
	m_PFT01.push_back("2455775.253565 2.173423949775");
	m_PFT01.push_back("2455775.253576 2.173437183302");
	m_PFT01.push_back("2455775.253588 2.173450263485");
	m_PFT01.push_back("  2455775.2536 2.173463502972");
	m_PFT01.push_back("2455775.253611 2.173476589124");
	m_PFT01.push_back("2455775.253623 2.173489834579");
	m_PFT01.push_back("2455775.253634 2.173502926698");
	m_PFT01.push_back("2455775.253646 2.173516178117");
	m_PFT01.push_back("2455775.253657 2.173529276209");
	m_PFT01.push_back("2455775.253669 2.173542533595");
	m_PFT01.push_back("2455775.253681 2.173555637656");
	m_PFT01.push_back("2455775.253692 2.173568901008");
	m_PFT01.push_back("2455775.253704 2.173582011045");
	m_PFT01.push_back("2455775.253715 2.173595280364");
	m_PFT01.push_back("2455775.253727 2.173608396376");
	m_PFT01.push_back("2455775.253738 2.173621671665");
	m_PFT01.push_back(" 2455775.25375 2.173634793649");
	m_PFT01.push_back("2455775.253762 2.173648074908");
	m_PFT01.push_back("2455775.253773 2.173661202867");
	m_PFT01.push_back("2455775.253785 2.173674490098");
	m_PFT01.push_back("2455775.253796 2.173687624035");
	m_PFT01.push_back("2455775.253808 2.173700917234");
	m_PFT01.push_back("2455775.253819 2.17371405715");
	m_PFT01.push_back("2455775.253831 2.173727356323");
	m_PFT01.push_back("2455775.253843 2.173740502216");
	m_PFT01.push_back("2455775.253854 2.173753807365");
	m_PFT01.push_back("2455775.253866 2.173766959236");
	m_PFT01.push_back("2455775.253877 2.17378027036");
	m_PFT01.push_back("2455775.253889 2.173793428214");
	m_PFT01.push_back("  2455775.2539 2.173806745312");
	m_PFT01.push_back("2455775.253912 2.173819909145");
	m_PFT01.push_back("2455775.253924 2.17383323222");
	m_PFT01.push_back("2455775.253935 2.173846402038");
	m_PFT01.push_back("2455775.253947 2.17385973109");
	m_PFT01.push_back("2455775.253958 2.173872906889");
	m_PFT01.push_back(" 2455775.25397 2.17388624192");
	m_PFT01.push_back("2455775.253981 2.173899423705");
	m_PFT01.push_back("2455775.253993 2.173912764715");
	m_PFT01.push_back("2455775.254005 2.173925952485");
	m_PFT01.push_back("2455775.254016 2.173939299475");
	m_PFT01.push_back("2455775.254028 2.17395249323");
	m_PFT01.push_back("2455775.254039 2.173965846204");
	m_PFT01.push_back("2455775.254051 2.173979045948");
	m_PFT01.push_back("2455775.254063 2.173992404902");
	m_PFT01.push_back("2455775.254074 2.174005610632");
	m_PFT01.push_back("2455775.254086 2.174018975571");
	m_PFT01.push_back("2455775.254097 2.174032187291");
	m_PFT01.push_back("2455775.254109 2.174045558213");
	m_PFT01.push_back(" 2455775.25412 2.174058775924");
	m_PFT01.push_back("2455775.254132 2.174072152831");
	m_PFT01.push_back("2455775.254144 2.174085376533");
	m_PFT01.push_back("2455775.254155 2.174098759427");
	m_PFT01.push_back("2455775.254167 2.174111989121");
	m_PFT01.push_back("2455775.254178 2.174125378002");
	m_PFT01.push_back(" 2455775.25419 2.174138613687");
	m_PFT01.push_back("2455775.254201 2.174152008557");
	m_PFT01.push_back("2455775.254213 2.174165250238");
	m_PFT01.push_back("2455775.254225 2.174178651096");
	m_PFT01.push_back("2455775.254236 2.174191898771");
	m_PFT01.push_back("2455775.254248 2.174205305619");
	m_PFT01.push_back("2455775.254259 2.174218559291");
	m_PFT01.push_back("2455775.254271 2.17423197213");
	m_PFT01.push_back("2455775.254282 2.174245231797");
	m_PFT01.push_back("2455775.254294 2.174258650628");
	m_PFT01.push_back("2455775.254306 2.174271916294");
	m_PFT01.push_back("2455775.254317 2.174285341121");
	m_PFT01.push_back("2455775.254329 2.174298612783");
	m_PFT01.push_back(" 2455775.25434 2.174312043603");
	m_PFT01.push_back("2455775.254352 2.174325321267");
	m_PFT01.push_back("2455775.254363 2.17433875808");
	m_PFT01.push_back("2455775.254375 2.174352041744");
	m_PFT01.push_back("2455775.254387 2.174365484555");
	m_PFT01.push_back("2455775.254398 2.174378774221");
	m_PFT01.push_back(" 2455775.25441 2.174392223027");
	m_PFT01.push_back("2455775.254421 2.174405518698");
	m_PFT01.push_back("2455775.254433 2.1744189735");
	m_PFT01.push_back("2455775.254444 2.174432275171");
	m_PFT01.push_back("2455775.254456 2.174445735977");
	m_PFT01.push_back("2455775.254468 2.174459043653");
	m_PFT01.push_back("2455775.254479 2.174472510456");
	m_PFT01.push_back("2455775.254491 2.17448582414");
	m_PFT01.push_back("2455775.254502 2.174499296944");
	m_PFT01.push_back("2455775.254514 2.174512616632");
	m_PFT01.push_back("2455775.254525 2.174526095437");
	m_PFT01.push_back("2455775.254537 2.174539421133");
	m_PFT01.push_back("2455775.254549 2.174552905942");
	m_PFT01.push_back(" 2455775.25456 2.174566237647");
	m_PFT01.push_back("2455775.254572 2.174579728457");
	m_PFT01.push_back("2455775.254583 2.174593066175");
	m_PFT01.push_back("2455775.254595 2.17460656299");
	m_PFT01.push_back("2455775.254606 2.174619906715");
	m_PFT01.push_back("2455775.254618 2.174633409537");
	m_PFT01.push_back(" 2455775.25463 2.174646759273");
	m_PFT01.push_back("2455775.254641 2.1746602681");
	m_PFT01.push_back("2455775.254653 2.174673623851");
	m_PFT01.push_back("2455775.254664 2.174687138686");
	m_PFT01.push_back("2455775.254676 2.174700500448");
	m_PFT01.push_back("2455775.254688 2.174714021294");
	m_PFT01.push_back("2455775.254699 2.174727389071");
	m_PFT01.push_back("2455775.254711 2.174740915926");
	m_PFT01.push_back("2455775.254722 2.174754289717");
	m_PFT01.push_back("2455775.254734 2.174767822581");
	m_PFT01.push_back("2455775.254745 2.174781202389");
	m_PFT01.push_back("2455775.254757 2.174794741265");
	m_PFT01.push_back("2455775.254769 2.174808127092");
	m_PFT01.push_back(" 2455775.25478 2.17482167198");
	m_PFT01.push_back("2455775.254792 2.174835063824");
	m_PFT01.push_back("2455775.254803 2.174848614725");
	m_PFT01.push_back("2455775.254815 2.174862012588");
	m_PFT01.push_back("2455775.254826 2.174875569505");
	m_PFT01.push_back("2455775.254838 2.174888973386");
	m_PFT01.push_back(" 2455775.25485 2.174902536318");
	m_PFT01.push_back("2455775.254861 2.174915946223");
	m_PFT01.push_back("2455775.254873 2.174929515172");
	m_PFT01.push_back("2455775.254884 2.174942931097");
	m_PFT01.push_back("2455775.254896 2.174956506063");
	m_PFT01.push_back("2455775.254907 2.174969928012");
	m_PFT01.push_back("2455775.254919 2.174983508996");
	m_PFT01.push_back("2455775.254931 2.174996936971");
	m_PFT01.push_back("2455775.254942 2.175010523972");
	m_PFT01.push_back("2455775.254954 2.175023957971");
	m_PFT01.push_back("2455775.254965 2.175037550995");
	m_PFT01.push_back("2455775.254977 2.175050991019");
	m_PFT01.push_back("2455775.254988 2.175064590065");
	m_PFT01.push_back("   2455775.255 2.175078036116");
	m_PFT01.push_back("2455775.255012 2.175091641183");
	m_PFT01.push_back("2455775.255023 2.175105093263");
	m_PFT01.push_back("2455775.255035 2.175118704353");
	m_PFT01.push_back("2455775.255046 2.175132162461");
	m_PFT01.push_back("2455775.255058 2.175145779576");
	m_PFT01.push_back("2455775.255069 2.175159243714");
	m_PFT01.push_back("2455775.255081 2.175172866853");
	m_PFT01.push_back("2455775.255093 2.175186337024");
	m_PFT01.push_back("2455775.255104 2.175199966189");
	m_PFT01.push_back("2455775.255116 2.175213442391");
	m_PFT01.push_back("2455775.255127 2.175227077584");
	m_PFT01.push_back("2455775.255139 2.17524055982");
	m_PFT01.push_back(" 2455775.25515 2.17525420104");
	m_PFT01.push_back("2455775.255162 2.175267689309");
	m_PFT01.push_back("2455775.255174 2.175281336557");
	m_PFT01.push_back("2455775.255185 2.175294830862");
	m_PFT01.push_back("2455775.255197 2.175308484142");
	m_PFT01.push_back("2455775.255208 2.175321984482");
	m_PFT01.push_back(" 2455775.25522 2.175335643792");
	m_PFT01.push_back("2455775.255231 2.175349150168");
	m_PFT01.push_back("2455775.255243 2.175362815512");
	m_PFT01.push_back("2455775.255255 2.175376327926");
	m_PFT01.push_back("2455775.255266 2.175389999303");
	m_PFT01.push_back("2455775.255278 2.175403517756");
	m_PFT01.push_back("2455775.255289 2.175417195166");
	m_PFT01.push_back("2455775.255301 2.175430719659");
	m_PFT01.push_back("2455775.255313 2.175444403105");
	m_PFT01.push_back("2455775.255324 2.17545793364");
	m_PFT01.push_back("2455775.255336 2.175471623121");
	m_PFT01.push_back("2455775.255347 2.175485159695");
	m_PFT01.push_back("2455775.255359 2.175498855215");
	m_PFT01.push_back(" 2455775.25537 2.175512397833");
	m_PFT01.push_back("2455775.255382 2.175526099389");
	m_PFT01.push_back("2455775.255394 2.175539648054");
	m_PFT01.push_back("2455775.255405 2.175553355648");
	m_PFT01.push_back("2455775.255417 2.175566910355");
	m_PFT01.push_back("2455775.255428 2.17558062399");
	m_PFT01.push_back(" 2455775.25544 2.175594184744");
	m_PFT01.push_back("2455775.255451 2.175607904421");
	m_PFT01.push_back("2455775.255463 2.17562147122");
	m_PFT01.push_back("2455775.255475 2.175635196936");
	m_PFT01.push_back("2455775.255486 2.175648769785");
	m_PFT01.push_back("2455775.255498 2.175662501547");
	m_PFT01.push_back("2455775.255509 2.175676080443");
	m_PFT01.push_back("2455775.255521 2.175689818247");
	m_PFT01.push_back("2455775.255532 2.175703403194");
	m_PFT01.push_back("2455775.255544 2.175717147043");
	m_PFT01.push_back("2455775.255556 2.175730738041");
	m_PFT01.push_back("2455775.255567 2.175744487936");
	m_PFT01.push_back("2455775.255579 2.175758084985");
	m_PFT01.push_back(" 2455775.25559 2.175771840928");
	m_PFT01.push_back("2455775.255602 2.175785444032");
	m_PFT01.push_back("2455775.255613 2.175799206021");
	m_PFT01.push_back("2455775.255625 2.175812815177");
	m_PFT01.push_back("2455775.255637 2.175826583215");
	m_PFT01.push_back("2455775.255648 2.175840198428");
	m_PFT01.push_back(" 2455775.25566 2.175853972515");
	m_PFT01.push_back("2455775.255671 2.175867593782");
	m_PFT01.push_back("2455775.255683 2.17588137392");
	m_PFT01.push_back("2455775.255694 2.175895001247");
	m_PFT01.push_back("2455775.255706 2.175908787437");
	m_PFT01.push_back("2455775.255718 2.175922420817");
	m_PFT01.push_back("2455775.255729 2.175936213062");
	m_PFT01.push_back("2455775.255741 2.175949852504");
	m_PFT01.push_back("2455775.255752 2.175963650801");
	m_PFT01.push_back("2455775.255764 2.175977296302");
	m_PFT01.push_back("2455775.255775 2.175991100654");
	m_PFT01.push_back("2455775.255787 2.176004752213");
	m_PFT01.push_back("2455775.255799 2.176018562623");
	m_PFT01.push_back(" 2455775.25581 2.176032220245");
	m_PFT01.push_back("2455775.255822 2.176046036711");
	m_PFT01.push_back("2455775.255833 2.176059700396");
	m_PFT01.push_back("2455775.255845 2.176073522919");
	m_PFT01.push_back("2455775.255856 2.176087192669");
	m_PFT01.push_back("2455775.255868 2.17610102125");
	m_PFT01.push_back(" 2455775.25588 2.176114697065");
	m_PFT01.push_back("2455775.255891 2.176128531706");
	m_PFT01.push_back("2455775.255903 2.176142213587");
	m_PFT01.push_back("2455775.255914 2.176156054289");
	m_PFT01.push_back("2455775.255926 2.176169742236");
	m_PFT01.push_back("2455775.255938 2.176183589");
	m_PFT01.push_back("2455775.255949 2.176197283014");
	m_PFT01.push_back("2455775.255961 2.176211135842");
	m_PFT01.push_back("2455775.255972 2.176224835924");
	m_PFT01.push_back("2455775.255984 2.176238694815");
	m_PFT01.push_back("2455775.255995 2.17625240097");
	m_PFT01.push_back("2455775.256007 2.176266265925");
	m_PFT01.push_back("2455775.256019 2.17627997815");
	m_PFT01.push_back(" 2455775.25603 2.17629384917");
	m_PFT01.push_back("2455775.256042 2.176307567466");
	m_PFT01.push_back("2455775.256053 2.176321444554");
	m_PFT01.push_back("2455775.256065 2.176335168924");
	m_PFT01.push_back("2455775.256076 2.17634905208");
	m_PFT01.push_back("2455775.256088 2.176362782522");
	m_PFT01.push_back("  2455775.2561 2.176376671746");
	m_PFT01.push_back("2455775.256111 2.176390408264");
	m_PFT01.push_back("2455775.256123 2.176404303559");
	m_PFT01.push_back("2455775.256134 2.176418046153");
	m_PFT01.push_back("2455775.256146 2.176431947518");
	m_PFT01.push_back("2455775.256157 2.176445696188");
	m_PFT01.push_back("2455775.256169 2.176459603626");
	m_PFT01.push_back("2455775.256181 2.176473358373");
	m_PFT01.push_back("2455775.256192 2.176487271883");
	m_PFT01.push_back("2455775.256204 2.17650103271");
	m_PFT01.push_back("2455775.256215 2.176514952294");
	m_PFT01.push_back("2455775.256227 2.176528719201");
	m_PFT01.push_back("2455775.256238 2.17654264486");
	m_PFT01.push_back(" 2455775.25625 2.176556417849");
	m_PFT01.push_back("2455775.256262 2.176570349583");
	m_PFT01.push_back("2455775.256273 2.176584128652");
	m_PFT01.push_back("2455775.256285 2.176598066463");
	m_PFT01.push_back("2455775.256296 2.176611851615");
	m_PFT01.push_back("2455775.256308 2.176625795505");
	m_PFT01.push_back("2455775.256319 2.176639586741");
	m_PFT01.push_back("2455775.256331 2.176653536711");
	m_PFT01.push_back("2455775.256343 2.17666733403");
	m_PFT01.push_back("2455775.256354 2.17668129008");
	m_PFT01.push_back("2455775.256366 2.176695093486");
	m_PFT01.push_back("2455775.256377 2.176709055615");
	m_PFT01.push_back("2455775.256389 2.176722865109");
	m_PFT01.push_back("  2455775.2564 2.17673683332");
	m_PFT01.push_back("2455775.256412 2.176750648901");
	m_PFT01.push_back("2455775.256424 2.176764623196");
	m_PFT01.push_back("2455775.256435 2.176778444867");
	m_PFT01.push_back("2455775.256447 2.176792425246");
	m_PFT01.push_back("2455775.256458 2.176806253006");
	m_PFT01.push_back(" 2455775.25647 2.176820239468");
	m_PFT01.push_back("2455775.256481 2.176834073319");
	m_PFT01.push_back("2455775.256493 2.176848065869");
	m_PFT01.push_back("2455775.256505 2.176861905812");
	m_PFT01.push_back("2455775.256516 2.176875904449");
	m_PFT01.push_back("2455775.256528 2.176889750484");
	m_PFT01.push_back("2455775.256539 2.17690375521");
	m_PFT01.push_back("2455775.256551 2.176917607339");
	m_PFT01.push_back("2455775.256563 2.176931618151");
	m_PFT01.push_back("2455775.256574 2.176945476379");
	m_PFT01.push_back("2455775.256586 2.176959493282");
	m_PFT01.push_back("2455775.256597 2.176973357603");
	m_PFT01.push_back("2455775.256609 2.176987380597");
	m_PFT01.push_back(" 2455775.25662 2.177001251015");
	m_PFT01.push_back("2455775.256632 2.177015280101");
	m_PFT01.push_back("2455775.256644 2.177029156617");
	m_PFT01.push_back("2455775.256655 2.177043191798");
	m_PFT01.push_back("2455775.256667 2.177057074414");
	m_PFT01.push_back("2455775.256678 2.177071115686");
	m_PFT01.push_back(" 2455775.25669 2.177085004401");
	m_PFT01.push_back("2455775.256701 2.177099051769");
	m_PFT01.push_back("2455775.256713 2.177112946587");
	m_PFT01.push_back("2455775.256725 2.177127000049");
	m_PFT01.push_back("2455775.256736 2.177140900971");
	m_PFT01.push_back("2455775.256748 2.177154960532");
	m_PFT01.push_back("2455775.256759 2.177168867557");
	m_PFT01.push_back("2455775.256771 2.177182933214");
	m_PFT01.push_back("2455775.256782 2.17719684634");
	m_PFT01.push_back("2455775.256794 2.177210918101");
	m_PFT01.push_back("2455775.256806 2.177224837331");
	m_PFT01.push_back("2455775.256817 2.17723891519");
	m_PFT01.push_back("2455775.256829 2.177252840529");
	m_PFT01.push_back(" 2455775.25684 2.177266924488");
	m_PFT01.push_back("2455775.256852 2.177280855935");
	m_PFT01.push_back("2455775.256863 2.177294945996");
	m_PFT01.push_back("2455775.256875 2.177308883551");
	m_PFT01.push_back("2455775.256887 2.177322979715");
	m_PFT01.push_back("2455775.256898 2.177336923378");
	m_PFT01.push_back(" 2455775.25691 2.177351025646");
	m_PFT01.push_back("2455775.256921 2.177364975422");
	m_PFT01.push_back("2455775.256933 2.177379083796");
#endif /* ENABLE_LBCSIIFSERVICEWORKERTEST */

#if 0
	// derotateWithLBT_025.txt - 900 entries - SegmentUnderrunException
	m_poly.push_back("GetRotatorPolynomials result status: OK 0 0");
	m_poly.push_back("56750.333342199083 4.0050696446449399 -0.000435060111354987811 0.000000000002025895491995079891");
	m_poly.push_back("56750.333347986110 4.0048521316248049 -0.000434984351332128884 0.000000000002025542712011876016");
	m_poly.push_back("56750.333353773152 4.0046346565073936 -0.000434908568998107004 0.000000000002025189828133413956");
	m_poly.push_back("56750.333359560187 4.0044172192769052 -0.000434832730655250292 0.000000000002024836683442682114");
	m_poly.push_back("56750.333365347222 4.0041998199738789 -0.000434756887658313291 0.000000000002024483517078890876");
	m_poly.push_back("56750.333371134257 4.0039824628493657 -0.000434681093244999525 0.000000000002024130576948897005");
	m_poly.push_back("56750.333376921299 4.0037651394008744 -0.000434605169672079193 0.000000000002023777035372325859");
	m_poly.push_back("56750.333382708333 4.0035478539119866 -0.000434529177620231308 0.000000000002023423174915088871");
	m_poly.push_back("56750.333388495368 4.0033306063937859 -0.000434453196315400577 0.000000000002023069364501740143");
	m_poly.push_back("56750.333394282410 4.0031134002573063 -0.000434377267395617318 0.000000000002022715798024074015");
	m_poly.push_back("56750.333400069438 4.0028962287286207 -0.000434301282022907416 0.000000000002022361968666563814");
	m_poly.push_back("56750.333405856480 4.0026790951913984 -0.000434225224440900324 0.000000000002022007803057499990");
	m_poly.push_back("56750.333411643522 4.0024619996783786 -0.000434149082232794298 0.000000000002021653243376643900");
	m_poly.push_back("56750.333417430556 4.0022449456078846 -0.000434073136384238473 0.000000000002021299598064974183");
	m_poly.push_back("56750.333423217591 4.0020279261837013 -0.000433997044731522399 0.000000000002020945273799373075");
	m_poly.push_back("56750.333429004633 4.0018109447994954 -0.000433920880105671521 0.000000000002020590609725321112");
	m_poly.push_back("56750.333434791668 4.0015940015044533 -0.000433844675778029688 0.000000000002020235760774585919");
	m_poly.push_back("56750.333440578703 4.0013770997101563 -0.000433768513781201406 0.000000000002019881108940856201");
	m_poly.push_back("56750.333446365737 4.0011602325931017 -0.000433692282452789301 0.000000000002019526134256360034");
	m_poly.push_back("56750.333452152780 4.0009434036182157 -0.000433616085585887014 0.000000000002019171320044960056");
	m_poly.push_back("56750.333457939807 4.0007266127349652 -0.000433539755988881506 0.000000000002018815887760038971");
	m_poly.push_back("56750.333463726849 4.0005098634187819 -0.000433463535798287174 0.000000000002018460964937204083");
	m_poly.push_back("56750.333469513891 4.0002931488521307 -0.000433387158061750404 0.000000000002018105308483493967");
	m_poly.push_back("56750.333475300933 4.0000764724419113 -0.000433310823295518521 0.000000000002017749852125045042");
	m_poly.push_back("56750.333481087961 3.9998598342168088 -0.000433234467035252275 0.000000000002017394295676382886");
	m_poly.push_back("56750.333486875003 3.9996432375774669 -0.000433158020544451017 0.000000000002017038319057844974");
	m_poly.push_back("56750.333492662037 3.9994266757447710 -0.000433081571522327385 0.000000000002016682330651493002");
	m_poly.push_back("56750.333498449072 3.9992101521501939 -0.000433005210590217497 0.000000000002016326752445418010");
	m_poly.push_back("56750.333504236107 3.9989936667675861 -0.000432928676409085114 0.000000000002015970367484897887");
	m_poly.push_back("56750.333510023149 3.9987772230436631 -0.000432852209641060596 0.000000000002015614296439896018");
	m_poly.push_back("56750.333515810184 3.9985608141522282 -0.000432775662794435210 0.000000000002015257852499446187");
	m_poly.push_back("56750.333521597233 3.9983444435507720 -0.000432699092917218609 0.000000000002014901301313630840");
	m_poly.push_back("56750.333527384260 3.9981281112218370 -0.000432622533671555318 0.000000000002014544799634027066");
	m_poly.push_back("56750.333533171302 3.9979118205981989 -0.000432545897882260675 0.000000000002014187941519146948");
	m_poly.push_back("56750.333538958330 3.9976955648658890 -0.000432469256355302216 0.000000000002013831056685768800");
	m_poly.push_back("56750.333544745372 3.9974793474806289 -0.000432392642424196986 0.000000000002013474300354778856");
	m_poly.push_back("56750.333550532407 3.9972631684243818 -0.000432315926343137679 0.000000000002013117068350178871");
	m_poly.push_back("56750.333556319441 3.9970470311061952 -0.000432239219721229878 0.000000000002012759880391963017");
	m_poly.push_back("56750.333562106483 3.9968309287545360 -0.000432162449034665123 0.000000000002012402394109026028");
	m_poly.push_back("56750.333567893518 3.9966148647614221 -0.000432085774209717082 0.000000000002012045354215708863");
	m_poly.push_back("56750.333573680553 3.9963988391595811 -0.000432008979417859208 0.000000000002011687755682115873");
	m_poly.push_back("56750.333579467588 3.9961828553538208 -0.000431932133084728023 0.000000000002011329917139549047");
	m_poly.push_back("56750.333585254630 3.9959669065767369 -0.000431855281582370517 0.000000000002010972054525473057");
	m_poly.push_back("56750.333591041657 3.9957509962178421 -0.000431778454895852715 0.000000000002010614307468335982");
	m_poly.push_back("56750.333596828707 3.9955351242734309 -0.000431701509823391016 0.000000000002010256009132868096");
	m_poly.push_back("56750.333602615741 3.9953192942035600 -0.000431624656065048126 0.000000000002009898136010676134");
	m_poly.push_back("56750.333608402783 3.9951034991817620 -0.000431547740391451389 0.000000000002009539974572700148");
	m_poly.push_back("56750.333614189818 3.9948877426390501 -0.000431470776800369513 0.000000000002009181590000723133");
	m_poly.push_back("56750.333619976853 3.9946720245812402 -0.000431393798874009826 0.000000000002008823138674086056");
	m_poly.push_back("56750.333625763888 3.9944563484320450 -0.000431316771413671224 0.000000000002008464456685647164");
	m_poly.push_back("56750.333631550922 3.9942407073813602 -0.000431239785085098205 0.000000000002008105966231510804");
	m_poly.push_back("56750.333637337957 3.9940251048521489 -0.000431162664267819315 0.000000000002007746849514713977");
	m_poly.push_back("56750.333643124999 3.9938095408680852 -0.000431085578453841898 0.000000000002007387895793919055");
	m_poly.push_back("56750.333648912027 3.9935940188405250 -0.000431008462885725983 0.000000000002007028803518415053");
	m_poly.push_back("56750.333654699083 3.9933785319643298 -0.000430931317741108284 0.000000000002006669573516149990");
	m_poly.push_back("56750.333660486111 3.9931630836493581 -0.000430854155721009104 0.000000000002006310264930477077");
	m_poly.push_back("56750.333666273153 3.9929476739503760 -0.000430776992848253980 0.000000000002005950952373491868");
	m_poly.push_back("56750.333672060187 3.9927323062278508 -0.000430699717797955884 0.000000000002005591117447868051");
	m_poly.push_back("56750.333677847222 3.9925169737108162 -0.000430622511963777712 0.000000000002005231604834518123");
	m_poly.push_back("56750.333683634257 3.9923016798377482 -0.000430545276348815419 0.000000000002004871953542762091");
	m_poly.push_back("56750.333689421299 3.9920864245952989 -0.000430467956312036474 0.000000000002004511909130461803");
	m_poly.push_back("56750.333695208326 3.9918712114109431 -0.000430390676874113295 0.000000000002004152053770111992");
	m_poly.push_back("56750.333700995368 3.9916560334585691 -0.000430313332287949582 0.000000000002003791895039181149");
	m_poly.push_back("56750.333706782410 3.9914408942050170 -0.000430236018255289281 0.000000000002003431878583261947");
	m_poly.push_back("56750.333712569452 3.9912257936228150 -0.000430158595553410011 0.000000000002003071356096032165");
	m_poly.push_back("56750.333718356480 3.9910107351493211 -0.000430081148160036503 0.000000000002002710718628838908");
	m_poly.push_back("56750.333724143522 3.9907957119800441 -0.000430003789487286726 0.000000000002002350494298824974");
	m_poly.push_back("56750.333729930557 3.9905807275336809 -0.000429926318299484389 0.000000000002001989746028942857");
	m_poly.push_back("56750.333735717591 3.9903657818197940 -0.000429848835512009115 0.000000000002001628943743047947");
	m_poly.push_back("56750.333741504634 3.9901508782463271 -0.000429771310535829814 0.000000000002001267944999461806");
	m_poly.push_back("56750.333747291668 3.9899360100424350 -0.000429693781243080082 0.000000000002000906926154852021");
	m_poly.push_back("56750.333753078703 3.9897211806090569 -0.000429616221245836011 0.000000000002000545764330471098");
	m_poly.push_back("56750.333758865738 3.9895063899495300 -0.000429538660307117394 0.000000000002000184598121179898");
	m_poly.push_back("56750.333764652780 3.9892916414885820 -0.000429461017673304782 0.000000000001999823051488232822");
	m_poly.push_back("56750.333770439807 3.9890769284298009 -0.000429383435489243278 0.000000000001999461786345947201");
	m_poly.push_back("56750.333776226849 3.9888622542017660 -0.000429305801675090187 0.000000000001999100280781483967");
	m_poly.push_back("56750.333782013891 3.9886476187947140 -0.000429228137129797186 0.000000000001998738632113170079");
	m_poly.push_back("56750.333787800933 3.9884330256347091 -0.000429150359509896078 0.000000000001998376456898949027");
	m_poly.push_back("56750.333793587961 3.9882184679206629 -0.000429072733911437876 0.000000000001998014989589309126");
	m_poly.push_back("56750.333799375003 3.9880039490639492 -0.000428994928091719016 0.000000000001997652683058196834");
	m_poly.push_back("56750.333805162038 3.9877894691197531 -0.000428917134386819611 0.000000000001997290432940278061");
	m_poly.push_back("56750.333810949072 3.9875750314446292 -0.000428839397259192317 0.000000000001996928446279572197");
	m_poly.push_back("56750.333816736107 3.9873606292629900 -0.000428761556862500010 0.000000000001996565978734388808");
	m_poly.push_back("56750.333822523149 3.9871462660114991 -0.000428683658156578081 0.000000000001996203239664865925");
	m_poly.push_back("56750.333828310177 3.9869319416995359 -0.000428605800768940611 0.000000000001995840692997765972");
	m_poly.push_back("56750.333834097219 3.9867176597082632 -0.000428527963773990084 0.000000000001995478241290293924");
	m_poly.push_back("56750.333839884261 3.9865034132517931 -0.000428449988480027611 0.000000000001995115145577420807");
	m_poly.push_back("56750.333845671303 3.9862892057983839 -0.000428372052799037709 0.000000000001994752234325993926");
	m_poly.push_back("56750.333851458330 3.9860750372735438 -0.000428294138727545689 0.000000000001994389423700802199");
	m_poly.push_back("56750.333857245372 3.9858609111655960 -0.000428216163531283602 0.000000000001994026328439971977");
	m_poly.push_back("56750.333863032407 3.9856468206459978 -0.000428138098886087523 0.000000000001993662816649666065");
	m_poly.push_back("56750.333868819442 3.9854327691243498 -0.000428060104638294792 0.000000000001993299632671980814");
	m_poly.push_back("56750.333874606476 3.9852187566247061 -0.000427982052969455790 0.000000000001992936181305880135");
	m_poly.push_back("56750.333880393518 3.9850047865774418 -0.000427903982675414388 0.000000000001992572643208195055");
	m_poly.push_back("56750.333886180553 3.9847908521548021 -0.000427825875521768587 0.000000000001992208933469160833");
	m_poly.push_back("56750.333891967603 3.9845769568045588 -0.000427747793157316975 0.000000000001991845339162973894");
	m_poly.push_back("56750.333897754630 3.9843631005092570 -0.000427669681580518574 0.000000000001991481608825362850");
	m_poly.push_back("56750.333903541657 3.9841492866913342 -0.000427591453491843205 0.000000000001991117335935812144");
	m_poly.push_back("56750.333909328707 3.9839355085611481 -0.000427513201741966311 0.000000000001990752952864769063");
	m_poly.push_back("56750.333915115742 3.9837217695405132 -0.000427435046155688527 0.000000000001990389017589545979");
	m_poly.push_back("56750.333920902784 3.9835080696440581 -0.000427356777459275607 0.000000000001990024555603375831");
	m_poly.push_back("56750.333926689818 3.9832944122562850 -0.000427278561840692688 0.000000000001989660340778874093");
	m_poly.push_back("56750.333932476853 3.9830807906071031 -0.000427200233929103277 0.000000000001989295603048878989");
	m_poly.push_back("56750.333938263888 3.9828672081145009 -0.000427121927724712080 0.000000000001988930966400067971");
	m_poly.push_back("56750.333944050923 3.9826536647728341 -0.000427043580903701495 0.000000000001988566140614547930");
	m_poly.push_back("56750.333949837957 3.9824401639947631 -0.000426965276830950106 0.000000000001988201513889869835");
	m_poly.push_back("56750.333955624999 3.9822266990170321 -0.000426886890903231577 0.000000000001987836505997877068");
	m_poly.push_back("56750.333961412027 3.9820132732201512 -0.000426808494894633178 0.000000000001987471451162291998");
	m_poly.push_back("56750.333967199083 3.9817998866365492 -0.000426730027449457206 0.000000000001987106063673384124");
	m_poly.push_back("56750.333972986111 3.9815865426663271 -0.000426651568859467891 0.000000000001986740717418443066");
	m_poly.push_back("56750.333978773153 3.9813732345247881 -0.000426573105695335103 0.000000000001986375349863020027");
	m_poly.push_back("56750.333984560188 3.9811599656147609 -0.000426494640443971591 0.000000000001986009972587332986");
	m_poly.push_back("56750.333990347222 3.9809467359705830 -0.000426416162011968882 0.000000000001985644533933674809");
	m_poly.push_back("56750.333996134257 3.9807335489744098 -0.000426337595934044185 0.000000000001985278687145634945");
	m_poly.push_back("56750.334001921299 3.9805203978625951 -0.000426259025281976016 0.000000000001984912819057155105");
	m_poly.push_back("56750.334007708327 3.9803072860349942 -0.000426180513001310122 0.000000000001984547222780859880");
	m_poly.push_back("56750.334013495369 3.9800942134992350 -0.000426101856284286117 0.000000000001984180953919520890");
	m_poly.push_back("56750.334019282411 3.9798811836689509 -0.000426023227758198027 0.000000000001983814816331222067");
	m_poly.push_back("56750.334025069453 3.9796681897565671 -0.000425944549085284496 0.000000000001983448445228127028");
	m_poly.push_back("56750.334030856480 3.9794552351839649 -0.000425865902049457905 0.000000000001983082221445857080");
	m_poly.push_back("56750.334036643508 3.9792423199422098 -0.000425787228359252197 0.000000000001982715873543624029");
	m_poly.push_back("56750.334042430557 3.9790294474482151 -0.000425708498464811213 0.000000000001982349263918777024");
	m_poly.push_back("56750.334048217592 3.9788166109225380 -0.000425629765292974080 0.000000000001981982639031931941");
	m_poly.push_back("56750.334054004627 3.9786038137821040 -0.000425550925832249099 0.000000000001981615519198042934");
	m_poly.push_back("56750.334059791669 3.9783910559952891 -0.000425472211800760722 0.000000000001981248983438710088");
	m_poly.push_back("56750.334065578703 3.9781783410358651 -0.000425393413889247620 0.000000000001980882057081329150");
	m_poly.push_back("56750.334071365738 3.9779656620623398 -0.000425314550740675383 0.000000000001980514826939782125");
	m_poly.push_back("56750.334077152780 3.9777530225273590 -0.000425235692033019692 0.000000000001980147617477020976");
	m_poly.push_back("56750.334082939808 3.9775404224061579 -0.000425156801501758402 0.000000000001979780259823208017");
	m_poly.push_back("56750.334088726850 3.9773278651473789 -0.000425077994681767395 0.000000000001979413291979131140");
	m_poly.push_back("56750.334094513892 3.9771153439176041 -0.000424999035174195873 0.000000000001979045613127658050");
	m_poly.push_back("56750.334100300926 3.9769028621578162 -0.000424920096911967509 0.000000000001978678033206680876");
	m_poly.push_back("56750.334106087961 3.9766904198544450 -0.000424841173047189175 0.000000000001978310520328212924");
	m_poly.push_back("56750.334111875003 3.9764780204689898 -0.000424762195580552916 0.000000000001977942757845381114");
	m_poly.push_back("56750.334117662038 3.9762656571511790 -0.000424683171732987720 0.000000000001977574779384093147");
	m_poly.push_back("56750.334123449073 3.9760533333438270 -0.000424604167105707719 0.000000000001977206890423390009");
	m_poly.push_back("56750.334129236107 3.9758410490629430 -0.000424525025689327488 0.000000000001976838364487955886");
	m_poly.push_back("56750.334135023149 3.9756288076964941 -0.000424446068020295324 0.000000000001976470694191561123");
	m_poly.push_back("56750.334140810177 3.9754166024748332 -0.000424366953276057615 0.000000000001976102292456280982");
	m_poly.push_back("56750.334146597219 3.9752044368086370 -0.000424287803848264508 0.000000000001975733729212268063");
	m_poly.push_back("56750.334152384261 3.9749923106838638 -0.000424208749580423422 0.000000000001975365609090465907");
	m_poly.push_back("56750.334158171303 3.9747802275510420 -0.000424129497123374778 0.000000000001974996566077002811");
	m_poly.push_back("56750.334163958330 3.9745681805840549 -0.000424050379092859081 0.000000000001974628149035419943");
	m_poly.push_back("56750.334169745372 3.9743561732155759 -0.000423971139887504287 0.000000000001974259167728676135");
	m_poly.push_back("56750.334175532407 3.9741442054615939 -0.000423891935250241317 0.000000000001973890347391306995");
	m_poly.push_back("56750.334181319442 3.9739322807023969 -0.000423812700254875001 0.000000000001973521385686765823");
	m_poly.push_back("56750.334187106477 3.9737203921713040 -0.000423733452798297910 0.000000000001973152365954736064");
	m_poly.push_back("56750.334192893519 3.9735085432837538 -0.000423654198635937485 0.000000000001972783314995623189");
	m_poly.push_back("56750.334198680554 3.9732967340322780 -0.000423574847439558326 0.000000000001972413812185802082");
	m_poly.push_back("56750.334204467603 3.9730849678484530 -0.000423495547633461280 0.000000000001972044548679360055");
	m_poly.push_back("56750.334210254630 3.9728732379295368 -0.000423416262500151417 0.000000000001971675353497984118");
	m_poly.push_back("56750.334216041672 3.9726615476710521 -0.000423336803762545684 0.000000000001971305349907761839");
	m_poly.push_back("56750.334221828707 3.9724498971110611 -0.000423257431427405590 0.000000000001970935748659525192");
	m_poly.push_back("56750.334227615742 3.9722382896400301 -0.000423178042225665584 0.000000000001970566068868830176");
	m_poly.push_back("56750.334233402777 3.9720267185018461 -0.000423098557979437618 0.000000000001970195946492285188");
	m_poly.push_back("56750.334239189811 3.9718151870685219 -0.000423019164594354595 0.000000000001969826247220069876");
	m_poly.push_back("56750.334244976853 3.9716036953501161 -0.000422939734456249288 0.000000000001969456376802409085");
	m_poly.push_back("56750.334250763888 3.9713922467959391 -0.000422860180922846800 0.000000000001969085931779351888");
	m_poly.push_back("56750.334256550923 3.9711808345868431 -0.000422780635161047714 0.000000000001968715522945213038");
	m_poly.push_back("56750.334262337958 3.9709694621378149 -0.000422701185234219880 0.000000000001968345560376585011");
	m_poly.push_back("56750.334268125000 3.9707581294526788 -0.000422621657937750212 0.000000000001967975237526638143");
	m_poly.push_back("56750.334273912027 3.9705468399568531 -0.000422542064365047289 0.000000000001967604606052688192");
	m_poly.push_back("56750.334279699076 3.9703355868464412 -0.000422462418878034277 0.000000000001967233732833435963");
	m_poly.push_back("56750.334285486111 3.9701243735403251 -0.000422382828644900482 0.000000000001966863116909258933");
	m_poly.push_back("56750.334291273153 3.9699132000623010 -0.000422303151378691224 0.000000000001966492095704892170");
	m_poly.push_back("56750.334297060188 3.9697020697867300 -0.000422223447227175323 0.000000000001966120949304805031");
	m_poly.push_back("56750.334302847223 3.9694909759522630 -0.000422143857464778475 0.000000000001965750335570116921");
	m_poly.push_back("56750.334308634257 3.9692799219628911 -0.000422064204419326074 0.000000000001965379427149885956");
	m_poly.push_back("56750.334314421299 3.9690689078209438 -0.000421984382405894579 0.000000000001965007731910336938");
	m_poly.push_back("56750.334320208327 3.9688579369539401 -0.000421904660428541192 0.000000000001964636502498827001");
	m_poly.push_back("56750.334325995369 3.9686470025657208 -0.000421824927455479283 0.000000000001964265221884106094");
	m_poly.push_back("56750.334331782396 3.9684361080523409 -0.000421745148758744318 0.000000000001963893728351088956");
	m_poly.push_back("56750.334337569453 3.9682252534259019 -0.000421665360318639179 0.000000000001963522189446117025");
	m_poly.push_back("56750.334343356481 3.9680144421203409 -0.000421585511686355975 0.000000000001963150370248222990");
	m_poly.push_back("56750.334349143523 3.9678036673170460 -0.000421505629924838227 0.000000000001962778396779853860");
	m_poly.push_back("56750.334354930557 3.9675929324465189 -0.000421425828837003816 0.000000000001962406798976746166");
	m_poly.push_back("56750.334360717592 3.9673822375131338 -0.000421345955744154281 0.000000000001962034865873304992");
	m_poly.push_back("56750.334366504627 3.9671715859193610 -0.000421266029182674020 0.000000000001961662683785891074");
	m_poly.push_back("56750.334372291669 3.9669609708744611 -0.000421186080372203510 0.000000000001961290398093094885");
	m_poly.push_back("56750.334378078704 3.9667503958124661 -0.000421106192917431084 0.000000000001960918398109178185");
	m_poly.push_back("56750.334383865738 3.9665398607102591 -0.000421026237951850624 0.000000000001960546083752710998");
	m_poly.push_back("56750.334389652780 3.9663293690088000 -0.000420946226622162176 0.000000000001960173506929149939");
	m_poly.push_back("56750.334395439822 3.9661189138781499 -0.000420866215319119026 0.000000000001959800930229220070");
	m_poly.push_back("56750.334401226850 3.9659084987674089 -0.000420786201573572212 0.000000000001959428342154641143");
	m_poly.push_back("56750.334407013892 3.9656981236825750 -0.000420706186406932011 0.000000000001959055747461725917");
	m_poly.push_back("56750.334412800927 3.9654877920140201 -0.000420626085575018596 0.000000000001958682753857544127");
	m_poly.push_back("56750.334418587961 3.9652774969755660 -0.000420545999193846511 0.000000000001958309827544413080");
	m_poly.push_back("56750.334424375003 3.9650672419969570 -0.000420465865117234489 0.000000000001957936679131226882");
	m_poly.push_back("56750.334430162038 3.9648570270550159 -0.000420385844541558494 0.000000000001957564059247104076");
	m_poly.push_back("56750.334435949073 3.9646468555930952 -0.000420305584973536890 0.000000000001957190326466896016");
	m_poly.push_back("56750.334441736108 3.9644367207842901 -0.000420225530131751522 0.000000000001956817547017056995");
	m_poly.push_back("56750.334447523150 3.9642266260757100 -0.000420145357037249886 0.000000000001956444216909239908");
	m_poly.push_back("56750.334453310177 3.9640165714654851 -0.000420065126370710890 0.000000000001956070618709876811");
	m_poly.push_back("56750.334459097219 3.9638065603554580 -0.000419984963614662222 0.000000000001955697336742113853");
	m_poly.push_back("56750.334464884261 3.9635965859671161 -0.000419904641891577783 0.000000000001955323314525954835");
	m_poly.push_back("56750.334470671303 3.9633866516873382 -0.000419824405611720708 0.000000000001954949690184971810");
	m_poly.push_back("56750.334476458331 3.9631767575450709 -0.000419744127018814085 0.000000000001954575868807641095");
	m_poly.push_back("56750.334482245373 3.9629669069370181 -0.000419663861100282411 0.000000000001954202106448710046");
	m_poly.push_back("56750.334488032408 3.9627570930967950 -0.000419583559228093206 0.000000000001953828176667057037");
	m_poly.push_back("56750.334493819442 3.9625473194139480 -0.000419503183689986289 0.000000000001953453903850884968");
	m_poly.push_back("56750.334499606477 3.9623375858981729 -0.000419422857268412423 0.000000000001953079859750535165");
	m_poly.push_back("56750.334505393519 3.9621278959624568 -0.000419342408739407078 0.000000000001952705247041885960");
	m_poly.push_back("56750.334511180547 3.9619182428339048 -0.000419262034604629815 0.000000000001952330980757901028");
	m_poly.push_back("56750.334516967603 3.9617086298915489 -0.000419181678046998618 0.000000000001951956796323024118");
	m_poly.push_back("56750.334522754631 3.9614990571569360 -0.000419101234713865217 0.000000000001951582207807366131");
	m_poly.push_back("56750.334528541673 3.9612895280489049 -0.000419020772995338782 0.000000000001951207533676782836");
	m_poly.push_back("56750.334534328700 3.9610800357619409 -0.000418940285865889903 0.000000000001950832741216921182");
	m_poly.push_back("56750.334540115742 3.9608705837253182 -0.000418859787643032389 0.000000000001950457897098490074");
	m_poly.push_back("56750.334545902777 3.9606611719385318 -0.000418779271620982904 0.000000000001950082970095252962");
	m_poly.push_back("56750.334551689812 3.9604518037842360 -0.000418698760572759404 0.000000000001949708066251950169");
	m_poly.push_back("56750.334557476846 3.9602424725276690 -0.000418618218145022278 0.000000000001949333016285927164");
	m_poly.push_back("56750.334563263888 3.9600331815362009 -0.000418537685913628597 0.000000000001948958013799469108");
	m_poly.push_back("56750.334569050923 3.9598239308206740 -0.000418457051274707687 0.000000000001948582534439939850");
	m_poly.push_back("56750.334574837972 3.9596147238241910 -0.000418376501217476574 0.000000000001948207448943326828");
	m_poly.push_back("56750.334580625000 3.9594055537141539 -0.000418295951932964698 0.000000000001947832367044516090");
	m_poly.push_back("56750.334586412027 3.9591964239191251 -0.000418215206848458277 0.000000000001947456373380069957");
	m_poly.push_back("56750.334592199077 3.9589873344451272 -0.000418134574607632002 0.000000000001947080905184095929");
	m_poly.push_back("56750.334597986111 3.9587782887165108 -0.000418053876863292090 0.000000000001946705131962360137");
	m_poly.push_back("56750.334603773154 3.9585692799261820 -0.000417973216529230124 0.000000000001946329532945339966");
	m_poly.push_back("56750.334609560188 3.9583603114781432 -0.000417892490407435692 0.000000000001945953627579455178");
	m_poly.push_back("56750.334615347223 3.9581513834039068 -0.000417811758619032313 0.000000000001945577695825503954");
	m_poly.push_back("56750.334621134258 3.9579424990891998 -0.000417730999083784616 0.000000000001945201634863966802");
	m_poly.push_back("56750.334626921300 3.9577336517586992 -0.000417650300824298293 0.000000000001944825859239478934");
	m_poly.push_back("56750.334632708327 3.9575248448114788 -0.000417569442576365597 0.000000000001944449338609867043");
	m_poly.push_back("56750.334638495369 3.9573160782693981 -0.000417488716632207879 0.000000000001944073434066856116");
	m_poly.push_back("56750.334644282397 3.9571073555190859 -0.000417407789558955878 0.000000000001943696592942251811");
	m_poly.push_back("56750.334650069453 3.9568986698028281 -0.000417327021790249615 0.000000000001943320493636782002");
	m_poly.push_back("56750.334655856481 3.9566900245000092 -0.000417246152528844721 0.000000000001942943921718242146");
	m_poly.push_back("56750.334661643523 3.9564814196214781 -0.000417165215010557909 0.000000000001942567031952964150");
	m_poly.push_back("56750.334667430558 3.9562728586090721 -0.000417084349390704323 0.000000000001942190476989680877");
	m_poly.push_back("56750.334673217592 3.9560643346439681 -0.000417003475688383315 0.000000000001941813884388922023");
	m_poly.push_back("56750.334679004627 3.9558558511302691 -0.000416922496185674922 0.000000000001941436799115853003");
	m_poly.push_back("56750.334684791669 3.9556474080873820 -0.000416841501752606412 0.000000000001941059644317011920");
	m_poly.push_back("56750.334690578697 3.9554390089395040 -0.000416760563061917798 0.000000000001940682749087489062");
	m_poly.push_back("56750.334696365739 3.9552306468685989 -0.000416679626396286985 0.000000000001940305863287512974");
	m_poly.push_back("56750.334702152781 3.9550223252846579 -0.000416598595965151698 0.000000000001939928540857016953");
	m_poly.push_back("56750.334707939823 3.9548140442192228 -0.000416517569157804095 0.000000000001939551235300231965");
	m_poly.push_back("56750.334713726850 3.9546058070728192 -0.000416436508563966285 0.000000000001939173772411582089");
	m_poly.push_back("56750.334719513892 3.9543976070451992 -0.000416355469339817100 0.000000000001938796409032826951");
	m_poly.push_back("56750.334725300927 3.9541894475421069 -0.000416274400397056608 0.000000000001938418907265138937");
	m_poly.push_back("56750.334731087962 3.9539813285933052 -0.000416193308778978221 0.000000000001938041299906407051");
	m_poly.push_back("56750.334736874996 3.9537732535942101 -0.000416112182157598524 0.000000000001937663529549583903");
	m_poly.push_back("56750.334742662038 3.9535652157636632 -0.000416031048386343680 0.000000000001937285725898094865");
	m_poly.push_back("56750.334748449073 3.9533572184831560 -0.000415949917928012285 0.000000000001936907937672741903");
	m_poly.push_back("56750.334754236108 3.9531492617855610 -0.000415868710481957782 0.000000000001936529790944499935");
	m_poly.push_back("56750.334760023150 3.9529413490828111 -0.000415787565954803999 0.000000000001936151937203956070");
	m_poly.push_back("56750.334765810177 3.9527334735740922 -0.000415706406825917796 0.000000000001935774015468326152");
	m_poly.push_back("56750.334771597220 3.9525256386390848 -0.000415625143779592486 0.000000000001935395609828545798");
	m_poly.push_back("56750.334777384262 3.9523178443552331 -0.000415543885129774585 0.000000000001935017224660729915");
	m_poly.push_back("56750.334783171304 3.9521100940744729 -0.000415462639687241289 0.000000000001934638900992742039");
	m_poly.push_back("56750.334788958331 3.9519023810256888 -0.000415381409770151011 0.000000000001934260649620380090");
	m_poly.push_back("56750.334794745373 3.9516947086113752 -0.000415300069833802977 0.000000000001933881885930058165");
	m_poly.push_back("56750.334800532408 3.9514870768616159 -0.000415218815687074113 0.000000000001933503521727912004");
	m_poly.push_back("56750.334806319443 3.9512794891549872 -0.000415137514280115015 0.000000000001933124937452022080");
	m_poly.push_back("56750.334812106477 3.9510719387137940 -0.000415056186742804995 0.000000000001932746231496615200");
	m_poly.push_back("56750.334817893519 3.9508644289310020 -0.000414974793684217715 0.000000000001932367220433115025");
	m_poly.push_back("56750.334823680547 3.9506569598478301 -0.000414893400146011378 0.000000000001931988207135345894");
	m_poly.push_back("56750.334829467589 3.9504495348490600 -0.000414812038839974719 0.000000000001931609343929011003");
	m_poly.push_back("56750.334835254631 3.9502421471697979 -0.000414730636855145912 0.000000000001931230291296852923");
	m_poly.push_back("56750.334841041673 3.9500348001767698 -0.000414649241300763720 0.000000000001930851268607924179");
	m_poly.push_back("56750.334846828700 3.9498274938858491 -0.000414567771618608327 0.000000000001930471900733784165");
	m_poly.push_back("56750.334852615742 3.9496202317398499 -0.000414486332676474790 0.000000000001930092676002770132");
	m_poly.push_back("56750.334858402777 3.9494130069236770 -0.000414404845097427420 0.000000000001929713224787991052");
	m_poly.push_back("56750.334864189812 3.9492058228552862 -0.000414323348405619917 0.000000000001929333731137745929");
	m_poly.push_back("56750.334869976847 3.9489986795334850 -0.000414241859698579774 0.000000000001928954274668600083");
	m_poly.push_back("56750.334875763889 3.9487915803553220 -0.000414160268077747777 0.000000000001928574338968495004");
	m_poly.push_back("56750.334881550923 3.9485845185579849 -0.000414078737377404106 0.000000000001928194686951050115");
	m_poly.push_back("56750.334887337973 3.9483774975265962 -0.000413997174160672091 0.000000000001927814883516502174");
	m_poly.push_back("56750.334893125000 3.9481705172879580 -0.000413915573933344586 0.000000000001927434907737066978");
	m_poly.push_back("56750.334898912028 3.9479635812328731 -0.000413833977614023378 0.000000000001927054950154431130");
	m_poly.push_back("56750.334904699077 3.9477566825959172 -0.000413752419477699624 0.000000000001926675170374763127");
	m_poly.push_back("56750.334910486112 3.9475498247433771 -0.000413670799310658394 0.000000000001926295101741186089");
	m_poly.push_back("56750.334916273147 3.9473430077130489 -0.000413589144131433907 0.000000000001925914870068552961");
	m_poly.push_back("56750.334922060189 3.9471362349257721 -0.000413507480914151408 0.000000000001925534600964511987");
	m_poly.push_back("56750.334927847223 3.9469294995634230 -0.000413425798298946783 0.000000000001925154241531396945");
	m_poly.push_back("56750.334933634258 3.9467228050259200 -0.000413344130569693084 0.000000000001924773951415520038");
	m_poly.push_back("56750.334939421293 3.9465161513437859 -0.000413262388659375477 0.000000000001924393315866272830");
	m_poly.push_back("56750.334945208328 3.9463095419275769 -0.000413180671849116405 0.000000000001924012797197014079");
	m_poly.push_back("56750.334950995370 3.9461029699830159 -0.000413098952916099308 0.000000000001923632268642413863");
	m_poly.push_back("56750.334956782397 3.9458964389050331 -0.000413017170744435524 0.000000000001923251445609044942");
	m_poly.push_back("56750.334962569454 3.9456899487088348 -0.000412935380747877525 0.000000000001922870586137298903");
	m_poly.push_back("56750.334968356481 3.9454835028042021 -0.000412853638677686990 0.000000000001922489949838823163");
	m_poly.push_back("56750.334974143523 3.9452770943923592 -0.000412771787067857810 0.000000000001922108803456176977");
	m_poly.push_back("56750.334979930558 3.9450707269065628 -0.000412689952528910394 0.000000000001921727736565147006");
	m_poly.push_back("56750.334985717593 3.9448644003231959 -0.000412608145612461776 0.000000000001921346798300524976");
	m_poly.push_back("56750.334991504627 3.9446581180593450 -0.000412526281976631579 0.000000000001920965595914403824");
	m_poly.push_back("56750.334997291669 3.9444518733353600 -0.000412444398161278182 0.000000000001920584299559603811");
	m_poly.push_back("56750.335003078697 3.9442456695364441 -0.000412362519417450992 0.000000000001920203026820063078");
	m_poly.push_back("56750.335008865739 3.9440395066979219 -0.000412280660817619920 0.000000000001919821847882437877");
	m_poly.push_back("56750.335014652781 3.9438333882264551 -0.000412198710122068800 0.000000000001919440240089431033");
	m_poly.push_back("56750.335020439823 3.9436273072846730 -0.000412116782625863716 0.000000000001919058740326315182");
	m_poly.push_back("56750.335026226850 3.9434212673416651 -0.000412034860396585391 0.000000000001918677265088362131");
	m_poly.push_back("56750.335032013892 3.9432152683672639 -0.000411952862343103977 0.000000000001918295436765572877");
	m_poly.push_back("56750.335037800927 3.9430093137678810 -0.000411870842990988482 0.000000000001917913509262013065");
	m_poly.push_back("56750.335043587962 3.9428033967702190 -0.000411788866920042326 0.000000000001917531783301629080");
	m_poly.push_back("56750.335049374997 3.9425975207869182 -0.000411706874764097602 0.000000000001917149982438759064");
	m_poly.push_back("56750.335055162039 3.9423916858103638 -0.000411624886027658389 0.000000000001916768197498325010");
	m_poly.push_back("56750.335060949074 3.9421858952540281 -0.000411542795318901812 0.000000000001916385937711009163");
	m_poly.push_back("56750.335066736108 3.9419801423150842 -0.000411460747562686390 0.000000000001916003877936572893");
	m_poly.push_back("56750.335072523150 3.9417744304086559 -0.000411378677681826621 0.000000000001915621715135355880");
	m_poly.push_back("56750.335078310192 3.9415687595573359 -0.000411296566056354690 0.000000000001915239357944763887");
	m_poly.push_back("56750.335084097220 3.9413631331472638 -0.000411214443497347215 0.000000000001914856949839640898");
	m_poly.push_back("56750.335089884262 3.9411575443883242 -0.000411132335477899075 0.000000000001914474609439152093");
	m_poly.push_back("56750.335095671297 3.9409519967029261 -0.000411050223177407913 0.000000000001914092249102625016");
	m_poly.push_back("56750.335101458331 3.9407464900890101 -0.000410968054548335791 0.000000000001913709626464825911");
	m_poly.push_back("56750.335107245373 3.9405410279629658 -0.000410885907217897696 0.000000000001913327103005247967");
	m_poly.push_back("56750.335113032408 3.9403356035162509 -0.000410803644174947383 0.000000000001912944040716812944");
	m_poly.push_back("56750.335118819443 3.9401302201562318 -0.000410721510504767488 0.000000000001912561580866407093");
	m_poly.push_back("56750.335124606478 3.9399248779173748 -0.000410639292403889211 0.000000000001912178727854045045");
	m_poly.push_back("56750.335130393520 3.9397195801976421 -0.000410557035427230410 0.000000000001911795693810944925");
	m_poly.push_back("56750.335136180547 3.9395143201757161 -0.000410474806428343618 0.000000000001911412790049019888");
	m_poly.push_back("56750.335141967589 3.9393091012773618 -0.000410392463440020121 0.000000000001911029355481539132");
	m_poly.push_back("56750.335147754631 3.9391039235243839 -0.000410310319919888209 0.000000000001910646849759327137");
	m_poly.push_back("56750.335153541673 3.9388987903048398 -0.000410227956139194987 0.000000000001910263318367656843");
	m_poly.push_back("56750.335159328701 3.9386936948346478 -0.000410145599970232293 0.000000000001909879822420403975");
	m_poly.push_back("56750.335165115743 3.9384886405295720 -0.000410063318302079580 0.000000000001909496673393702079");
	m_poly.push_back("56750.335170902777 3.9382836273927482 -0.000409981028666923182 0.000000000001909113487266870094");
	m_poly.push_back("56750.335176689812 3.9380786588211349 -0.000409898639226616475 0.000000000001908729836384826030");
	m_poly.push_back("56750.335182476847 3.9378737280212519 -0.000409816309365641199 0.000000000001908346462940167946");
	m_poly.push_back("56750.335188263889 3.9376688384252052 -0.000409733858427527211 0.000000000001907962525685265948");
	m_poly.push_back("56750.335194050916 3.9374639900022661 -0.000409651531923884522 0.000000000001907579167872646078");
	m_poly.push_back("56750.335199837973 3.9372591861928741 -0.000409569125936723975 0.000000000001907195439934766131");
	m_poly.push_back("56750.335205625001 3.9370544202045559 -0.000409486684741980106 0.000000000001906811548048326027");
	m_poly.push_back("56750.335211412043 3.9368496954167669 -0.000409404247526297025 0.000000000001906427674689952125");
	m_poly.push_back("56750.335217199077 3.9366450118524310 -0.000409321735818685693 0.000000000001906043454450624886");
	m_poly.push_back("56750.335222986112 3.9364403729186259 -0.000409239303665646979 0.000000000001905659604664859166");
	m_poly.push_back("56750.335228773147 3.9362357718126240 -0.000409156794249548492 0.000000000001905275395094458193");
	m_poly.push_back("56750.335234560182 3.9360312119724510 -0.000409074344004217412 0.000000000001904891461058531972");
	m_poly.push_back("56750.335240347224 3.9358266933800601 -0.000408991835138792473 0.000000000001904507254050642025");
	m_poly.push_back("56750.335246134258 3.9356222194449360 -0.000408909260307998516 0.000000000001904122739866292853");
	m_poly.push_back("56750.335251921293 3.9354177833755930 -0.000408826763060010407 0.000000000001903738586954545994");
	m_poly.push_back("56750.335257708342 3.9352133885833389 -0.000408744196924530287 0.000000000001903354113259533873");
	m_poly.push_back("56750.335263495370 3.9350090350615901 -0.000408661609472652627 0.000000000001902969540301428953");
	m_poly.push_back("56750.335269282397 3.9348047262404720 -0.000408579026399518211 0.000000000001902584987732141906");
	m_poly.push_back("56750.335275069447 3.9346004553127432 -0.000408496423875170911 0.000000000001902200344585616915");
	m_poly.push_back("56750.335280856481 3.9343962256948588 -0.000408413797947195509 0.000000000001901815592456571997");
	m_poly.push_back("56750.335286643523 3.9341920373910488 -0.000408331187908818013 0.000000000001901430914318351022");
	m_poly.push_back("56750.335292430558 3.9339878937905302 -0.000408248567221123422 0.000000000001901046186589096919");
	m_poly.push_back("56750.335298217593 3.9337837881144688 -0.000408165917827346778 0.000000000001900661325186251810");
	m_poly.push_back("56750.335304004628 3.9335797237748351 -0.000408083156309318085 0.000000000001900275941663289016");
	m_poly.push_back("56750.335309791670 3.9333757007717391 -0.000408000503265108295 0.000000000001899891063260023991");
	m_poly.push_back("56750.335315578697 3.9331717225297660 -0.000407917781244588327 0.000000000001899505863659497885");
	m_poly.push_back("56750.335321365739 3.9329677822255009 -0.000407835098437358200 0.000000000001899120846659602122");
	m_poly.push_back("56750.335327152781 3.9327638832873628 -0.000407752346103144295 0.000000000001898735505898575140");
	m_poly.push_back("56750.335332939823 3.9325600257172559 -0.000407669654609482224 0.000000000001898350448447559848");
	m_poly.push_back("56750.335338726851 3.9323562129205540 -0.000407586892896053194 0.000000000001897965064008996165");
	m_poly.push_back("56750.335344513893 3.9321524380974289 -0.000407504076008681601 0.000000000001897579422646250852");
	m_poly.push_back("56750.335350300928 3.9319487046671360 -0.000407421313993270391 0.000000000001897194036800098800");
	m_poly.push_back("56750.335356087962 3.9317450126337001 -0.000407338525891917506 0.000000000001896808529480930910");
	m_poly.push_back("56750.335361874997 3.9315413653946378 -0.000407255726421819103 0.000000000001896422969220622156");
	m_poly.push_back("56750.335367662039 3.9313377561596390 -0.000407172848534022584 0.000000000001896037043798975087");
	m_poly.push_back("56750.335373449067 3.9311341883684099 -0.000407090032641416405 0.000000000001895651407064043980");
	m_poly.push_back("56750.335379236109 3.9309306619738651 -0.000407007200015437726 0.000000000001895265692407390949");
	m_poly.push_back("56750.335385023151 3.9307271804361741 -0.000406924335725726918 0.000000000001894879830303714066");
	m_poly.push_back("56750.335390810193 3.9305237369110579 -0.000406841479776056411 0.000000000001894494007035885831");
	m_poly.push_back("56750.335396597220 3.9303203348185671 -0.000406758564282581610 0.000000000001894107906494728025");
	m_poly.push_back("56750.335402384262 3.9301169741880702 -0.000406675665513597415 0.000000000001893721883832160815");
	m_poly.push_back("56750.335408171297 3.9299136584312020 -0.000406592740099115998 0.000000000001893335737090527069");
	m_poly.push_back("56750.335413958332 3.9297103807259628 -0.000406509807357122990 0.000000000001892949556227037936");
	m_poly.push_back("56750.335419745366 3.9295071444769389 -0.000406426821652763285 0.000000000001892563128737426122");
	m_poly.push_back("56750.335425532408 3.9293039497256590 -0.000406343856572018592 0.000000000001892176797283117091");
	m_poly.push_back("56750.335431319443 3.9291007998332321 -0.000406260951115014414 0.000000000001891790743472193921");
	m_poly.push_back("56750.335437106478 3.9288976880582620 -0.000406177892899375393 0.000000000001891403978322963922");
	m_poly.push_back("56750.335442893520 3.9286946177564159 -0.000406094950689348878 0.000000000001891017753366077074");
	m_poly.push_back("56750.335448680547 3.9284915889830820 -0.000406011913816753075 0.000000000001890631087601364173");
	m_poly.push_back("56750.335454467589 3.9282886051014998 -0.000405928903154444576 0.000000000001890244543886535115");
	m_poly.push_back("56750.335460254631 3.9280856593375471 -0.000405845888646302487 0.000000000001889857982262652068");
	m_poly.push_back("56750.335466041673 3.9278827551069782 -0.000405762784316189898 0.000000000001889471002371735830");
	m_poly.push_back("56750.335471828701 3.9276798924083520 -0.000405679725336713316 0.000000000001889084233660288067");
	m_poly.push_back("56750.335477615743 3.9274770746459029 -0.000405596606760141409 0.000000000001888697187426940940");
	m_poly.push_back("56750.335483402778 3.9272742950092292 -0.000405513534173697386 0.000000000001888310355351339009");
	m_poly.push_back("56750.335489189813 3.9270715569427121 -0.000405430493028940291 0.000000000001887923669686612200");
	m_poly.push_back("56750.335494976847 3.9268688604104840 -0.000405347287188365286 0.000000000001887536217096332963");
	m_poly.push_back("56750.335500763889 3.9266662088585731 -0.000405264203810495080 0.000000000001887149334766181922");
	m_poly.push_back("56750.335506550917 3.9264635954690701 -0.000405181067852177282 0.000000000001886762207588743951");
	m_poly.push_back("56750.335512337973 3.9262610236506470 -0.000405097926946678690 0.000000000001886375057373312140");
	m_poly.push_back("56750.335518125001 3.9260584934188549 -0.000405014726408557690 0.000000000001885987629470953990");
	m_poly.push_back("56750.335523912043 3.9258560081905820 -0.000404931535080896776 0.000000000001885600244456849969");
	m_poly.push_back("56750.335529699078 3.9256535611552650 -0.000404848300063502715 0.000000000001885212655996086975");
	m_poly.push_back("56750.335535486112 3.9254511557035192 -0.000404765106213401374 0.000000000001884825259234581999");
	m_poly.push_back("56750.335541273147 3.9252487918693348 -0.000404681888533343903 0.000000000001884437751505305853");
	m_poly.push_back("56750.335547060182 3.9250464730407648 -0.000404598678216325377 0.000000000001884050278061567903");
	m_poly.push_back("56750.335552847217 3.9248441924385160 -0.000404515437914240825 0.000000000001883662664988490877");
	m_poly.push_back("56750.335558634259 3.9246419534583721 -0.000404432181340639316 0.000000000001883274976144370002");
	m_poly.push_back("56750.335564421293 3.9244397561265312 -0.000404348921098840984 0.000000000001882887270217987200");
	m_poly.push_back("56750.335570208343 3.9242376038124949 -0.000404265598764152686 0.000000000001882499275147754825");
	m_poly.push_back("56750.335575995370 3.9240354897545200 -0.000404182320749807791 0.000000000001882111486459723036");
	m_poly.push_back("56750.335581782398 3.9238334173656231 -0.000404099012581642211 0.000000000001881723557356129923");
	m_poly.push_back("56750.335587569447 3.9236313866021120 -0.000404015669001610593 0.000000000001881335463352296878");
	m_poly.push_back("56750.335593356482 3.9234294009302948 -0.000403932324506750485 0.000000000001880947365087168187");
	m_poly.push_back("56750.335599143516 3.9232274535076579 -0.000403848979780962688 0.000000000001880559265746211138");
	m_poly.push_back("56750.335604930558 3.9230255477913980 -0.000403765684189471674 0.000000000001880171395203760891");
	m_poly.push_back("56750.335610717593 3.9228236837195261 -0.000403682225243319427 0.000000000001879782763981004154");
	m_poly.push_back("56750.335616504628 3.9226218647650271 -0.000403598901842811225 0.000000000001879394763940539939");
	m_poly.push_back("56750.335622291670 3.9224200841021979 -0.000403515428481445219 0.000000000001879006065589930186");
	m_poly.push_back("56750.335628078697 3.9222183451221668 -0.000403432067279858181 0.000000000001878617889523101135");
	m_poly.push_back("56750.335633865740 3.9220166478694281 -0.000403348623850267223 0.000000000001878229330551397084");
	m_poly.push_back("56750.335639652767 3.9218149957311970 -0.000403265242229347782 0.000000000001877841059397454954");
	m_poly.push_back("56750.335645439824 3.9216133819044772 -0.000403181770200256614 0.000000000001877452367247166910");
	m_poly.push_back("56750.335651226851 3.9214118098016950 -0.000403098274589900522 0.000000000001877063565287165806");
	m_poly.push_back("56750.335657013893 3.9212102794320240 -0.000403014854501764907 0.000000000001876675115004019140");
	m_poly.push_back("56750.335662800928 3.9210087942066671 -0.000402931273252781007 0.000000000001876285914255897179");
	m_poly.push_back("56750.335668587963 3.9208073473260150 -0.000402847873806024025 0.000000000001875897560089873899");
	m_poly.push_back("56750.335674374997 3.9206059421862212 -0.000402764337108311218 0.000000000001875508566798391073");
	m_poly.push_back("56750.335680162039 3.9204045787991770 -0.000402680768604756812 0.000000000001875119425398524010");
	m_poly.push_back("56750.335685949067 3.9202032605944410 -0.000402597295607545909 0.000000000001874730728733428848");
	m_poly.push_back("56750.335691736109 3.9200019807338120 -0.000402513805805908016 0.000000000001874341953816134933");
	m_poly.push_back("56750.335697523151 3.9198007426584320 -0.000402430190752670123 0.000000000001873952595649735983");
	m_poly.push_back("56750.335703310193 3.9195995463647799 -0.000402346626281467482 0.000000000001873563473023400159");
	m_poly.push_back("56750.335709097220 3.9193983952594200 -0.000402263057289412290 0.000000000001873174329343902896");
	m_poly.push_back("56750.335714884262 3.9191972825272670 -0.000402179433798433494 0.000000000001872784931883532140");
	m_poly.push_back("56750.335720671297 3.9189962116016481 -0.000402095861990837424 0.000000000001872395775091772986");
	m_poly.push_back("56750.335726458332 3.9187951824906690 -0.000402012264088417786 0.000000000001872006496785628865");
	m_poly.push_back("56750.335732245367 3.9185941985674542 -0.000401928657988066774 0.000000000001871617180303601017");
	m_poly.push_back("56750.335738032409 3.9183932530742021 -0.000401845039275514006 0.000000000001871227805090935826");
	m_poly.push_back("56750.335743819443 3.9181923493973190 -0.000401761367005511909 0.000000000001870838180481069072");
	m_poly.push_back("56750.335749606478 3.9179914875465731 -0.000401677680524578011 0.000000000001870448489695500830");
	m_poly.push_back("56750.335755393520 3.9177906709368111 -0.000401594074539691006 0.000000000001870059173747567913");
	m_poly.push_back("56750.335761180548 3.9175898927454469 -0.000401510306665645077 0.000000000001869669103943999922");
	m_poly.push_back("56750.335766967590 3.9173891564103629 -0.000401426591896075205 0.000000000001869279281426521955");
	m_poly.push_back("56750.335772754632 3.9171884619097330 -0.000401342898549484825 0.000000000001868889558666672857");
	m_poly.push_back("56750.335778541667 3.9169878126832951 -0.000401259193771976222 0.000000000001868499782676161198");
	m_poly.push_back("56750.335784328701 3.9167872019105960 -0.000401175531618381187 0.000000000001868110205168226808");
	m_poly.push_back("56750.335790115743 3.9165866329944352 -0.000401091751736098025 0.000000000001867720079442472924");
	m_poly.push_back("56750.335795902778 3.9163861059564682 -0.000401008019789064220 0.000000000001867330176931723186");
	m_poly.push_back("56750.335801689813 3.9161856242175670 -0.000400924182023879624 0.000000000001866939781665500127");
	m_poly.push_back("56750.335807476848 3.9159851809360471 -0.000400840514505657823 0.000000000001866550179172996194");
	m_poly.push_back("56750.335813263890 3.9157847795372351 -0.000400756644366194025 0.000000000001866159633150912934");
	m_poly.push_back("56750.335819050917 3.9155844200401808 -0.000400672877922123320 0.000000000001865769569997236891");
	m_poly.push_back("56750.335824837974 3.9153841058468819 -0.000400589130263127782 0.000000000001865379594317205018");
	m_poly.push_back("56750.335830625001 3.9151838301637341 -0.000400505228788559605 0.000000000001864988902376984960");
	m_poly.push_back("56750.335836412043 3.9149835963754538 -0.000400421422118670022 0.000000000001864598651904555875");
	m_poly.push_back("56750.335842199071 3.9147834045138490 -0.000400337629775175978 0.000000000001864208468143710048");
	m_poly.push_back("56750.335847986113 3.9145832579839701 -0.000400253768171153306 0.000000000001863817961861978048");
	m_poly.push_back("56750.335853773147 3.9143831499629549 -0.000400169899443900785 0.000000000001863427422409746915");
	m_poly.push_back("56750.335859560182 3.9141830838835001 -0.000400086084060933688 0.000000000001863037131360306096");
	m_poly.push_back("56750.335865347217 3.9139830597205370 -0.000400002148568947180 0.000000000001862646281008762972");
	m_poly.push_back("56750.335871134259 3.9137830809399992 -0.000399918211282830887 0.000000000001862255422301351088");
	m_poly.push_back("56750.335876921286 3.9135831406911801 -0.000399834379948092798 0.000000000001861865056967999028");
	m_poly.push_back("56750.335882708343 3.9133832423851200 -0.000399750441347465125 0.000000000001861474192138026875");
	m_poly.push_back("56750.335888495370 3.9131833860362941 -0.000399666532092411491 0.000000000001861083463958136820");
	m_poly.push_back("56750.335894282412 3.9129835750641790 -0.000399582562671825525 0.000000000001860692455609360960");
	m_poly.push_back("56750.335900069447 3.9127838026619939 -0.000399498618369061498 0.000000000001860301564224057001");
	m_poly.push_back("56750.335905856482 3.9125840722332401 -0.000399414709549217675 0.000000000001859910838068079995");
	m_poly.push_back("56750.335911643517 3.9123843837877321 -0.000399330723128804490 0.000000000001859519750555394826");
	m_poly.push_back("56750.335917430559 3.9121847407288111 -0.000399246798428244600 0.000000000001859128950446880060");
	m_poly.push_back("56750.335923217594 3.9119851362654190 -0.000399162778212459315 0.000000000001858737705560435002");
	m_poly.push_back("56750.335929004628 3.9117855737850218 -0.000399078791490063787 0.000000000001858346616638843145");
	m_poly.push_back("56750.335934791663 3.9115860533046871 -0.000398994771069996273 0.000000000001857955370799339810");
	m_poly.push_back("56750.335940578712 3.9113865782398460 -0.000398910688459338408 0.000000000001857563835361027871");
	m_poly.push_back("56750.335946365740 3.9111871417984090 -0.000398826719571662421 0.000000000001857172829486127080");
	m_poly.push_back("56750.335952152767 3.9109877473445720 -0.000398742621417797914 0.000000000001856781221667731166");
	m_poly.push_back("56750.335957939817 3.9107883949317999 -0.000398658557903079722 0.000000000001856389775149532081");
	m_poly.push_back("56750.335963726851 3.9105890879536598 -0.000398574490373773389 0.000000000001855998309935650035");
	m_poly.push_back("56750.335969513893 3.9103898196038389 -0.000398490503384922926 0.000000000001855607219766994952");
	m_poly.push_back("56750.335975300928 3.9101905932867460 -0.000398406381827430100 0.000000000001855215502963368963");
	m_poly.push_back("56750.335981087963 3.9099914090157020 -0.000398322294189655220 0.000000000001854823944109838936");
	m_poly.push_back("56750.335986874998 3.9097922701858501 -0.000398238162231536721 0.000000000001854432178872325928");
	m_poly.push_back("56750.335992662040 3.9095931700044670 -0.000398154102864634193 0.000000000001854040751663507137");
	m_poly.push_back("56750.335998449067 3.9093941118991031 -0.000398070000820527283 0.000000000001853649125722569914");
	m_poly.push_back("56750.336004236109 3.9091950958364361 -0.000397985815544768984 0.000000000001853257112203154919");
	m_poly.push_back("56750.336010023151 3.9089961252612300 -0.000397901685247553183 0.000000000001852865354696229981");
	m_poly.push_back("56750.336015810193 3.9087971933595651 -0.000397817495166723721 0.000000000001852473318799660098");
	m_poly.push_back("56750.336021597221 3.9085983035140539 -0.000397733332548519912 0.000000000001852081410785006929");
	m_poly.push_back("56750.336027384263 3.9083994557771349 -0.000397649230095848490 0.000000000001851689782937048020");
	m_poly.push_back("56750.336033171297 3.9082006535182039 -0.000397565015545570913 0.000000000001851297633092632908");
	m_poly.push_back("56750.336038958332 3.9080018899429172 -0.000397480751132686183 0.000000000001850905251056860074");
	m_poly.push_back("56750.336044745367 3.9078031684710210 -0.000397396622745064285 0.000000000001850513502437178177");
	m_poly.push_back("56750.336050532409 3.9076044891157040 -0.000397312410184317210 0.000000000001850121361854937028");
	m_poly.push_back("56750.336056319436 3.9074058552594608 -0.000397228189692093480 0.000000000001849729184337465134");
	m_poly.push_back("56750.336062106478 3.9072072601121688 -0.000397143904451311997 0.000000000001849336705310640076");
	m_poly.push_back("56750.336067893521 3.9070087070801161 -0.000397059706732106601 0.000000000001848944633837013872");
	m_poly.push_back("56750.336073680563 3.9068101961687201 -0.000396975452746493114 0.000000000001848552300351607043");
	m_poly.push_back("56750.336079467590 3.9066117307835690 -0.000396891228639363700 0.000000000001848160105997545056");
	m_poly.push_back("56750.336085254632 3.9064133041404849 -0.000396806975515288105 0.000000000001847767776522185033");
	m_poly.push_back("56750.336091041667 3.9062149196337450 -0.000396722671009812073 0.000000000001847375207782634822");
	m_poly.push_back("56750.336096828702 3.9060165772653530 -0.000396638460287604085 0.000000000001846983075754557174");
	m_poly.push_back("56750.336102615744 3.9058182804383450 -0.000396554042183492022 0.000000000001846589978027920983");
	m_poly.push_back("56750.336108402778 3.9056200223644431 -0.000396469783268461580 0.000000000001846197621582724828");
	m_poly.push_back("56750.336114189813 3.9054218064489921 -0.000396385456718277584 0.000000000001845804950185900059");
	m_poly.push_back("56750.336119976862 3.9052236326788350 -0.000396301127468016501 0.000000000001845412266214961030");
	m_poly.push_back("56750.336125763890 3.9050255044955140 -0.000396216761304860315 0.000000000001845019410353653933");
	m_poly.push_back("56750.336131550917 3.9048274150759150 -0.000396132459170833512 0.000000000001844626852650717828");
	m_poly.push_back("56750.336137337959 3.9046293678247399 -0.000396048091124728721 0.000000000001844233988019849076");
	m_poly.push_back("56750.336143125001 3.9044313627546088 -0.000395963733807877408 0.000000000001843841173350060050");
	m_poly.push_back("56750.336148912043 3.9042334032854300 -0.000395879413999004318 0.000000000001843448533339086992");
	m_poly.push_back("56750.336154699071 3.9040354825890340 -0.000395794917645949013 0.000000000001843055071229716028");
	m_poly.push_back("56750.336160486113 3.9038376040777218 -0.000395710589292753122 0.000000000001842662391429752035");
	m_poly.push_back("56750.336166273148 3.9036397677599179 -0.000395626206778143215 0.000000000001842269459420148972");
	m_poly.push_back("56750.336172060182 3.9034419770658708 -0.000395541736395565904 0.000000000001841876118242142814");
	m_poly.push_back("56750.336177847217 3.9032442251697090 -0.000395457389701386116 0.000000000001841483353032612006");
	m_poly.push_back("56750.336183634259 3.9030465154679210 -0.000395372936984773307 0.000000000001841090094116750838");
	m_poly.push_back("56750.336189421287 3.9028488479879662 -0.000395288517237522114 0.000000000001840696988725541890");
	m_poly.push_back("56750.336195208343 3.9026512261374768 -0.000395204079557851483 0.000000000001840303799828679017");
	m_poly.push_back("56750.336200995371 3.9024536431186601 -0.000395119603597483392 0.000000000001839910432672940919");
	m_poly.push_back("56750.336206782413 3.9022561023252100 -0.000395035186701300485 0.000000000001839517340555346183");
	m_poly.push_back("56750.336212569448 3.9020586037441101 -0.000394950691005500789 0.000000000001839123881497539011");
	m_poly.push_back("56750.336218356482 3.9018611508237262 -0.000394866201011837611 0.000000000001838730448991080818");
	m_poly.push_back("56750.336224143517 3.9016637367256490 -0.000394781645017277789 0.000000000001838336709143486201");
	m_poly.push_back("56750.336229930559 3.9014663649078072 -0.000394697188152849426 0.000000000001837943430905633062");
	m_poly.push_back("56750.336235717587 3.9012690353128132 -0.000394612656601092422 0.000000000001837549804876900853");
	m_poly.push_back("56750.336241504629 3.9010717513699751 -0.000394528218877012490 0.000000000001837156615766030846");
	m_poly.push_back("56750.336247291663 3.9008745063120052 -0.000394443672421540884 0.000000000001836762920334664200");
	m_poly.push_back("56750.336253078713 3.9006773034897790 -0.000394359115432215974 0.000000000001836369175850317141");
	m_poly.push_back("56750.336258865740 3.9004801429439260 -0.000394274609984164321 0.000000000001835975671372823057");
	m_poly.push_back("56750.336264652768 3.9002830280834599 -0.000394190023169646423 0.000000000001835581788001891798");
	m_poly.push_back("56750.336270439817 3.9000859520861670 -0.000394105484547950709 0.000000000001835188129045774007");
	m_poly.push_back("56750.336276226852 3.8998889183757610 -0.000394020951903728393 0.000000000001834794497923530965");
	m_poly.push_back("56750.336282013886 3.8996919269442918 -0.000393936310226131788 0.000000000001834400359074169037");
	m_poly.push_back("56750.336287800928 3.8994949812261841 -0.000393851758548142722 0.000000000001834006639316935189");
	m_poly.push_back("56750.336293587963 3.8992980743756882 -0.000393767161608335708 0.000000000001833612708792333197");
	m_poly.push_back("56750.336299374998 3.8991012098316480 -0.000393682575024745284 0.000000000001833218826491719125");
	m_poly.push_back("56750.336305162040 3.8989043875798890 -0.000393597914615364326 0.000000000001832824600412072956");
	m_poly.push_back("56750.336310949067 3.8987076110575369 -0.000393513354712798425 0.000000000001832430842352343838");
	m_poly.push_back("56750.336316736109 3.8985108734262641 -0.000393428685377175723 0.000000000001832036574704837864");
	m_poly.push_back("56750.336322523151 3.8983141781061552 -0.000393344050316544178 0.000000000001831642466661793116");
	m_poly.push_back("56750.336328310193 3.8981175251234199 -0.000393259481132463484 0.000000000001831248665379437859");
	m_poly.push_back("56750.336334097221 3.8979209178569278 -0.000393174814763372913 0.000000000001830854411542810003");
	m_poly.push_back("56750.336339884263 3.8977243495041711 -0.000393090145818550884 0.000000000001830460145711492962");
	m_poly.push_back("56750.336345671298 3.8975278234959689 -0.000393005402692665003 0.000000000001830065534446774925");
	m_poly.push_back("56750.336351458333 3.8973313398263341 -0.000392920764034891397 0.000000000001829671409648707848");
	m_poly.push_back("56750.336357245367 3.8971349018812012 -0.000392836112063251017 0.000000000001829277222851810001");
	m_poly.push_back("56750.336363032409 3.8969385028935992 -0.000392751378547507625 0.000000000001828882656335090043");
	m_poly.push_back("56750.336368819437 3.8967421462528362 -0.000392666731700684699 0.000000000001828488493401076149");
	m_poly.push_back("56750.336374606479 3.8965458319425110 -0.000392581981318341610 0.000000000001828093848341321057");
	m_poly.push_back("56750.336380393521 3.8963495634059742 -0.000392497316983190625 0.000000000001827699603968717813");
	m_poly.push_back("56750.336386180563 3.8961533338357661 -0.000392412553127107996 0.000000000001827304896165162913");
	m_poly.push_back("56750.336391967590 3.8959571466146921 -0.000392327836495727923 0.000000000001826910408267859818");
	m_poly.push_back("56750.336397754632 3.8957610017571871 -0.000392243148339502484 0.000000000001826516052967423931");
	m_poly.push_back("56750.336403541667 3.8955649026777910 -0.000392158375939096891 0.000000000001826121305373140936");
	m_poly.push_back("56750.336409328702 3.8953688425765169 -0.000392073554866250113 0.000000000001825726331129643039");
	m_poly.push_back("56750.336415115737 3.8951728248466209 -0.000391988929264770075 0.000000000001825332267119722085");
	m_poly.push_back("56750.336420902779 3.8949768494936530 -0.000391904044162793587 0.000000000001824936994715542906");
	m_poly.push_back("56750.336426689813 3.8947809199465850 -0.000391819290858327722 0.000000000001824542336040047176");
	m_poly.push_back("56750.336432476848 3.8945850293780269 -0.000391734471242101412 0.000000000001824147368575842891");
	m_poly.push_back("56750.336438263890 3.8943891812080391 -0.000391649734146979690 0.000000000001823752785379571949");
	m_poly.push_back("56750.336444050918 3.8941933754277258 -0.000391564876480983285 0.000000000001823357640730506896");
	m_poly.push_back("56750.336449837960 3.8939976154572000 -0.000391480113157810527 0.000000000001822962935398143978");
	m_poly.push_back("56750.336455625002 3.8938018944936950 -0.000391395282172838808 0.000000000001822567914990469960");
	m_poly.push_back("56750.336461412036 3.8936062159414462 -0.000391310413955225694 0.000000000001822172721203890916");
	m_poly.push_back("56750.336467199071 3.8934105797853089 -0.000391225665136085026 0.000000000001821778083408870960");
	m_poly.push_back("56750.336472986113 3.8932149894636652 -0.000391140858123178797 0.000000000001821383174626680904");
	m_poly.push_back("56750.336478773148 3.8930194381485399 -0.000391055973598521481 0.000000000001820987904901789812");
	m_poly.push_back("56750.336484560183 3.8928239292623492 -0.000390971120826414677 0.000000000001820592783035329816");
	m_poly.push_back("56750.336490347217 3.8926284628055150 -0.000390886252626565294 0.000000000001820197589326937112");
	m_poly.push_back("56750.336496134260 3.8924330421831370 -0.000390801329323827785 0.000000000001819802139024430056");
	m_poly.push_back("56750.336501921287 3.8922376605919302 -0.000390716580202704990 0.000000000001819407499817754087");
	m_poly.push_back("56750.336507708344 3.8920423214442690 -0.000390631595792960819 0.000000000001819011764962148076");
	m_poly.push_back("56750.336513495371 3.8918470247338321 -0.000390546758199905716 0.000000000001818616713774119970");
	m_poly.push_back("56750.336519282413 3.8916517738625789 -0.000390461832019454514 0.000000000001818221250067573967");
	m_poly.push_back("56750.336525069448 3.8914565620571331 -0.000390376939634375412 0.000000000001817825943732487057");
	m_poly.push_back("56750.336530856483 3.8912613926928761 -0.000390291990387805587 0.000000000001817430372614542053");
	m_poly.push_back("56750.336536643517 3.8910662657882411 -0.000390207075345172104 0.000000000001817034960770180981");
	m_poly.push_back("56750.336542430552 3.8908711847438830 -0.000390122195208139925 0.000000000001816639711466377904");
	m_poly.push_back("56750.336548217587 3.8906761427571048 -0.000390037270519836491 0.000000000001816244254704050909");
	m_poly.push_back("56750.336554004629 3.8904811432493140 -0.000389952308110391092 0.000000000001815848622288060085");
	m_poly.push_back("56750.336559791664 3.8902861862041211 -0.000389867397526437616 0.000000000001815453231202414043");
	m_poly.push_back("56750.336565578713 3.8900912750466108 -0.000389782440259573077 0.000000000001815057622731216028");
	m_poly.push_back("56750.336571365740 3.8898964029525351 -0.000389697453789243696 0.000000000001814661878270280096");
	m_poly.push_back("56750.336577152782 3.8897015733274412 -0.000389612509356626819 0.000000000001814266329561788999");
	m_poly.push_back("56750.336582939817 3.8895067862076460 -0.000389527491338028883 0.000000000001813870438190963065");
	m_poly.push_back("56750.336588726852 3.8893120449755720 -0.000389442564891122691 0.000000000001813474973232744969");
	m_poly.push_back("56750.336594513887 3.8891173428353611 -0.000389357512268905795 0.000000000001813078920724445959");
	m_poly.push_back("56750.336600300929 3.8889226831874981 -0.000389272629396269284 0.000000000001812683658673319911");
	m_poly.push_back("56750.336606087963 3.8887280660296710 -0.000389187564232905121 0.000000000001812287547763935060");
	m_poly.push_back("56750.336611874998 3.8885334947786951 -0.000389102497497456591 0.000000000001811891429532659034");
	m_poly.push_back("56750.336617662040 3.8883389626315519 -0.000389017570180130777 0.000000000001811495960517094992");
	m_poly.push_back("56750.336623449068 3.8881444729821300 -0.000388932499430093997 0.000000000001811099823590014027");
	m_poly.push_back("56750.336629236110 3.8879500258664552 -0.000388847526522322811 0.000000000001810704142275576931");
	m_poly.push_back("56750.336635023137 3.8877556246548850 -0.000388762480961162514 0.000000000001810308122641010055");
	m_poly.push_back("56750.336640810187 3.8875612625484881 -0.000388677425763214214 0.000000000001809912058131259013");
	m_poly.push_back("56750.336646597221 3.8873669429702762 -0.000388592382227111773 0.000000000001809516047925300050");
	m_poly.push_back("56750.336652384263 3.8871726659302430 -0.000388507233103786286 0.000000000001809119546039610095");
	m_poly.push_back("56750.336658171298 3.8869784348265801 -0.000388422237316414921 0.000000000001808723758178895058");
	m_poly.push_back("56750.336663958333 3.8867842428288770 -0.000388337197146526906 0.000000000001808327763645367016");
	m_poly.push_back("56750.336669745368 3.8865900933656641 -0.000388252110178264091 0.000000000001807931551189015017");
	m_poly.push_back("56750.336675532410 3.8863959864488460 -0.000388167045458991577 0.000000000001807535442336689005");
	m_poly.push_back("56750.336681319437 3.8862019254709539 -0.000388081893253670114 0.000000000001807138926094370954");
	m_poly.push_back("56750.336687106479 3.8860079036445350 -0.000387996847594809812 0.000000000001806742905997186963");
	m_poly.push_back("56750.336692893521 3.8858139243558911 -0.000387911710080039100 0.000000000001806346458161678056");
	m_poly.push_back("56750.336698668980 3.8856203754392569 -0.000387826873090948521 0.000000000001805951409747789966");
	m_poly.push_back("56750.336704456022 3.8854264845711151 -0.000387741787730297481 0.000000000001805555204771616091");
	m_poly.push_back("56750.336710243057 3.8852326328678570 -0.000387656597457442518 0.000000000001805158511259406093");
	m_poly.push_back("56750.336716030091 3.8850388237096962 -0.000387571507646993526 0.000000000001804762285560819027");
	m_poly.push_back("56750.336721817133 3.8848450571190680 -0.000387486337038515406 0.000000000001804365683616181086");
	m_poly.push_back("56750.336727604168 3.8846513364864581 -0.000387401267922735809 0.000000000001803969554282434977");
	m_poly.push_back("56750.336733391203 3.8844576550283740 -0.000387316100058743805 0.000000000001803572965115987012");
	m_poly.push_back("56750.336739178238 3.8842640161471009 -0.000387230944220752927 0.000000000001803176431949059056");
	m_poly.push_back("56750.336744965280 3.8840704198282912 -0.000387145778639391821 0.000000000001802779853410118953");
	m_poly.push_back("56750.336750752307 3.8838768695045118 -0.000387060606361129273 0.000000000001802383243684972915");
	m_poly.push_back("56750.336756539349 3.8836833583527790 -0.000386975432013399677 0.000000000001801986624322628086");
	m_poly.push_back("56750.336762326391 3.8834898897647632 -0.000386890289817903214 0.000000000001801590154679875983");
	m_poly.push_back("56750.336768113433 3.8832964637825982 -0.000386805097742036193 0.000000000001801193452762637938");
	m_poly.push_back("56750.336773900461 3.8831030838015090 -0.000386719919273136503 0.000000000001800796814206478994");
	m_poly.push_back("56750.336779687503 3.8829097429952202 -0.000386634724097509883 0.000000000001800400097853039936");
	m_poly.push_back("56750.336785474537 3.8827164447772509 -0.000386549589416043779 0.000000000001800003663196577051");
	m_poly.push_back("56750.336791261572 3.8825231891651590 -0.000386464325024297615 0.000000000001799606624528643077");
	m_poly.push_back("56750.336797048607 3.8823299795601671 -0.000386379195858449313 0.000000000001799210215554052084");
	m_poly.push_back("56750.336802835649 3.8821368091534429 -0.000386293997174499016 0.000000000001798813482860065942");
	m_poly.push_back("56750.336808622676 3.8819436813664998 -0.000386208751896456094 0.000000000001798416533194516980");
	m_poly.push_back("56750.336814409718 3.8817505961498382 -0.000386123509860281913 0.000000000001798019598624184998");
	m_poly.push_back("56750.336820196761 3.8815575569857912 -0.000386038323610896983 0.000000000001797622923830028997");
	m_poly.push_back("56750.336825983803 3.8813645570243578 -0.000385953018922277288 0.000000000001797225697509691935");
	m_poly.push_back("56750.336831770830 3.8811715996811911 -0.000385867811471958500 0.000000000001796828923989566079");
	m_poly.push_back("56750.336837557872 3.8809786849424110 -0.000385782590077162683 0.000000000001796432085534499961");
	m_poly.push_back("56750.336843344907 3.8807858162544360 -0.000385697312113976703 0.000000000001796034983661000073");
	m_poly.push_back("56750.336849131942 3.8805929867707358 -0.000385612015569997406 0.000000000001795637795263454915");
	m_poly.push_back("56750.336854918984 3.8804001999212918 -0.000385526784494004393 0.000000000001795240911724067077");
	m_poly.push_back("56750.336860706018 3.8802074557028918 -0.000385441537510649690 0.000000000001794843954109342923");
	m_poly.push_back("56750.336866493053 3.8800147575259492 -0.000385356286939034709 0.000000000001794446979784180991");
	m_poly.push_back("56750.336872280102 3.8798220985675189 -0.000385270988814089097 0.000000000001794049784021044035");
	m_poly.push_back("56750.336878067130 3.8796294822629882 -0.000385185702350989507 0.000000000001793652642561696937");
	m_poly.push_back("56750.336883854157 3.8794369086084548 -0.000385100433767018488 0.000000000001793255584357619059");
	m_poly.push_back("56750.336889641207 3.8792443809863428 -0.000385015086605476316 0.000000000001792858160246893949");
	m_poly.push_back("56750.336895428241 3.8790518926003701 -0.000384929748467875785 0.000000000001792460778156667086");
	m_poly.push_back("56750.336901215283 3.8788594468828892 -0.000384844526815507088 0.000000000001792063938492158101");
	m_poly.push_back("56750.336907002318 3.8786670438173889 -0.000384759206548153506 0.000000000001791666639614929025");
	m_poly.push_back("56750.336912789353 3.8784746868178259 -0.000384673917971177786 0.000000000001791269488306204038");
	m_poly.push_back("56750.336918576388 3.8782823690508339 -0.000384588578252611211 0.000000000001790872098850382953");
	m_poly.push_back("56750.336924363422 3.8780900939524399 -0.000384503275544640125 0.000000000001790474881737665919");
	m_poly.push_back("56750.336930150457 3.8778978615168769 -0.000384417964914074602 0.000000000001790077627731581032");
	m_poly.push_back("56750.336935937499 3.8777056751438161 -0.000384332646769478881 0.000000000001789680338734253940");
	m_poly.push_back("56750.336941724527 3.8775135280346391 -0.000384247274783215220 0.000000000001789282799016611929");
	m_poly.push_back("56750.336947511583 3.8773214236036160 -0.000384161920409625303 0.000000000001788885341313459940");
	m_poly.push_back("56750.336953298611 3.8771293618430791 -0.000384076571267434575 0.000000000001788487907969998076");
	m_poly.push_back("56750.336959085653 3.8769373461586540 -0.000383991230571866426 0.000000000001788090513957873922");
	m_poly.push_back("56750.336964872688 3.8767453697437988 -0.000383905850361025984 0.000000000001787692935937912942");
	m_poly.push_back("56750.336970659722 3.8765534360245129 -0.000383820517428179507 0.000000000001787295578072354922");
	m_poly.push_back("56750.336976446757 3.8763615449744209 -0.000383735149358803874 0.000000000001786898056588670965");
	m_poly.push_back("56750.336982233799 3.8761697000169200 -0.000383649741854092817 0.000000000001786500351468586992");
	m_poly.push_back("56750.336988020827 3.8759778943458532 -0.000383564387933476225 0.000000000001786102895868371065");
	m_poly.push_back("56750.336993807869 3.8757861313531570 -0.000383479002953091919 0.000000000001785705295633929069");
	m_poly.push_back("56750.336999594911 3.8755944110565101 -0.000383393569033811112 0.000000000001785307467509078979");
	m_poly.push_back("56750.337005381953 3.8754027368755111 -0.000383308218550463924 0.000000000001784910027911731018");
	m_poly.push_back("56750.337011168980 3.8752111019548452 -0.000383222785901284979 0.000000000001784512205699429006");
	m_poly.push_back("56750.337016956008 3.8750195097526561 -0.000383137448065667009 0.000000000001784114824996262053");
	m_poly.push_back("56750.337022743057 3.8748279602491351 -0.000383052084126343523 0.000000000001783717322737331917");
	m_poly.push_back("56750.337028530092 3.8746364568466238 -0.000382966665501578100 0.000000000001783319565828166918");
	m_poly.push_back("56750.337034317126 3.8744449927492242 -0.000382881277821116816 0.000000000001782921953014122918");
	m_poly.push_back("56750.337040104168 3.8742535713671118 -0.000382795841681377925 0.000000000001782524114543069988");
	m_poly.push_back("56750.337045891203 3.8740621926943808 -0.000382710362775616323 0.000000000001782126076926291961");
	m_poly.push_back("56750.337051678238 3.8738708601319489 -0.000382624943102794727 0.000000000001781728315133068924");
	m_poly.push_back("56750.337057465280 3.8736795668684731 -0.000382539519895003721 0.000000000001781330536878352920");
	m_poly.push_back("56750.337063252307 3.8734883163482210 -0.000382454139462117589 0.000000000001780932957808896036");
	m_poly.push_back("56750.337069039349 3.8732971085231722 -0.000382368647437889497 0.000000000001780534859100846945");
	m_poly.push_back("56750.337074826391 3.8731059468315912 -0.000382283216387440598 0.000000000001780137044322770912");
	m_poly.push_back("56750.337080613433 3.8729148244690750 -0.000382197741087703005 0.000000000001779739023492371942");
	m_poly.push_back("56750.337086400461 3.8727237448088512 -0.000382112316467700722 0.000000000001779341238656968068");
	m_poly.push_back("56750.337092187503 3.8725327078871028 -0.000382026779625746276 0.000000000001778942931246465062");
	m_poly.push_back("56750.337097974538 3.8723417171106580 -0.000381941385932284510 0.000000000001778545290421861059");
	m_poly.push_back("56750.337103761573 3.8721507656592640 -0.000381855916663311021 0.000000000001778147297670742909");
	m_poly.push_back("56750.337109548607 3.8719598569315581 -0.000381770415011176601 0.000000000001777749154122867988");
	m_poly.push_back("56750.337115335649 3.8717689909362361 -0.000381684969856377907 0.000000000001777351273660300041");
	m_poly.push_back("56750.337121122677 3.8715781710815889 -0.000381599468204243378 0.000000000001776953130110086961");
	m_poly.push_back("56750.337126909719 3.8713873905710519 -0.000381513970353533302 0.000000000001776555004261233039");
	m_poly.push_back("56750.337132696761 3.8711966528150952 -0.000381428543237736215 0.000000000001776157207796572058");
	m_poly.push_back("56750.337138483803 3.8710059577857798 -0.000381342994689526881 0.000000000001775758845867262924");
	m_poly.push_back("56750.337144270830 3.8708153088950281 -0.000381257536025461077 0.000000000001775360902492293908");
	m_poly.push_back("56750.337150057872 3.8706246993763349 -0.000381172008917995075 0.000000000001774962640402399913");
	m_poly.push_back("56750.337155844907 3.8704341325744229 -0.000381086525642371796 0.000000000001774564582419513916");
	m_poly.push_back("56750.337161631942 3.8702436085530949 -0.000381001001785656482 0.000000000001774166335465279085");
	m_poly.push_back("56750.337167418977 3.8700531306879720 -0.000380915530909071379 0.000000000001773768335217686989");
	m_poly.push_back("56750.337173206019 3.8698626921590180 -0.000380830047811084706 0.000000000001773370278059246012");
	m_poly.push_back("56750.337178993053 3.8696722963988961 -0.000380744553104543312 0.000000000001772972166843343949");
	m_poly.push_back("56750.337184780103 3.8694819433881880 -0.000380658965112116391 0.000000000001772573621230273054");
	m_poly.push_back("56750.337190567130 3.8692916365501642 -0.000380573433092997110 0.000000000001772175336261900012");
	m_poly.push_back("56750.337196354158 3.8691013690616911 -0.000380487926102881385 0.000000000001771777167843406949");
	m_poly.push_back("56750.337202141207 3.8689111443376838 -0.000380402355287918288 0.000000000001771378702216393018");
	m_poly.push_back("56750.337207928242 3.8687209623949852 -0.000380316864986765708 0.000000000001770980611510110924");
	m_poly.push_back("56750.337213715276 3.8685308266093532 -0.000380231329938941483 0.000000000001770582312434486969");
	m_poly.push_back("56750.337219502311 3.8683407301951420 -0.000380145765616597925 0.000000000001770183877038345949");
	m_poly.push_back("56750.337225289353 3.8681506765491980 -0.000380060207538182376 0.000000000001769785470716846962");
	m_poly.push_back("56750.337231076388 3.8679606656884520 -0.000379974710975337893 0.000000000001769387350848634076");
	m_poly.push_back("56750.337236863423 3.8677707010072981 -0.000379889148926743395 0.000000000001768988926037431005");
	m_poly.push_back("56750.337242650457 3.8675807756811240 -0.000379803586220893289 0.000000000001768590498165018078");
	m_poly.push_back("56750.337248437500 3.8673908931479950 -0.000379717924287211882 0.000000000001768191608226092962");
	m_poly.push_back("56750.337254224527 3.8672010533878942 -0.000379632408086636017 0.000000000001767793396908910089");
	m_poly.push_back("56750.337260011584 3.8670112598285549 -0.000379546880632778479 0.000000000001767395133188225073");
	m_poly.push_back("56750.337265798611 3.8668215056423909 -0.000379461348640304714 0.000000000001766996848332457040");
	m_poly.push_back("56750.337271585653 3.8666317942336290 -0.000379375787337784100 0.000000000001766598426990233980");
	m_poly.push_back("56750.337277372688 3.8664421256306221 -0.000379290170452756573 0.000000000001766199746820878075");
	m_poly.push_back("56750.337283159723 3.8662525032014150 -0.000379204572912359816 0.000000000001765801156730644917");
	m_poly.push_back("56750.337288946757 3.8660629201672161 -0.000379119016201746515 0.000000000001765402756768411928");
	m_poly.push_back("56750.337294733799 3.8658733799077978 -0.000379033363114369981 0.000000000001765003908015989092");
	m_poly.push_back("56750.337300520827 3.8656838824736450 -0.000378947814237533005 0.000000000001764605544530789980");
	m_poly.push_back("56750.337306307869 3.8654944312301200 -0.000378862231378805214 0.000000000001764207022803777021");
	m_poly.push_back("56750.337312094911 3.8653050193627099 -0.000378776652605720325 0.000000000001763808520101497088");
	m_poly.push_back("56750.337317881953 3.8651156502948498 -0.000378691091685118275 0.000000000001763410100530403024");
	m_poly.push_back("56750.337323668980 3.8649263240392129 -0.000378605413755256793 0.000000000001763011136091569939");
	m_poly.push_back("56750.337329456022 3.8647370439834292 -0.000378519834760126107 0.000000000001762612632352160008");
	m_poly.push_back("56750.337335243057 3.8645478033430218 -0.000378434233382777814 0.000000000001762214024386915906");
	m_poly.push_back("56750.337341030092 3.8643586054739072 -0.000378348606221470026 0.000000000001761815296354838973");
	m_poly.push_back("56750.337346817127 3.8641694504516662 -0.000378263014338800702 0.000000000001761416732600809076");
	m_poly.push_back("56750.337352604169 3.8639803416183338 -0.000378177428691177386 0.000000000001761018197879650003");
	m_poly.push_back("56750.337358391203 3.8637912722000589 -0.000378091755557505391 0.000000000001760619255769325062");
	m_poly.push_back("56750.337364178238 3.8636022455789711 -0.000378006104131031718 0.000000000001760220414740104035");
	m_poly.push_back("56750.337369965280 3.8634132617586601 -0.000377920479598746310 0.000000000001759821698945792075");
	m_poly.push_back("56750.337375752308 3.8632243241844710 -0.000377834871213632284 0.000000000001759423058341296096");
	m_poly.push_back("56750.337381539350 3.8630354260071722 -0.000377749319077162618 0.000000000001759024679664440964");
	m_poly.push_back("56750.337387326392 3.8628465706465249 -0.000377663645979017923 0.000000000001758625737714978014");
	m_poly.push_back("56750.337393113427 3.8626577581114589 -0.000377577945462656713 0.000000000001758226668088578998");
	m_poly.push_back("56750.337398900461 3.8624689917951951 -0.000377492348108778508 0.000000000001757828078848611980");
	m_poly.push_back("56750.337404687503 3.8622802649024992 -0.000377406714952234024 0.000000000001757429322888973007");
	m_poly.push_back("56750.337410474538 3.8620915808186620 -0.000377321045184776291 0.000000000001757030396445569027");
	m_poly.push_back("56750.337416261573 3.8619029395450202 -0.000377235382673775821 0.000000000001756631503791761007");
	m_poly.push_back("56750.337422048608 3.8617143445257400 -0.000377149761161313520 0.000000000001756232802050952001");
	m_poly.push_back("56750.337427835650 3.8615257889122461 -0.000377064083266979175 0.000000000001755833837761083925");
	m_poly.push_back("56750.337433622677 3.8613372761259930 -0.000376978458379420719 0.000000000001755435120302333918");
	m_poly.push_back("56750.337439409719 3.8611488061776820 -0.000376892823633028081 0.000000000001755036356933895077");
	m_poly.push_back("56750.337445196761 3.8609603824703198 -0.000376807123471940191 0.000000000001754637288953311032");
	m_poly.push_back("56750.337450983803 3.8607719981688118 -0.000376721462479732896 0.000000000001754238403366443027");
	m_poly.push_back("56750.337456770831 3.8605836567033949 -0.000376635800963497623 0.000000000001753839515338464060");
	m_poly.push_back("56750.337462557873 3.8603953580669792 -0.000376550119054575082 0.000000000001753440532348718958");
	m_poly.push_back("56750.337468344907 3.8602071056854830 -0.000376464497151312325 0.000000000001753041828780691002");
	m_poly.push_back("56750.337474131942 3.8600188927235419 -0.000376378801031457982 0.000000000001752642779614306967");
	m_poly.push_back("56750.337479918977 3.8598307226045701 -0.000376293060768896788 0.000000000001752243524891510930");
	m_poly.push_back("56750.337485706019 3.8596425953291691 -0.000376207438643588179 0.000000000001751844820287249020");
	m_poly.push_back("56750.337491493046 3.8594545142866492 -0.000376121799456279374 0.000000000001751446036230538989");
	m_poly.push_back("56750.337497280103 3.8592664726765680 -0.000376036064540581378 0.000000000001751046806403267995");
	m_poly.push_back("56750.337503067130 3.8590784739116919 -0.000375950400146632304 0.000000000001750647904967564927");
	m_poly.push_back("56750.337508854172 3.8588905179954862 -0.000375864769095082113 0.000000000001750249158793588923");
	m_poly.push_back("56750.337514641200 3.8587026083153431 -0.000375779066544781209 0.000000000001749850079676298081");
	m_poly.push_back("56750.337520428242 3.8585147380897671 -0.000375693375567507889 0.000000000001749451054449597964");
	m_poly.push_back("56750.337526215277 3.8583269106996778 -0.000375607653725867083 0.000000000001749051885498570050");
	m_poly.push_back("56750.337532002311 3.8581391261591449 -0.000375521888958330475 0.000000000001748652516657348025");
	m_poly.push_back("56750.337537789354 3.8579513878700009 -0.000375436276345464187 0.000000000001748253856340195960");
	m_poly.push_back("56750.337543576388 3.8577636890122542 -0.000375350500839792209 0.000000000001747854437493802008");
	m_poly.push_back("56750.337549363423 3.8575760330335891 -0.000375264840771295620 0.000000000001747455556192730029");
	m_poly.push_back("56750.337555150472 3.8573884198633288 -0.000375179150868723988 0.000000000001747056535965004057");
	m_poly.push_back("56750.337560937500 3.8572008529789170 -0.000375093424523994092 0.000000000001746657346038949950");
	m_poly.push_back("56750.337566724527 3.8570133255465668 -0.000375007785478794701 0.000000000001746258562632486073");
	m_poly.push_back("56750.337572511577 3.8568258409507150 -0.000374922042240819809 0.000000000001745859294039686073");
	m_poly.push_back("56750.337578298611 3.8566383992076290 -0.000374836327406944574 0.000000000001745460157712867909");
	m_poly.push_back("56750.337584085653 3.8564510037466828 -0.000374750640106750023 0.000000000001745061149598404919");
	m_poly.push_back("56750.337589872688 3.8562636477152279 -0.000374664934447807682 0.000000000001744662055993864952");
	m_poly.push_back("56750.337595659723 3.8560763345507740 -0.000374579147715499710 0.000000000001744262584861130026");
	m_poly.push_back("56750.337601446758 3.8558890642502228 -0.000374493470398484481 0.000000000001743863623232140018");
	m_poly.push_back("56750.337607233800 3.8557018402056960 -0.000374407726574308580 0.000000000001743464351903729938");
	m_poly.push_back("56750.337613020827 3.8555146556129269 -0.000374322004110939520 0.000000000001743065180043814973");
	m_poly.push_back("56750.337618807869 3.8553275138798240 -0.000374236345205962895 0.000000000001742666304149812026");
	m_poly.push_back("56750.337624594897 3.8551404150176820 -0.000374150628755594425 0.000000000001742267160288280989");
	m_poly.push_back("56750.337630381953 3.8549533624176169 -0.000374064893786605289 0.000000000001741867930191368906");
	m_poly.push_back("56750.337636168981 3.8547663492659749 -0.000373979146516278526 0.000000000001741468642811389995");
	m_poly.push_back("56750.337641956023 3.8545793789970628 -0.000373893382272770004 0.000000000001741069276392958982");
	m_poly.push_back("56750.337647743057 3.8543924515821151 -0.000373807668256023414 0.000000000001740670143860198972");
	m_poly.push_back("56750.337653530092 3.8542055704565898 -0.000373721985885245819 0.000000000001740271158689157996");
	m_poly.push_back("56750.337659317127 3.8540187287613672 -0.000373636173688724209 0.000000000001739871568969361022");
	m_poly.push_back("56750.337665104169 3.8538319299404331 -0.000373550482631514506 0.000000000001739472543347526057");
	m_poly.push_back("56750.337670891196 3.8536451740045510 -0.000373464761074092312 0.000000000001739073375697093041");
	m_poly.push_back("56750.337676678238 3.8534584643304650 -0.000373379005419315622 0.000000000001738674049267157981");
	m_poly.push_back("56750.337682465281 3.8532717941211359 -0.000373293211554896124 0.000000000001738274544909197926");
	m_poly.push_back("56750.337688252323 3.8530851667616468 -0.000373207511713553983 0.000000000001737875478379394972");
	m_poly.push_back("56750.337694039350 3.8528985822987978 -0.000373121758057189687 0.000000000001737476161252949082");
	m_poly.push_back("56750.337699826392 3.8527120440995319 -0.000373036019340067400 0.000000000001737076913691455062");
	m_poly.push_back("56750.337705613427 3.8525255453789211 -0.000372950313130451620 0.000000000001736677817504317071");
	m_poly.push_back("56750.337711400462 3.8523390895095191 -0.000372864557173692586 0.000000000001736278489663074964");
	m_poly.push_back("56750.337717187504 3.8521526765264289 -0.000372778780442327721 0.000000000001735879065081619923");
	m_poly.push_back("56750.337722974538 3.8519663098386792 -0.000372693120356067510 0.000000000001735480183669932933");
	m_poly.push_back("56750.337728761573 3.8517799825972250 -0.000372607273387172227 0.000000000001735080432017656082");
	m_poly.push_back("56750.337734548608 3.8515936982456611 -0.000372521543711755313 0.000000000001734681226554784044");
	m_poly.push_back("56750.337740335650 3.8514074567482290 -0.000372435791520893216 0.000000000001734281916245285043");
	m_poly.push_back("56750.337746122677 3.8512212615533810 -0.000372350040182687118 0.000000000001733882609904960917");
	m_poly.push_back("56750.337751909719 3.8510351058053640 -0.000372264328182116491 0.000000000001733483486744257010");
	m_poly.push_back("56750.337757696761 3.8508489929469452 -0.000372178507405744889 0.000000000001733083857055841056");
	m_poly.push_back("56750.337763483803 3.8506629229745708 -0.000372092772658801714 0.000000000001732684627972252083");
	m_poly.push_back("56750.337769270831 3.8504768992738301 -0.000372007046500590316 0.000000000001732285438881743088");
	m_poly.push_back("56750.337775057873 3.8502909150566760 -0.000371921236409063108 0.000000000001731885858945761054");
	m_poly.push_back("56750.337780844908 3.8501049737109758 -0.000371835501111446277 0.000000000001731486627295148921");
	m_poly.push_back("56750.337786631942 3.8499190752495971 -0.000371749800293102724 0.000000000001731087556200248046");
	m_poly.push_back("56750.337792418977 3.8497332230646721 -0.000371663987448207398 0.000000000001730687963439742006");
	m_poly.push_back("56750.337798206019 3.8495474103545719 -0.000371578244965188121 0.000000000001730288698326743017");
	m_poly.push_back("56750.337803993047 3.8493616405268130 -0.000371492514623633814 0.000000000001729889489750930022");
	m_poly.push_back("56750.337809780103 3.8491759135865888 -0.000371406735787274602 0.000000000001729490055352666926");
	m_poly.push_back("56750.337815567131 3.8489902329334069 -0.000371320958629581819 0.000000000001729090628769983092");
	m_poly.push_back("56750.337821354173 3.8488045917484190 -0.000371235199200026299 0.000000000001728691284739858960");
	m_poly.push_back("56750.337827141200 3.8486189934390400 -0.000371149422664061824 0.000000000001728291861050894036");
	m_poly.push_back("56750.337832928242 3.8484334380387479 -0.000371063702634314915 0.000000000001727892700488592086");
	m_poly.push_back("56750.337838715277 3.8482479288972988 -0.000370977896646194124 0.000000000001727493139650101044");
	m_poly.push_back("56750.337844502312 3.8480624592593440 -0.000370892126904831423 0.000000000001727093747598216073");
	m_poly.push_back("56750.337850289347 3.8478770324723031 -0.000370806345617086383 0.000000000001726694301778383028");
	m_poly.push_back("56750.337856076389 3.8476916485814039 -0.000370720575502686577 0.000000000001726294907987575992");
	m_poly.push_back("56750.337861863423 3.8475063110090808 -0.000370634805850142315 0.000000000001725895516346127095");
	m_poly.push_back("56750.337867650473 3.8473210128884232 -0.000370549057878150588 0.000000000001725496225662109926");
	m_poly.push_back("56750.337873437500 3.8471357576518068 -0.000370463297365011596 0.000000000001725096876577908011");
	m_poly.push_back("56750.337879224528 3.8469505453018820 -0.000370377527668057979 0.000000000001724697484727319945");
	m_poly.push_back("56750.337885011577 3.8467653792449759 -0.000370291777804236026 0.000000000001724298185230634026");
	m_poly.push_back("56750.337890798612 3.8465802526748618 -0.000370205993292385975 0.000000000001723898724390974972");
	m_poly.push_back("56750.337896585646 3.8463951689865210 -0.000370120275598560886 0.000000000001723499574696074975");
	m_poly.push_back("56750.337902372688 3.8462101281745911 -0.000370034415466790082 0.000000000001723099761721886044");
	m_poly.push_back("56750.337908159723 3.8460251336594600 -0.000369948661819307517 0.000000000001722700444602360925");
	m_poly.push_back("56750.337913946758 3.8458401786284542 -0.000369862857776308090 0.000000000001722300892810033990");
	m_poly.push_back("56750.337919733793 3.8456552664766912 -0.000369777140793029586 0.000000000001721901746420207034");
	m_poly.push_back("56750.337925520827 3.8454703972184170 -0.000369691335275646013 0.000000000001721502187760410973");
	m_poly.push_back("56750.337931307869 3.8452855742536500 -0.000369605550364113505 0.000000000001721102725052892066");
	m_poly.push_back("56750.337937094897 3.8451007907615899 -0.000369519779690158313 0.000000000001720703328643618951");
	m_poly.push_back("56750.337942881953 3.8449160501600850 -0.000369434005969734608 0.000000000001720303918047204946");
	m_poly.push_back("56750.337948668981 3.8447313524508711 -0.000369348245589823192 0.000000000001719904569571475958");
	m_poly.push_back("56750.337954456023 3.8445467010272281 -0.000369262477456072105 0.000000000001719505184987776001");
	m_poly.push_back("56750.337960243058 3.8443620890804731 -0.000369176744396676926 0.000000000001719105963731280985");
	m_poly.push_back("56750.337966030093 3.8441775200270900 -0.000369090989843247493 0.000000000001718706642384472974");
	m_poly.push_back("56750.337971817127 3.8439929938532060 -0.000369005196636083604 0.000000000001718307141041270001");
	m_poly.push_back("56750.337977604169 3.8438085139934768 -0.000368919358557902495 0.000000000001717907430749801934");
	m_poly.push_back("56750.337983391197 3.8436240735950360 -0.000368833585050642695 0.000000000001717508021139593005");
	m_poly.push_back("56750.337989178239 3.8434396760819340 -0.000368747843828843712 0.000000000001717108761869360039");
	m_poly.push_back("56750.337994965281 3.8432553214683720 -0.000368662048667676675 0.000000000001716709251423454074");
	m_poly.push_back("56750.338000752323 3.8430710131521981 -0.000368576271136946895 0.000000000001716309823074334928");
	m_poly.push_back("56750.338006539350 3.8428867442994630 -0.000368490537926560681 0.000000000001715910601107399000");
	m_poly.push_back("56750.338012326392 3.8427025183483519 -0.000368404736450410927 0.000000000001715511061252312990");
	m_poly.push_back("56750.338018113427 3.8425183352719898 -0.000368318909714329820 0.000000000001715111403770590084");
	m_poly.push_back("56750.338023900462 3.8423341984982708 -0.000368233162683812313 0.000000000001714712117445622085");
	m_poly.push_back("56750.338029687497 3.8421501011971788 -0.000368147425707529108 0.000000000001714312877938696084");
	m_poly.push_back("56750.338035474539 3.8419660467746071 -0.000368061647093215693 0.000000000001713913444538665907");
	m_poly.push_back("56750.338041261573 3.8417820352546039 -0.000367975879705538490 0.000000000001713514063415816932");
	m_poly.push_back("56750.338047048608 3.8415980700303440 -0.000367890096943409520 0.000000000001713114610698652046");
	m_poly.push_back("56750.338052835650 3.8414141442781968 -0.000367804319057406421 0.000000000001712715180687331029");
	m_poly.push_back("56750.338058622678 3.8412302614074050 -0.000367718571929188910 0.000000000001712315893902192923");
	m_poly.push_back("56750.338064409720 3.8410464214346720 -0.000367632827936258079 0.000000000001711916621715954947");
	m_poly.push_back("56750.338070196762 3.8408626277438591 -0.000367547043930672491 0.000000000001711517163204967039");
	m_poly.push_back("56750.338075983796 3.8406788735400861 -0.000367461271942206083 0.000000000001711117760652525024");
	m_poly.push_back("56750.338081770831 3.8404951622141832 -0.000367375450970434307 0.000000000001710718130002873903");
	m_poly.push_back("56750.338087557873 3.8403114937750109 -0.000367289703122788318 0.000000000001710318839863053932");
	m_poly.push_back("56750.338093344908 3.8401278716335732 -0.000367203912198255219 0.000000000001709919349129535058");
	m_poly.push_back("56750.338099131943 3.8399442889670530 -0.000367118100507998182 0.000000000001709519761697556080");
	m_poly.push_back("56750.338104918977 3.8397607491767221 -0.000367032418083929285 0.000000000001709120776207256911");
	m_poly.push_back("56750.338110706020 3.8395772522757250 -0.000366946619494375008 0.000000000001708721249778346017");
	m_poly.push_back("56750.338116493047 3.8393938016649112 -0.000366860825816474009 0.000000000001708321746219773024");
	m_poly.push_back("56750.338122280089 3.8392103905352428 -0.000366775062283512018 0.000000000001707922383034008072");
	m_poly.push_back("56750.338128067131 3.8390270222861211 -0.000366689336702619791 0.000000000001707523196575422997");
	m_poly.push_back("56750.338133854173 3.8388436969241040 -0.000366603550894022199 0.000000000001707123729658813920");
	m_poly.push_back("56750.338139641201 3.8386604178470791 -0.000366517810453824509 0.000000000001706724474003965066");
	m_poly.push_back("56750.338145428243 3.8384771782464870 -0.000366432078211557921 0.000000000001706325256523190078");
	m_poly.push_back("56750.338151215277 3.8382939815337660 -0.000366346247514179595 0.000000000001705925580574155934");
	m_poly.push_back("56750.338157002312 3.8381108277026721 -0.000366260566511204084 0.000000000001705526601693089030");
	m_poly.push_back("56750.338162789347 3.8379277201461659 -0.000366174721149920305 0.000000000001705126857457681097");
	m_poly.push_back("56750.338168576389 3.8377446520790439 -0.000366089011414217117 0.000000000001704727744777591023");
	m_poly.push_back("56750.338174363424 3.8375616268735651 -0.000366003188843715295 0.000000000001704328106668608912");
	m_poly.push_back("56750.338180150473 3.8373786445616660 -0.000365917462809849580 0.000000000001703928918092453989");
	m_poly.push_back("56750.338185937500 3.8371957085377622 -0.000365831716827388410 0.000000000001703529636622094904");
	m_poly.push_back("56750.338191724542 3.8370128119831950 -0.000365745938932503527 0.000000000001703130206547417022");
	m_poly.push_back("56750.338197511577 3.8368299583169390 -0.000365660241045164805 0.000000000001702731149035991085");
	m_poly.push_back("56750.338203298612 3.8366471475128039 -0.000365574437455138811 0.000000000001702331599307154036");
	m_poly.push_back("56750.338209085647 3.8364643830052358 -0.000365488680929942499 0.000000000001701932268739684977");
	m_poly.push_back("56750.338214872689 3.8362816579601811 -0.000365402946023126178 0.000000000001701533038840134028");
	m_poly.push_back("56750.338220659723 3.8360989758037651 -0.000365317176077481411 0.000000000001701133645777407085");
	m_poly.push_back("56750.338226446758 3.8359163365175459 -0.000365231406531518884 0.000000000001700734254574928965");
	m_poly.push_back("56750.338232233793 3.8357337435170389 -0.000365145677397893679 0.000000000001700335051555743941");
	m_poly.push_back("56750.338238020828 3.8355511899714059 -0.000365059905809109783 0.000000000001699935650838805002");
	m_poly.push_back("56750.338243807870 3.8353686793113999 -0.000364974172598723622 0.000000000001699536428834289911");
	m_poly.push_back("56750.338249594897 3.8351862115182729 -0.000364888399233573074 0.000000000001699137019843666928");
	m_poly.push_back("56750.338255381947 3.8350037900140852 -0.000364802639004652802 0.000000000001698737672022061907");
	m_poly.push_back("56750.338261168981 3.8348214079789140 -0.000364716940069257690 0.000000000001698338609620163041");
	m_poly.push_back("56750.338266956023 3.8346390688002772 -0.000364631132899853204 0.000000000001697939043213465922");
	m_poly.push_back("56750.338272743058 3.8344567725118708 -0.000364545445919404619 0.000000000001697540036479291916");
	m_poly.push_back("56750.338278530093 3.8342745224893191 -0.000364459725845248409 0.000000000001697140875639209954");
	m_poly.push_back("56750.338284317128 3.8340923119387269 -0.000364373966415692889 0.000000000001696741531535760986");
	m_poly.push_back("56750.338290104170 3.8339101442573291 -0.000364288174727322393 0.000000000001696342037214581072");
	m_poly.push_back("56750.338295891197 3.8337280194418728 -0.000364202469690108523 0.000000000001695942946393376033");
	m_poly.push_back("56750.338301678239 3.8335459409134800 -0.000364116768525373489 0.000000000001695543873603489994");
	m_poly.push_back("56750.338307465281 3.8333639018287280 -0.000364030969260799818 0.000000000001695144344000116989");
	m_poly.push_back("56750.338313252323 3.8331819056093268 -0.000363945297290647679 0.000000000001694745407156669920");
	m_poly.push_back("56750.338319039351 3.8329999522710572 -0.000363859549993675400 0.000000000001694346119544465100");
	m_poly.push_back("56750.338324826393 3.8328180452061660 -0.000363773823862109625 0.000000000001693946930490024089");
	m_poly.push_back("56750.338330613427 3.8326361775932298 -0.000363688019526009693 0.000000000001693547377266852005");
	m_poly.push_back("56750.338336400462 3.8324543528429809 -0.000363602293865180975 0.000000000001693148190403043951");
	m_poly.push_back("56750.338342187497 3.8322725709552201 -0.000363516593828438605 0.000000000001692749122859769051");
	m_poly.push_back("56750.338347974539 3.8320908353486929 -0.000363430903552830085 0.000000000001692350100769000907");
	m_poly.push_back("56750.338353761566 3.8319091391882361 -0.000363345185263922311 0.000000000001691950948230625953");
	m_poly.push_back("56750.338359548608 3.8317274858903421 -0.000363259452453218500 0.000000000001691551728068952945");
	m_poly.push_back("56750.338365335650 3.8315458754489602 -0.000363173695146421004 0.000000000001691152393837538051");
	m_poly.push_back("56750.338371122692 3.8313643112839508 -0.000363088046304560518 0.000000000001690753564684034978");
	m_poly.push_back("56750.338376909720 3.8311827865669188 -0.000363002300579672499 0.000000000001690354284383256974");
	m_poly.push_back("56750.338382696762 3.8310013047002309 -0.000362916604877264290 0.000000000001689955237016887019");
	m_poly.push_back("56750.338388483797 3.8308198657039330 -0.000362830870467830812 0.000000000001689556009405967918");
	m_poly.push_back("56750.338394270831 3.8306384729667848 -0.000362745174214749002 0.000000000001689156959473080079");
	m_poly.push_back("56750.338400057874 3.8304571196728769 -0.000362659392145402778 0.000000000001688757509926566942");
	m_poly.push_back("56750.338405844908 3.8302758092413769 -0.000362573727049789120 0.000000000001688358605080526042");
	m_poly.push_back("56750.338411631943 3.8300945416503680 -0.000362487988359312186 0.000000000001687959357530781985");
	m_poly.push_back("56750.338417418978 3.8299133203393700 -0.000362402328956953098 0.000000000001687560479193797024");
	m_poly.push_back("56750.338423206020 3.8297321384604039 -0.000362316616778745920 0.000000000001687161355099596975");
	m_poly.push_back("56750.338428993047 3.8295509994298600 -0.000362230907789116508 0.000000000001686762245852454068");
	m_poly.push_back("56750.338434780089 3.8293699032591482 -0.000362145198488623076 0.000000000001686363135156821982");
	m_poly.push_back("56750.338440567131 3.8291888533341059 -0.000362059509731807786 0.000000000001685964120123824992");
	m_poly.push_back("56750.338446354173 3.8290078428596650 -0.000361973841110106612 0.000000000001685565198851749023");
	m_poly.push_back("56750.338452141201 3.8288268752245611 -0.000361888194035730722 0.000000000001685166377916308963");
	m_poly.push_back("56750.338457928243 3.8286459504350931 -0.000361802377984494009 0.000000000001684766770120128004");
	m_poly.push_back("56750.338463715278 3.8284650719144500 -0.000361716747607962886 0.000000000001684368026937856066");
	m_poly.push_back("56750.338469502312 3.8282842328090529 -0.000361631005000597979 0.000000000001683968761139522996");
	m_poly.push_back("56750.338475289347 3.8281034365560171 -0.000361545354551125880 0.000000000001683569924483902933");
	m_poly.push_back("56750.338481076389 3.8279226831499291 -0.000361459725906552217 0.000000000001683171189364338949");
	m_poly.push_back("56750.338486863417 3.8277419760024061 -0.000361374047639181092 0.000000000001682772223169183924");
	m_poly.push_back("56750.338492650473 3.8275613082614171 -0.000361288303175513205 0.000000000001682372948723103054");
	m_poly.push_back("56750.338498437501 3.8273806833654298 -0.000361202651855621376 0.000000000001681974108010614048");
	m_poly.push_back("56750.338504224543 3.8272001013265879 -0.000361117012641667219 0.000000000001681575323669873902");
	m_poly.push_back("56750.338510011577 3.8270195655130541 -0.000361031327890558393 0.000000000001681176327279037089");
	m_poly.push_back("56750.338515798612 3.8268390691426348 -0.000360945637535013491 0.000000000001680777304789876063");
	m_poly.push_back("56750.338521585647 3.8266586155840780 -0.000360859963735203778 0.000000000001680378359393449921");
	m_poly.push_back("56750.338527372682 3.8264782048821289 -0.000360774310177090405 0.000000000001679979508253852096");
	m_poly.push_back("56750.338533159716 3.8262978404133858 -0.000360688587918003519 0.000000000001679580337199080934");
	m_poly.push_back("56750.338538946758 3.8261175153645990 -0.000360602961405069610 0.000000000001679181611996583091");
	m_poly.push_back("56750.338544733793 3.8259372331408450 -0.000360517247512668812 0.000000000001678782479900819004");
#endif /* 0 */

#ifdef ENABLE_LBCSIIFSERVICEWORKERTEST
	// derotateWithLBT_27.txt - 1200 entries - 20min
	// time, position, velocity, acceleration/2
	m_poly.push_back("GetRotatorPolynomials result status: OK 0 0");
	m_poly.push_back("56750.354175532397 3.4767503161088369 -0.000175047096517531387 0.000000000000815128515456615868");
	m_poly.push_back("56750.354187106481 3.4765753143863791 -0.000174951635551291810 0.000000000000814683995381571148");
	m_poly.push_back("56750.354198680558 3.4764004080323829 -0.000174856374115416497 0.000000000000814240404440179008");
	m_poly.push_back("56750.354210254627 3.4762256023584310 -0.000174760982116605395 0.000000000000813796205515090776");
	m_poly.push_back("56750.354221828697 3.4760508865933439 -0.000174665729003007112 0.000000000000813352653323303001");
	m_poly.push_back("56750.354233402781 3.4758762695993708 -0.000174570606256944309 0.000000000000812909708200215870");
	m_poly.push_back("56750.354244976850 3.4757017441878402 -0.000174475554778704501 0.000000000000812467094942221929");
	m_poly.push_back("56750.354256550927 3.4755273174370922 -0.000174380428781891091 0.000000000000812024134677668911");
	m_poly.push_back("56750.354268124996 3.4753529821515872 -0.000174285406418297795 0.000000000000811581656991305298");
	m_poly.push_back("56750.354279699073 3.4751787454148140 -0.000174190492706160008 0.000000000000811139685250275869");
	m_poly.push_back("56750.354291273150 3.4750046000364092 -0.000174095543155828588 0.000000000000810697546623082070");
	m_poly.push_back("56750.354302847220 3.4748305530892760 -0.000174000705738630812 0.000000000000810255930154066767");
	m_poly.push_back("56750.354314421304 3.4746565973791719 -0.000173905950496146196 0.000000000000809814696339295989");
	m_poly.push_back("56750.354325995373 3.4744827399980411 -0.000173811243091210796 0.000000000000809373685283061569");
	m_poly.push_back("56750.354337569443 3.4743089737418500 -0.000173716514805087504 0.000000000000808932576989637197");
	m_poly.push_back("56750.354349143519 3.4741353056919082 -0.000173621972362424787 0.000000000000808492334094840889");
	m_poly.push_back("56750.354360717589 3.4739617286604112 -0.000173527435337679812 0.000000000000808052116427625288");
	m_poly.push_back("56750.354372291673 3.4737882497181629 -0.000173432859357218011 0.000000000000807611717356291152");
	m_poly.push_back("56750.354383865742 3.4736148616837870 -0.000173338451794061702 0.000000000000807172102537593001");
	m_poly.push_back("56750.354395439812 3.4734415716266760 -0.000173244057331608099 0.000000000000806732548721397276");
	m_poly.push_back("56750.354407013889 3.4732683723582838 -0.000173149701842634999 0.000000000000806293176388033105");
	m_poly.push_back("56750.354418587973 3.4730952709713141 -0.000173055515534805007 0.000000000000805854591863352353");
	m_poly.push_back("56750.354430162028 3.4729222602529868 -0.000172961312884403404 0.000000000000805415931236074537");
	m_poly.push_back("56750.354441736112 3.4727493472843358 -0.000172867065345221012 0.000000000000804977061576712137");
	m_poly.push_back("56750.354453310189 3.4725765248856861 -0.000172773014236643312 0.000000000000804539106617074435");
	m_poly.push_back("56750.354464884258 3.4724038001398472 -0.000172678976264295807 0.000000000000804101212825370939");
	m_poly.push_back("56750.354476458328 3.4722311658405092 -0.000172584994709347591 0.000000000000803663581746095593");
	m_poly.push_back("56750.354488032397 3.4720586290675159 -0.000172491045111232091 0.000000000000803226099475025131");
	m_poly.push_back("56750.354499606481 3.4718861826424061 -0.000172397131075973503 0.000000000000802788782804847478");
	m_poly.push_back("56750.354511180558 3.4717138336358002 -0.000172303357480248399 0.000000000000802352120104806955");
	m_poly.push_back("56750.354522754627 3.4715415748541378 -0.000172209522102497294 0.000000000000801915169708237564");
	m_poly.push_back("56750.354534328697 3.4713694133866859 -0.000172115843888770090 0.000000000000801478951161285075");
	m_poly.push_back("56750.354545902781 3.4711973420273972 -0.000172022193759396901 0.000000000000801042863390748033");
	m_poly.push_back("56750.354557476850 3.4710253678715079 -0.000171928597480573302 0.000000000000800607026378967594");
	m_poly.push_back("56750.354569050927 3.4708534837090692 -0.000171835036826779403 0.000000000000800171355257621350");
	m_poly.push_back("56750.354580624997 3.4706816966380520 -0.000171741613503877713 0.000000000000799736323630699413");
	m_poly.push_back("56750.354592199074 3.4705099994554871 -0.000171648115733456910 0.000000000000799300945328939233");
	m_poly.push_back("56750.354603773150 3.4703383992510828 -0.000171554761564502112 0.000000000000798866235721208533");
	m_poly.push_back("56750.354615347220 3.4701668888228432 -0.000171461458927938705 0.000000000000798431766078346140");
	m_poly.push_back("56750.354626921297 3.4699954752578730 -0.000171368182270734913 0.000000000000797997417408929854");
	m_poly.push_back("56750.354638495373 3.4698241513515651 -0.000171274934381786205 0.000000000000797563202700585480");
	m_poly.push_back("56750.354650069443 3.4696529242124732 -0.000171181904257603705 0.000000000000797130002036180664");
	m_poly.push_back("56750.354661643520 3.4694817866146579 -0.000171088703735466994 0.000000000000796696007892818809");
	m_poly.push_back("56750.354673217589 3.4693107456601400 -0.000170995623011485512 0.000000000000796262571600598939");
	m_poly.push_back("56750.354684791673 3.4691397941484081 -0.000170902657502633513 0.000000000000795829671819204148");
	m_poly.push_back("56750.354696365743 3.4689689391741050 -0.000170809718044195809 0.000000000000795396893342120943");
	m_poly.push_back("56750.354707939812 3.4687981735285689 -0.000170716874465138711 0.000000000000794964561336663754");
	m_poly.push_back("56750.354719513889 3.4686275042965851 -0.000170624039776795706 0.000000000000794532270729443150");
	m_poly.push_back("56750.354731087973 3.4684569242864809 -0.000170531281800838900 0.000000000000794100337340575388");
	m_poly.push_back("56750.354742662043 3.4682864405851652 -0.000170438646660623890 0.000000000000793668975947828827");
	m_poly.push_back("56750.354754236112 3.4681160459966160 -0.000170345950679857207 0.000000000000793237331242655677");
	m_poly.push_back("56750.354765810182 3.4679457476035180 -0.000170253372481069491 0.000000000000792806235000074175");
	m_poly.push_back("56750.354777384258 3.4677755382029392 -0.000170160763631078304 0.000000000000792374996025291885");
	m_poly.push_back("56750.354788958342 3.4676054248923500 -0.000170068342045640887 0.000000000000791944629066786933");
	m_poly.push_back("56750.354800532397 3.4674354004661438 -0.000169975915903823512 0.000000000000791514240889343226");
	m_poly.push_back("56750.354812106481 3.4672654720192351 -0.000169883541614143413 0.000000000000791084094164856763");
	m_poly.push_back("56750.354823680558 3.4670956323448490 -0.000169791221139485793 0.000000000000790654198034578747");
	m_poly.push_back("56750.354835254628 3.4669258885415259 -0.000169698958450029602 0.000000000000790224570985257041");
	m_poly.push_back("56750.354846828697 3.4667562334017341 -0.000169606720496476488 0.000000000000789795059119931207");
	m_poly.push_back("56750.354858402781 3.4665866740208169 -0.000169514614695707397 0.000000000000789366162636593554");
	m_poly.push_back("56750.354869976851 3.4664172031881759 -0.000169422491237855506 0.000000000000788937183929542499");
	m_poly.push_back("56750.354881550928 3.4662478280055389 -0.000169330418264338807 0.000000000000788508440306139492");
	m_poly.push_back("56750.354893124997 3.4660785412548218 -0.000169238392684279511 0.000000000000788079917374201806");
	m_poly.push_back("56750.354904699067 3.4659093500582241 -0.000169146524019552993 0.000000000000787652125133860074");
	m_poly.push_back("56750.354916273151 3.4657402471821119 -0.000169054635326294393 0.000000000000787224239626888831");
	m_poly.push_back("56750.354927847220 3.4655712397390390 -0.000168962832280545412 0.000000000000786796752944860604");
	m_poly.push_back("56750.354939421297 3.4654023205120930 -0.000168871059290917109 0.000000000000786369406221037138");
	m_poly.push_back("56750.354950995366 3.4652334966095828 -0.000168779322770092505 0.000000000000785942229315960909");
	m_poly.push_back("56750.354962569443 3.4650647608168490 -0.000168687670822075913 0.000000000000785515446232202375");
	m_poly.push_back("56750.354974143520 3.4648961202269488 -0.000168596030153986292 0.000000000000785088715672333781");
	m_poly.push_back("56750.354985717589 3.4647275676430329 -0.000168504529898784802 0.000000000000784662638959433891");
	m_poly.push_back("56750.354997291673 3.4645591101571500 -0.000168413015707988202 0.000000000000784236497351629704");
	m_poly.push_back("56750.355008865743 3.4643907405564942 -0.000168321563992000593 0.000000000000783810646663205654");
	m_poly.push_back("56750.355020439813 3.4642224659528469 -0.000168230236595020709 0.000000000000783385374877827093");
	m_poly.push_back("56750.355032013889 3.4640542791235780 -0.000168138900742536407 0.000000000000782960063717036595");
	m_poly.push_back("56750.355043587973 3.4638861871861990 -0.000168047578763474292 0.000000000000782534817157038814");
	m_poly.push_back("56750.355055162043 3.4637181829083579 -0.000167956346206700490 0.000000000000782109987000540813");
	m_poly.push_back("56750.355066736112 3.4635502734064141 -0.000167865242489787208 0.000000000000781685756799062486");
	m_poly.push_back("56750.355078310182 3.4633824514588878 -0.000167774071617339212 0.000000000000781261213878752196");
	m_poly.push_back("56750.355089884259 3.4632147241882421 -0.000167683066879565810 0.000000000000780837444580973679");
	m_poly.push_back("56750.355101458343 3.4630470843572900 -0.000167592060853926706 0.000000000000780413669284602530");
	m_poly.push_back("56750.355113032398 3.4628795390845779 -0.000167501122445677507 0.000000000000779990208853898642");
	m_poly.push_back("56750.355124606482 3.4627120811464618 -0.000167410177873436600 0.000000000000779566719718383840");
	m_poly.push_back("56750.355136180558 3.4625447176647151 -0.000167319399462515802 0.000000000000779144004329464070");
	m_poly.push_back("56750.355147754628 3.4623774414055148 -0.000167228549632780890 0.000000000000778720956369298271");
	m_poly.push_back("56750.355159328697 3.4622102594930140 -0.000167137897246179599 0.000000000000778298827823050658");
	m_poly.push_back("56750.355170902767 3.4620431646942520 -0.000167047170909442010 0.000000000000777876354918147055");
	m_poly.push_back("56750.355182476851 3.4618761641298770 -0.000166956560747071801 0.000000000000777454422989978894");
	m_poly.push_back("56750.355194050928 3.4617092505760318 -0.000166865983660645405 0.000000000000777032645082181428");
	m_poly.push_back("56750.355205624997 3.4615424311486769 -0.000166775484556707287 0.000000000000776611230306307751");
	m_poly.push_back("56750.355217199067 3.4613756986180402 -0.000166685060211152193 0.000000000000776190163649808355");
	m_poly.push_back("56750.355228773151 3.4612090601062140 -0.000166594650378511298 0.000000000000775769164572013376");
	m_poly.push_back("56750.355240347220 3.4610425083798400 -0.000166504296838923901 0.000000000000775348427627689791");
	m_poly.push_back("56750.355251921297 3.4608760505683311 -0.000166414033529871706 0.000000000000774928110849703725");
	m_poly.push_back("56750.355263495367 3.4607096794420191 -0.000166323782557684492 0.000000000000774507851518256258");
	m_poly.push_back("56750.355275069443 3.4605434021013899 -0.000166233521104935196 0.000000000000774087543380570639");
	m_poly.push_back("56750.355286643520 3.4603772113464388 -0.000166143459511177994 0.000000000000773668165907362450");
	m_poly.push_back("56750.355298217590 3.4602111142803369 -0.000166053346615957010 0.000000000000773248549540775247");
	m_poly.push_back("56750.355309791667 3.4600451036835538 -0.000165963322521296111 0.000000000000772829346682550732");
	m_poly.push_back("56750.355321365743 3.4598791866682088 -0.000165873362331418994 0.000000000000772410441401849408");
	m_poly.push_back("56750.355332939813 3.4597133560070370 -0.000165783492851696108 0.000000000000771991958521749411");
	m_poly.push_back("56750.355344513890 3.4595476188271981 -0.000165693641579729489 0.000000000000771573560425688659");
	m_poly.push_back("56750.355356087974 3.4593819678908599 -0.000165603819120427093 0.000000000000771155296497702979");
	m_poly.push_back("56750.355367662043 3.4592164103303520 -0.000165514041327859796 0.000000000000770737240563218563");
	m_poly.push_back("56750.355379236113 3.4590509389087560 -0.000165424380144869610 0.000000000000770319727632807884");
	m_poly.push_back("56750.355390810182 3.4588855607385658 -0.000165334625196375112 0.000000000000769901778070440408");
	m_poly.push_back("56750.355402384259 3.4587202686154792 -0.000165245140992776896 0.000000000000769485089260772657");
	m_poly.push_back("56750.355413958343 3.4585550696385750 -0.000165155587537531593 0.000000000000769068077970663340");
	m_poly.push_back("56750.355425532412 3.4583899565869931 -0.000165066085152822600 0.000000000000768651304494833113");
	m_poly.push_back("56750.355437106482 3.4582249365834139 -0.000164976672083819804 0.000000000000768234946925373479");
	m_poly.push_back("56750.355448680559 3.4580600024031152 -0.000164887287765308498 0.000000000000767818723234356956");
	m_poly.push_back("56750.355460254628 3.4578951611520341 -0.000164797944196644103 0.000000000000767402689297308756");
	m_poly.push_back("56750.355471828712 3.4577304056123341 -0.000164708657285188309 0.000000000000766986919189483407");
	m_poly.push_back("56750.355483402767 3.4575657429049542 -0.000164619388172924702 0.000000000000766571231963308617");
	m_poly.push_back("56750.355494976851 3.4574011658077430 -0.000164530253220739189 0.000000000000766156169467223397");
	m_poly.push_back("56750.355506550928 3.4572366814257309 -0.000164441124077271901 0.000000000000765741134017789520");
	m_poly.push_back("56750.355518124998 3.4570722825462892 -0.000164352186426110702 0.000000000000765326990272445880");
	m_poly.push_back("56750.355529699067 3.4569079762819590 -0.000164263160782636089 0.000000000000764912436778670953");
	m_poly.push_back("56750.355541273151 3.4567437554091551 -0.000164174153622254705 0.000000000000764497969352053657");
	m_poly.push_back("56750.355552847221 3.4565796270403260 -0.000164085353213167304 0.000000000000764084464683884108");
	m_poly.push_back("56750.355564421297 3.4564155839535178 -0.000163996411298726488 0.000000000000763670301078614426");
	m_poly.push_back("56750.355575995367 3.4562516332735629 -0.000163907700063236088 0.000000000000763257211653616748");
	m_poly.push_back("56750.355587569436 3.4560877677644042 -0.000163818926370700509 0.000000000000762843831388901719");
	m_poly.push_back("56750.355599143521 3.4559239945630580 -0.000163730293854890487 0.000000000000762431108527266340");
	m_poly.push_back("56750.355610717590 3.4557603064159141 -0.000163641624701521890 0.000000000000762018215057252286");
	m_poly.push_back("56750.355622291667 3.4555967104670851 -0.000163553087319018509 0.000000000000761605935190847772");
	m_poly.push_back("56750.355633865744 3.4554331994772278 -0.000163464578500487903 0.000000000000761193788334378938");
	m_poly.push_back("56750.355645439813 3.4552697805720758 -0.000163376018780176106 0.000000000000760781404445739269");
	m_poly.push_back("56750.355657013890 3.4551064465186752 -0.000163287630141719292 0.000000000000760369817217628646");
	m_poly.push_back("56750.355668587959 3.4549432044545871 -0.000163199306189647903 0.000000000000759958531206686801");
	m_poly.push_back("56750.355680162043 3.4547800471260750 -0.000163110956764481093 0.000000000000759547126575970601");
	m_poly.push_back("56750.355691736113 3.4546169816755992 -0.000163022726435804711 0.000000000000759136276529212398");
	m_poly.push_back("56750.355703310182 3.4544540008642621 -0.000162934525621457189 0.000000000000758725563917839069");
	m_poly.push_back("56750.355714884259 3.4542911118203512 -0.000162846406981822801 0.000000000000758315233960027882");
	m_poly.push_back("56750.355726458343 3.4541283073094520 -0.000162758313246846501 0.000000000000757905019972125429");
	m_poly.push_back("56750.355738032413 3.4539655944638130 -0.000162670227976256402 0.000000000000757494845397321613");
	m_poly.push_back("56750.355749606482 3.4538029660415859 -0.000162582175950364910 0.000000000000757084825628778232");
	m_poly.push_back("56750.355761180559 3.4536404291779208 -0.000162494261743866307 0.000000000000756675447629525195");
	m_poly.push_back("56750.355772754629 3.4534779766330450 -0.000162406391422501402 0.000000000000756266273984922300");
	m_poly.push_back("56750.355784328713 3.4533156155413089 -0.000162318550144728492 0.000000000000755857235582808994");
	m_poly.push_back("56750.355795902768 3.4531533386671440 -0.000162230877737234508 0.000000000000755448983542791475");
	m_poly.push_back("56750.355807476852 3.4529911531348811 -0.000162143108295721792 0.000000000000755040279650769012");
	m_poly.push_back("56750.355819050928 3.4528290517131039 -0.000162055364913505208 0.000000000000754631697105256363");
	m_poly.push_back("56750.355830624998 3.4526670415303800 -0.000161967723059514609 0.000000000000754223587334982054");
	m_poly.push_back("56750.355842199067 3.4525051153454580 -0.000161880175912502489 0.000000000000753815918577164973");
	m_poly.push_back("56750.355853773151 3.4523432803088481 -0.000161792727051847504 0.000000000000753408707498596195");
	m_poly.push_back("56750.355865347221 3.4521815291608018 -0.000161705206026304494 0.000000000000753001160374668657");
	m_poly.push_back("56750.355876921298 3.4520198690493791 -0.000161617839367008103 0.000000000000752594332072334763");
	m_poly.push_back("56750.355888495367 3.4518582927214498 -0.000161530466605892901 0.000000000000752187475354775670");
	m_poly.push_back("56750.355900069437 3.4516968073299932 -0.000161443160245356609 0.000000000000751780927836760054");
	m_poly.push_back("56750.355911643521 3.4515354056119971 -0.000161355886578845104 0.000000000000751374532560738311");
	m_poly.push_back("56750.355923217590 3.4513740947239122 -0.000161268689749065591 0.000000000000750968495081378315");
	m_poly.push_back("56750.355934791667 3.4512128674104510 -0.000161181528331151710 0.000000000000750562622499955434");
	m_poly.push_back("56750.355946365737 3.4510517308162649 -0.000161094462259708403 0.000000000000750157193907880125");
	m_poly.push_back("56750.355957939813 3.4508906777008841 -0.000161007456851180189 0.000000000000749752047798109614");
	m_poly.push_back("56750.355969513890 3.4507297151971912 -0.000160920423154015894 0.000000000000749346769956863668");
	m_poly.push_back("56750.355981087960 3.4505688360577378 -0.000160833510711627106 0.000000000000748942056750758516");
	m_poly.push_back("56750.355992662036 3.4504080474303271 -0.000160746645174195205 0.000000000000748537561960625094");
	m_poly.push_back("56750.356004236113 3.4502473420616040 -0.000160659759288485203 0.000000000000748132972415040456");
	m_poly.push_back("56750.356015810183 3.4500867271025899 -0.000160573010804722089 0.000000000000747729022694878791");
	m_poly.push_back("56750.356027384260 3.4499261953017331 -0.000160486197288182313 0.000000000000747324770140856254");
	m_poly.push_back("56750.356038958344 3.4497657538081312 -0.000160399580060138113 0.000000000000746921431624006534");
	m_poly.push_back("56750.356050532413 3.4496053953637000 -0.000160313012659173387 0.000000000000746518325131296319");
	m_poly.push_back("56750.356062106483 3.4494451271073769 -0.000160226359743926590 0.000000000000746114820429552463");
	m_poly.push_back("56750.356073680552 3.4492849418083540 -0.000160139832852999409 0.000000000000745711902572765709");
	m_poly.push_back("56750.356085254629 3.4491248466032758 -0.000160053459155916990 0.000000000000745309698078171469");
	m_poly.push_back("56750.356096828713 3.4489648342347450 -0.000159966993611805998 0.000000000000744907065886125052");
	m_poly.push_back("56750.356108402782 3.4488049118674962 -0.000159880707480708993 0.000000000000744505269148790438");
	m_poly.push_back("56750.356119976852 3.4486450722355459 -0.000159794381461302807 0.000000000000744103286665651340");
	m_poly.push_back("56750.356131550929 3.4484853224789140 -0.000159708032873160407 0.000000000000743701199086396015");
	m_poly.push_back("56750.356143124998 3.4483256553686430 -0.000159621819031297111 0.000000000000743299738967022207");
	m_poly.push_back("56750.356154699068 3.4481660780383110 -0.000159535683997932387 0.000000000000742898645826092697");
	m_poly.push_back("56750.356166273137 3.4480065832352160 -0.000159449612789415088 0.000000000000742497849891374169");
	m_poly.push_back("56750.356177847221 3.4478471781134452 -0.000159363559007053007 0.000000000000742097135101273060");
	m_poly.push_back("56750.356189421298 3.4476878554184061 -0.000159277498723189604 0.000000000000741696390034822583");
	m_poly.push_back("56750.356200995368 3.4475286223024679 -0.000159191619344488009 0.000000000000741296487371456591");
	m_poly.push_back("56750.356212569437 3.4473694715087628 -0.000159105716393403301 0.000000000000740896474939237554");
	m_poly.push_back("56750.356224143521 3.4472104101805199 -0.000159019851749661595 0.000000000000740496640887254023");
	m_poly.push_back("56750.356235717591 3.4470514310784308 -0.000158934096983068095 0.000000000000740097318489238091");
	m_poly.push_back("56750.356247291667 3.4468925413401390 -0.000158848340067071299 0.000000000000739697986080064222");
	m_poly.push_back("56750.356258865737 3.4467337337199409 -0.000158762698659304410 0.000000000000739299191546617464");
	m_poly.push_back("56750.356270439806 3.4465750153650512 -0.000158677035855203408 0.000000000000738900297376606162");
	m_poly.push_back("56750.356282013890 3.4464163790237952 -0.000158591460697024312 0.000000000000738501811338244590");
	m_poly.push_back("56750.356293587960 3.4462578318449082 -0.000158505865972168513 0.000000000000738103234183119043");
	m_poly.push_back("56750.356305162037 3.4460993665746922 -0.000158420458431929886 0.000000000000737705528673023955");
	m_poly.push_back("56750.356316736114 3.4459409903623510 -0.000158334986995789290 0.000000000000737307525622217435");
	m_poly.push_back("56750.356328310183 3.4457826959609048 -0.000158249596970524102 0.000000000000736909901668984389");
	m_poly.push_back("56750.356339884260 3.4456244905053208 -0.000158164241522232388 0.000000000000736512438725130372");
	m_poly.push_back("56750.356351458337 3.4454663667610710 -0.000158078975016610093 0.000000000000736115389951472187");
	m_poly.push_back("56750.356363032413 3.4453083318660380 -0.000157993771811807109 0.000000000000735718635943005809");
	m_poly.push_back("56750.356374606483 3.4451503785738291 -0.000157908567399074997 0.000000000000735321876308319236");
	m_poly.push_back("56750.356386180552 3.4449925140329558 -0.000157823461237172503 0.000000000000734925574187495128");
	m_poly.push_back("56750.356397754629 3.4448347309897280 -0.000157738388071277410 0.000000000000734529425714944131");
	m_poly.push_back("56750.356409328713 3.4446770365916910 -0.000157653342350244212 0.000000000000734133405040280434");
	m_poly.push_back("56750.356420902783 3.4445194235933152 -0.000157568403850691488 0.000000000000733737883653182277");
	m_poly.push_back("56750.356432476852 3.4443618991425620 -0.000157483508836590087 0.000000000000733342564758792003");
	m_poly.push_back("56750.356444050929 3.4442044559727520 -0.000157398596005532904 0.000000000000732947162896374206");
	m_poly.push_back("56750.356455624998 3.4440471012581062 -0.000157313754078143694 0.000000000000732552091202667153");
	m_poly.push_back("56750.356467199083 3.4438898277257448 -0.000157228991376699110 0.000000000000732157388432174176");
	m_poly.push_back("56750.356478773137 3.4437326425427832 -0.000157144355010438295 0.000000000000731763273953533097");
	m_poly.push_back("56750.356490347222 3.4435755384416740 -0.000157059663265952606 0.000000000000731368901598570673");
	m_poly.push_back("56750.356501921298 3.4434185225843099 -0.000156975033747584595 0.000000000000730974819004361592");
	m_poly.push_back("56750.356513495368 3.4432615877039829 -0.000156890437278514692 0.000000000000730580890306573948");
	m_poly.push_back("56750.356525069437 3.4431047409708202 -0.000156806000469263709 0.000000000000730187705080540623");
	m_poly.push_back("56750.356536643521 3.4429479751142500 -0.000156721573936292690 0.000000000000729794567705829956");
	m_poly.push_back("56750.356548217591 3.4427912972900310 -0.000156637076179907893 0.000000000000729401098669076220");
	m_poly.push_back("56750.356559791668 3.4426347002490849 -0.000156552718498901395 0.000000000000729008281907765963");
	m_poly.push_back("56750.356571365737 3.4424781911461069 -0.000156468475411296288 0.000000000000728615998761348825");
	m_poly.push_back("56750.356582939807 3.4423217627206002 -0.000156384196938471010 0.000000000000728223550838334455");
	m_poly.push_back("56750.356594513891 3.4421654221326672 -0.000156300005036865813 0.000000000000727831506041785655");
	m_poly.push_back("56750.356606087960 3.4420091621147679 -0.000156215864374551611 0.000000000000727439699845436281");
	m_poly.push_back("56750.356617662037 3.4418529898291150 -0.000156131737008340506 0.000000000000727047955561577167");
	m_poly.push_back("56750.356629236107 3.4416968980219438 -0.000156047751653747198 0.000000000000726656872569629127");
	m_poly.push_back("56750.356640810183 3.4415408938409842 -0.000155963681477654808 0.000000000000726265394594722921");
	m_poly.push_back("56750.356652384260 3.4413849700325052 -0.000155879699551448794 0.000000000000725874327563920816");
	m_poly.push_back("56750.356663958330 3.4412291337592040 -0.000155795833879546707 0.000000000000725483801882217401");
	m_poly.push_back("56750.356675532406 3.4410733777537419 -0.000155711967941189711 0.000000000000725093274958403847");
	m_poly.push_back("56750.356687106483 3.4409177091805798 -0.000155628224581001306 0.000000000000724703318831496808");
	m_poly.push_back("56750.356698680553 3.4407621207709438 -0.000155544434315856093 0.000000000000724313144285018005");
	m_poly.push_back("56750.356710254629 3.4406066196953118 -0.000155460757995738393 0.000000000000723923500334230354");
	m_poly.push_back("56750.356721828713 3.4404511986848210 -0.000155377066763024200 0.000000000000723533786939931086");
	m_poly.push_back("56750.356733402783 3.4402958649050910 -0.000155293498898961699 0.000000000000723144648023496501");
	m_poly.push_back("56750.356744976852 3.4401406110874060 -0.000155209913670916795 0.000000000000722755428248396453");
	m_poly.push_back("56750.356756550929 3.4399854443992339 -0.000155126485642423199 0.000000000000722366940488604997");
	m_poly.push_back("56750.356768124999 3.4398303575778648 -0.000155043028676919698 0.000000000000721978317979001287");
	m_poly.push_back("56750.356779699083 3.4396753577822961 -0.000154959603552785202 0.000000000000721589843740135395");
	m_poly.push_back("56750.356791273152 3.4395204377531412 -0.000154876344323515906 0.000000000000721202142008125068");
	m_poly.push_back("56750.356802847222 3.4393656046483891 -0.000154793039361691607 0.000000000000720814227315141362");
	m_poly.push_back("56750.356814421299 3.4392108512058241 -0.000154709735163705003 0.000000000000720426316177727381");
	m_poly.push_back("56750.356825995368 3.4390561845904060 -0.000154626519268895698 0.000000000000720038816231858706");
	m_poly.push_back("56750.356837569438 3.4389015975385542 -0.000154543338199751313 0.000000000000719651478454304937");
	m_poly.push_back("56750.356849143522 3.4387470972177518 -0.000154460317216753497 0.000000000000719264886133911017");
	m_poly.push_back("56750.356860717591 3.4385926763477581 -0.000154377256833947287 0.000000000000718878110342426684");
	m_poly.push_back("56750.356872291668 3.4384383421133999 -0.000154294252699785600 0.000000000000718491596476820592");
	m_poly.push_back("56750.356883865737 3.4382840872394702 -0.000154211336362536909 0.000000000000718105491446250679");
	m_poly.push_back("56750.356895439807 3.4381299188872401 -0.000154128446457621292 0.000000000000717719509498635749");
	m_poly.push_back("56750.356907013891 3.4379758298019110 -0.000154045573143968707 0.000000000000717333604808818724");
	m_poly.push_back("56750.356918587961 3.4378218271407430 -0.000153962773220896789 0.000000000000716948041868382888");
	m_poly.push_back("56750.356930162037 3.4376679036373772 -0.000153880035204196692 0.000000000000716562767200622768");
	m_poly.push_back("56750.356941736107 3.4375140664630819 -0.000153797341410139910 0.000000000000716177698458432878");
	m_poly.push_back("56750.356953310184 3.4373603083524911 -0.000153714758106077913 0.000000000000715793144224065489");
	m_poly.push_back("56750.356964884260 3.4372066364704068 -0.000153632153139226813 0.000000000000715408489112313766");
	m_poly.push_back("56750.356976458330 3.4370530435413542 -0.000153549642426380705 0.000000000000715024272903671752");
	m_poly.push_back("56750.356988032407 3.4368995367481818 -0.000153467157177748206 0.000000000000714640175269845534");
	m_poly.push_back("56750.356999606483 3.4367461088073070 -0.000153384606088092211 0.000000000000714255771038569657");
	m_poly.push_back("56750.357011180553 3.4365927668982752 -0.000153302255168292410 0.000000000000713872298918685493");
	m_poly.push_back("56750.357022754630 3.4364395037501598 -0.000153219894815986591 0.000000000000713488782873953377");
	m_poly.push_back("56750.357034328707 3.4362863265307810 -0.000153137615839029099 0.000000000000713105645760554756");
	m_poly.push_back("56750.357045902783 3.4361332279759131 -0.000153055375444751410 0.000000000000712722688310468225");
	m_poly.push_back("56750.357057476853 3.4359802152483501 -0.000152973202134953592 0.000000000000712340043244690184");
	m_poly.push_back("56750.357069050922 3.4358272810873820 -0.000152891042387713887 0.000000000000711957461333190738");
	m_poly.push_back("56750.357080624999 3.4356744326545638 -0.000152809011249406903 0.000000000000711575478301562972");
	m_poly.push_back("56750.357092199083 3.4355216626800540 -0.000152726878192073397 0.000000000000711193020671180941");
	m_poly.push_back("56750.357103773153 3.4353689783470589 -0.000152644881257702999 0.000000000000710811196910592053");
	m_poly.push_back("56750.357115347222 3.4352163723684419 -0.000152562969127068005 0.000000000000710429768046878810");
	m_poly.push_back("56750.357126921299 3.4350638519271510 -0.000152481027686386189 0.000000000000710048202695477340");
	m_poly.push_back("56750.357138495368 3.4349114097450122 -0.000152399222439721891 0.000000000000709667271545603578");
	m_poly.push_back("56750.357150069452 3.4347590530047869 -0.000152317399784666294 0.000000000000709286259329452420");
	m_poly.push_back("56750.357161643522 3.4346067744211579 -0.000152235642153505495 0.000000000000708905549903113655");
	m_poly.push_back("56750.357173217591 3.4344545811824729 -0.000152154023700657912 0.000000000000708525488574141515");
	m_poly.push_back("56750.357184791668 3.4343024660041608 -0.000152072349594248790 0.000000000000708145168086810170");
	m_poly.push_back("56750.357196365738 3.4341504360696948 -0.000151990676651359713 0.000000000000707764853015303330");
	m_poly.push_back("56750.357207939807 3.4339984840900120 -0.000151909192127662506 0.000000000000707385415337847014");
	m_poly.push_back("56750.357219513891 3.4338466172617421 -0.000151827651550721194 0.000000000000707005716640115284");
	m_poly.push_back("56750.357231087961 3.4336948282968689 -0.000151746163190072506 0.000000000000706626261092042826");
	m_poly.push_back("56750.357242662038 3.4335431243763241 -0.000151664786456293105 0.000000000000706247325344961594");
	m_poly.push_back("56750.357254236107 3.4333914982181470 -0.000151583474408898805 0.000000000000705868690816046159");
	m_poly.push_back("56750.357265810177 3.4332399570115739 -0.000151502156153103713 0.000000000000705490027374870143");
	m_poly.push_back("56750.357277384261 3.4330884934729169 -0.000151420923562581810 0.000000000000705111762842440474");
	m_poly.push_back("56750.357288958330 3.4329371147806458 -0.000151339687792364006 0.000000000000704733483501272564");
	m_poly.push_back("56750.357300532407 3.4327858136534140 -0.000151258507018452187 0.000000000000704355460255331151");
	m_poly.push_back("56750.357312106476 3.4326345972821848 -0.000151177364827220198 0.000000000000703977616671870607");
	m_poly.push_back("56750.357323680553 3.4324834583834920 -0.000151096310201973705 0.000000000000703600180848026931");
	m_poly.push_back("56750.357335254630 3.4323324041330729 -0.000151015336117183108 0.000000000000703223120067763127");
	m_poly.push_back("56750.357346828707 3.4321814272649251 -0.000150934403945759493 0.000000000000702846254460571186");
	m_poly.push_back("56750.357358402784 3.4320305349370712 -0.000150853459508525306 0.000000000000702469331734145675");
	m_poly.push_back("56750.357369976853 3.4318797198981068 -0.000150772642179194288 0.000000000000702093000898728106");
	m_poly.push_back("56750.357381550923 3.4317289893074872 -0.000150691783487169888 0.000000000000701716477451143705");
	m_poly.push_back("56750.357393124999 3.4315783358958618 -0.000150610963031433891 0.000000000000701340132053880406");
	m_poly.push_back("56750.357404699083 3.4314277668486661 -0.000150530347595034607 0.000000000000700964741350181715");
	m_poly.push_back("56750.357416273153 3.4312772748731151 -0.000150449622130495811 0.000000000000700588838286766379");
	m_poly.push_back("56750.357427847222 3.4311268671725301 -0.000150369032131664405 0.000000000000700213566032693353");
	m_poly.push_back("56750.357439421299 3.4309765364453488 -0.000150288437594216690 0.000000000000699838272642676102");
	m_poly.push_back("56750.357450995369 3.4308262898824542 -0.000150207897608983310 0.000000000000699463233278988091");
	m_poly.push_back("56750.357462569453 3.4306761202028810 -0.000150127446477601100 0.000000000000699088607672126336");
	m_poly.push_back("56750.357474143508 3.4305260346016881 -0.000150047024443101807 0.000000000000698714117556065066");
	m_poly.push_back("56750.357485717592 3.4303760257787701 -0.000149966626416195195 0.000000000000698339739232817550");
	m_poly.push_back("56750.357497291669 3.4302261009341621 -0.000149886248124720295 0.000000000000697965452807721612");
	m_poly.push_back("56750.357508865738 3.4300762527695832 -0.000149806000307362990 0.000000000000697591773948827495");
	m_poly.push_back("56750.357520439808 3.4299264884909899 -0.000149725771363899506 0.000000000000697218182976249796");
	m_poly.push_back("56750.357532013892 3.4297768007956790 -0.000149645620775961107 0.000000000000696844956873757211");
	m_poly.push_back("56750.357543587961 3.4296271968872531 -0.000149565471014033192 0.000000000000696471734615566429");
	m_poly.push_back("56750.357555162038 3.4294776694672300 -0.000149485466550001008 0.000000000000696099188952178036");
	m_poly.push_back("56750.357566736107 3.4293282257284670 -0.000149405329471329691 0.000000000000695726025751654168");
	m_poly.push_back("56750.357578310177 3.4291788583911171 -0.000149325381202650999 0.000000000000695353741764917992");
	m_poly.push_back("56750.357589884261 3.4290295746367110 -0.000149245433804391902 0.000000000000694981461829274430");
	m_poly.push_back("56750.357601458330 3.4288803671800441 -0.000149165526116805299 0.000000000000694609366809624160");
	m_poly.push_back("56750.357613032407 3.4287312432213279 -0.000149085662269943394 0.000000000000694237475937160833");
	m_poly.push_back("56750.357624606477 3.4285821954611619 -0.000149005887872015209 0.000000000000693866001592504195");
	m_poly.push_back("56750.357636180554 3.4284332310974270 -0.000148926131672961587 0.000000000000693494611990872699");
	m_poly.push_back("56750.357647754630 3.4282843428366760 -0.000148846485430992797 0.000000000000693123734415564277");
	m_poly.push_back("56750.357659328707 3.4281355378787590 -0.000148766790134663906 0.000000000000692752628410986442");
	m_poly.push_back("56750.357670902777 3.4279868089215220 -0.000148687203702954212 0.000000000000692382029345563643");
	m_poly.push_back("56750.357682476853 3.4278381631725892 -0.000148607608087429892 0.000000000000692011387512583596");
	m_poly.push_back("56750.357694050923 3.4276895933382279 -0.000148528056596848910 0.000000000000691640951151136441");
	m_poly.push_back("56750.357705625000 3.4275411066102071 -0.000148448638040653107 0.000000000000691271133811571248");
	m_poly.push_back("56750.357717199076 3.4273926956954481 -0.000148369186168703909 0.000000000000690901161332198870");
	m_poly.push_back("56750.357728773153 3.4272443677978379 -0.000148289869194024904 0.000000000000690531817015098552");
	m_poly.push_back("56750.357740347223 3.4270961156123878 -0.000148210537768604596 0.000000000000690162405405257991");
	m_poly.push_back("56750.357751921299 3.4269479463457202 -0.000148131229302721012 0.000000000000689793100707004562");
	m_poly.push_back("56750.357763495369 3.4267998527017220 -0.000148052054472887299 0.000000000000689424418298867721");
	m_poly.push_back("56750.357775069453 3.4266518418799272 -0.000147972855093668899 0.000000000000689055621571668862");
	m_poly.push_back("56750.357786643523 3.4265039065846010 -0.000147893746042685706 0.000000000000688687245466857781");
	m_poly.push_back("56750.357798217592 3.4263560540150628 -0.000147814703170235599 0.000000000000688319177527769004");
	m_poly.push_back("56750.357809791669 3.4262082768771331 -0.000147735654719994793 0.000000000000687951083613842104");
	m_poly.push_back("56750.357821365738 3.4260605823695629 -0.000147656662367407203 0.000000000000687583250922896216");
	m_poly.push_back("56750.357832939822 3.4259129631948890 -0.000147577769384760193 0.000000000000687215880958060092");
	m_poly.push_back("56750.357844513892 3.4257654265556630 -0.000147498834852901413 0.000000000000686848317512539808");
	m_poly.push_back("56750.357856087961 3.4256179651604368 -0.000147419951364933290 0.000000000000686480991757443161");
	m_poly.push_back("56750.357867662038 3.4254705862025632 -0.000147341216212447602 0.000000000000686114356741181688");
	m_poly.push_back("56750.357879236108 3.4253232823873550 -0.000147262470135308099 0.000000000000685747670851818729");
	m_poly.push_back("56750.357890810177 3.4251760609207480 -0.000147183749611200292 0.000000000000685381103950963252");
	m_poly.push_back("56750.357902384261 3.4250289144980561 -0.000147105135917772393 0.000000000000685015034517999964");
	m_poly.push_back("56750.357913958331 3.4248818503333149 -0.000147026530262402697 0.000000000000684649002513097126");
	m_poly.push_back("56750.357925532408 3.4247348611183011 -0.000146948006888328195 0.000000000000684283353659105943");
	m_poly.push_back("56750.357937106477 3.4245879540555260 -0.000146869454701589389 0.000000000000683917570633601011");
	m_poly.push_back("56750.357948680547 3.4244411218516859 -0.000146791001049899003 0.000000000000683552246446473727");
	m_poly.push_back("56750.357960254631 3.4242943717160692 -0.000146712618372931108 0.000000000000683187252759108056");
	m_poly.push_back("56750.357971828700 3.4241476963361039 -0.000146634242765901788 0.000000000000682822291992470737");
	m_poly.push_back("56750.357983402777 3.4240011029280999 -0.000146555931152474688 0.000000000000682457629217199048");
	m_poly.push_back("56750.357994976846 3.4238545841792658 -0.000146477679971035099 0.000000000000682093247849085477");
	m_poly.push_back("56750.358006550923 3.4237081473130639 -0.000146399449644137887 0.000000000000681728963590430712");
	m_poly.push_back("56750.358018125000 3.4235617850132511 -0.000146321244577172002 0.000000000000681364796956293900");
	m_poly.push_back("56750.358029699077 3.4234155044966230 -0.000146243178475355401 0.000000000000681001277426992200");
	m_poly.push_back("56750.358041273154 3.4232692984548940 -0.000146165038272411411 0.000000000000680637412836223207");
	m_poly.push_back("56750.358052847223 3.4231231740970798 -0.000146087042043969989 0.000000000000680274218676922248");
	m_poly.push_back("56750.358064421300 3.4229771241238418 -0.000146009054537488109 0.000000000000679911065131197698");
	m_poly.push_back("56750.358075995369 3.4228311557381992 -0.000145931070885721388 0.000000000000679547929533331319");
	m_poly.push_back("56750.358087569453 3.4226852616483971 -0.000145853260269813095 0.000000000000679185599695265977");
	m_poly.push_back("56750.358099143523 3.4225394490551682 -0.000145775394204625302 0.000000000000678823011649315575");
	m_poly.push_back("56750.358110717592 3.4223937106499429 -0.000145697555468835693 0.000000000000678460550864590172");
	m_poly.push_back("56750.358122291669 3.4222480536508000 -0.000145619859091055309 0.000000000000678098752983952675");
	m_poly.push_back("56750.358133865739 3.4221024707468022 -0.000145542102156941811 0.000000000000677736673114710786");
	m_poly.push_back("56750.358145439823 3.4219569691608478 -0.000145464482873466488 0.000000000000677375234229146574");
	m_poly.push_back("56750.358157013892 3.4218115415734669 -0.000145386901790752187 0.000000000000677013973228543274");
	m_poly.push_back("56750.358168587962 3.4216661952105070 -0.000145309302669036387 0.000000000000676652628225246527");
	m_poly.push_back("56750.358180162038 3.4215209227494499 -0.000145231713557145695 0.000000000000676291329832630362");
	m_poly.push_back("56750.358191736108 3.4213757314130691 -0.000145154282248770701 0.000000000000675930766267855920");
	m_poly.push_back("56750.358203310177 3.4212306138960158 -0.000145076892542898590 0.000000000000675570396428630587");
	m_poly.push_back("56750.358214884262 3.4210855774113971 -0.000144999498529337805 0.000000000000675210006528096780");
	m_poly.push_back("56750.358226458331 3.4209406146481580 -0.000144922197606261994 0.000000000000674850050112745794");
	m_poly.push_back("56750.358238032408 3.4207957328244141 -0.000144844872480193193 0.000000000000674489980991393246");
	m_poly.push_back("56750.358249606477 3.4206509246208312 -0.000144767604801815988 0.000000000000674130179380499195");
	m_poly.push_back("56750.358261180547 3.4205061972704218 -0.000144690458680196796 0.000000000000673770943810329967");
	m_poly.push_back("56750.358272754631 3.4203615434498760 -0.000144613304351764392 0.000000000000673411670023017845");
	m_poly.push_back("56750.358284328700 3.4202169703832781 -0.000144536261961065510 0.000000000000673052917484352082");
	m_poly.push_back("56750.358295902777 3.4200724707514989 -0.000144459169299195208 0.000000000000672693930851120707");
	m_poly.push_back("56750.358307476847 3.4199280517869912 -0.000144382197883218791 0.000000000000672335508811027989");
	m_poly.push_back("56750.358319050923 3.4197837061680580 -0.000144305148458108608 0.000000000000671976723511427861");
	m_poly.push_back("56750.358330625000 3.4196394411115509 -0.000144228311655183610 0.000000000000671618928309008919");
	m_poly.push_back("56750.358342199077 3.4194952493183730 -0.000144151436047533011 0.000000000000671260952406810383");
	m_poly.push_back("56750.358353773147 3.4193511379970438 -0.000144074644444253011 0.000000000000670903367678421974");
	m_poly.push_back("56750.358365347223 3.4192070998280570 -0.000143997858818446111 0.000000000000670545810783621647");
	m_poly.push_back("56750.358376921293 3.4190631420466628 -0.000143921147471403395 0.000000000000670188599774214523");
	m_poly.push_back("56750.358388495370 3.4189192573321159 -0.000143844460522754304 0.000000000000669831502377526745");
	m_poly.push_back("56750.358400069454 3.4187754529086529 -0.000143767824564704813 0.000000000000669474642422304161");
	m_poly.push_back("56750.358411643523 3.4186317214536310 -0.000143691252964412591 0.000000000000669118082155054723");
	m_poly.push_back("56750.358423217593 3.4184880702024332 -0.000143614707885271990 0.000000000000668761645384515867");
	m_poly.push_back("56750.358434791669 3.4183444918269781 -0.000143538211114417608 0.000000000000668405433565787711");
	m_poly.push_back("56750.358446365739 3.4182009935673161 -0.000143461822790736613 0.000000000000668049726741532661");
	m_poly.push_back("56750.358457939823 3.4180575680886540 -0.000143385339848895198 0.000000000000667693579315966407");
	m_poly.push_back("56750.358469513892 3.4179142226205319 -0.000143309036968441593 0.000000000000667338270364433678");
	m_poly.push_back("56750.358481087962 3.4177709498554032 -0.000143232725303855712 0.000000000000666982920507546473");
	m_poly.push_back("56750.358492662039 3.4176277570083040 -0.000143156406542685797 0.000000000000666627537602709993");
	m_poly.push_back("56750.358504236108 3.4174846367674361 -0.000143080291344232098 0.000000000000666273102609352874");
	m_poly.push_back("56750.358515810192 3.4173415963485101 -0.000143004093846719609 0.000000000000665918284379129414");
	m_poly.push_back("56750.358527384262 3.4171986284454809 -0.000142927935038468794 0.000000000000665563646308644246");
	m_poly.push_back("56750.358538958331 3.4170557402795461 -0.000142851814422097407 0.000000000000665209186080929942");
	m_poly.push_back("56750.358550532408 3.4169129245391141 -0.000142775861556894011 0.000000000000664855507004308128");
	m_poly.push_back("56750.358562106478 3.4167701884344268 -0.000142699829039417801 0.000000000000664501457015874411");
	m_poly.push_back("56750.358573680547 3.4166275246610871 -0.000142623925930239308 0.000000000000664148009630621038");
	m_poly.push_back("56750.358585254631 3.4164849404449091 -0.000142548023087515913 0.000000000000663794563484106882");
	m_poly.push_back("56750.358596828701 3.4163424284654371 -0.000142472192667253612 0.000000000000663441454579822259");
	m_poly.push_back("56750.358608402777 3.4161999959417511 -0.000142396346508384306 0.000000000000663088272384944899");
	m_poly.push_back("56750.358619976847 3.4160576355657528 -0.000142320577719156808 0.000000000000662735450469360536");
	m_poly.push_back("56750.358631550916 3.4159153545616978 -0.000142244923443394101 0.000000000000662383161796636025");
	m_poly.push_back("56750.358643125001 3.4157731456085432 -0.000142169269398559112 0.000000000000662030874198100759");
	m_poly.push_back("56750.358654699077 3.4156310159360990 -0.000142093541723333709 0.000000000000661678243729312770");
	m_poly.push_back("56750.358666273147 3.4154889582256129 -0.000142018029521361695 0.000000000000661326616634917286");
	m_poly.push_back("56750.358677847224 3.4153469797045890 -0.000141942515471968594 0.000000000000660974980935809911");
	m_poly.push_back("56750.358689421293 3.4152050730429608 -0.000141867029711211494 0.000000000000660623476964758481");
	m_poly.push_back("56750.358700995370 3.4150632454903218 -0.000141791579646538909 0.000000000000660272139214673208");
	m_poly.push_back("56750.358712569447 3.4149214897085902 -0.000141716221055857711 0.000000000000659921227422280465");
	m_poly.push_back("56750.358724143523 3.4147798129360969 -0.000141640837596045994 0.000000000000659570199821852741");
	m_poly.push_back("56750.358735717593 3.4146382078473070 -0.000141565570159610487 0.000000000000659219712496343550");
	m_poly.push_back("56750.358747291670 3.4144966816813960 -0.000141490349672541087 0.000000000000658869443793838937");
	m_poly.push_back("56750.358758865739 3.4143552271032038 -0.000141415085868775104 0.000000000000658518973381118804");
	m_poly.push_back("56750.358770439823 3.4142138513584110 -0.000141339982719593103 0.000000000000658169251072616756");
	m_poly.push_back("56750.358782013893 3.4140725471132920 -0.000141264832780918113 0.000000000000657819310882403202");
	m_poly.push_back("56750.358793587962 3.4139313216070049 -0.000141189746924663808 0.000000000000657469669097217277");
	m_poly.push_back("56750.358805162039 3.4137901675025200 -0.000141114730995075687 0.000000000000657120352932304715");
	m_poly.push_back("56750.358816736109 3.4136490920554472 -0.000141039729995847790 0.000000000000656771106290284534");
	m_poly.push_back("56750.358828310193 3.4135080879226098 -0.000140964769959630898 0.000000000000656422050396029092");
	m_poly.push_back("56750.358839884262 3.4133671623486639 -0.000140889923344413406 0.000000000000656073522657483036");
	m_poly.push_back("56750.358851458332 3.4132263080012750 -0.000140815018748594012 0.000000000000655724724924566680");
	m_poly.push_back("56750.358863032408 3.4130855321238629 -0.000140740230229442103 0.000000000000655376467713758940");
	m_poly.push_back("56750.358874606478 3.4129448273815588 -0.000140665467032394199 0.000000000000655028328417055473");
	m_poly.push_back("56750.358886180547 3.4128042010172548 -0.000140590780485559707 0.000000000000654680546048743812");
	m_poly.push_back("56750.358897754631 3.4126636457024619 -0.000140516081548569012 0.000000000000654332705983127570");
	m_poly.push_back("56750.358909328701 3.4125231686709250 -0.000140441499407674089 0.000000000000653985409789723236");
	m_poly.push_back("56750.358920902778 3.4123827625967551 -0.000140366974492425208 0.000000000000653638380072871203");
	m_poly.push_back("56750.358932476847 3.4122424347255680 -0.000140292397609574989 0.000000000000653291108361043550");
	m_poly.push_back("56750.358944050917 3.4121021777168981 -0.000140217916455114206 0.000000000000652944282418126033");
	m_poly.push_back("56750.358955625001 3.4119619988127030 -0.000140143491451597587 0.000000000000652597717946420250");
	m_poly.push_back("56750.358967199078 3.4118218906869391 -0.000140069044367845813 0.000000000000652251050654494010");
	m_poly.push_back("56750.358978773147 3.4116818605842241 -0.000139994759093316093 0.000000000000651905136843496996");
	m_poly.push_back("56750.358990347217 3.4115419011548660 -0.000139920412427561099 0.000000000000651558937156200730");
	m_poly.push_back("56750.359001921293 3.4114020196648469 -0.000139846089671698888 0.000000000000651212848806035405");
	m_poly.push_back("56750.359013495370 3.4112622087712352 -0.000139771919905399292 0.000000000000650867472867917524");
	m_poly.push_back("56750.359025069447 3.4111224757148801 -0.000139697730385904387 0.000000000000650522004944822070");
	m_poly.push_back("56750.359036643516 3.4109828131652300 -0.000139623660788910312 0.000000000000650177095453282921");
	m_poly.push_back("56750.359048217593 3.4108432283663261 -0.000139549541897746607 0.000000000000649831956415915131");
	m_poly.push_back("56750.359059791670 3.4107037139778180 -0.000139475503804611895 0.000000000000649487193622557931");
	m_poly.push_back("56750.359071365740 3.4105642772525719 -0.000139401461981107600 0.000000000000649142413456343348");
	m_poly.push_back("56750.359082939824 3.4104249108578011 -0.000139327505734058299 0.000000000000648798031785429605");
	m_poly.push_back("56750.359094513893 3.4102856220362519 -0.000139253664249164993 0.000000000000648454184515471384");
	m_poly.push_back("56750.359106087963 3.4101464034453519 -0.000139179800257708587 0.000000000000648110232440051766");
	m_poly.push_back("56750.359117662039 3.4100072623421349 -0.000139105969386604995 0.000000000000647766434591329174");
	m_poly.push_back("56750.359129236109 3.4098681913861131 -0.000139032257407709602 0.000000000000647423190376474742");
	m_poly.push_back("56750.359140810193 3.4097291978229558 -0.000138958533198531097 0.000000000000647079889207965236");
	m_poly.push_back("56750.359152384262 3.4095902743201552 -0.000138884816343509806 0.000000000000646736622283800290");
	m_poly.push_back("56750.359163958332 3.4094514281179178 -0.000138811178199286893 0.000000000000646393721883379104");
	m_poly.push_back("56750.359175532409 3.4093126518861649 -0.000138737613800918211 0.000000000000646051164887749960");
	m_poly.push_back("56750.359187106478 3.4091739528717451 -0.000138664028707880107 0.000000000000645708511523083169");
	m_poly.push_back("56750.359198680548 3.4090353237408739 -0.000138590557275651013 0.000000000000645366387431704124");
	m_poly.push_back("56750.359210254632 3.4088967717386511 -0.000138517086998059990 0.000000000000645024268715074391");
	m_poly.push_back("56750.359221828701 3.4087582895206840 -0.000138443691052524099 0.000000000000644682496132935246");
	m_poly.push_back("56750.359233402778 3.4086198843501481 -0.000138370327410212594 0.000000000000644340873972449282");
	m_poly.push_back("56750.359244976848 3.4084815488853688 -0.000138297019288235404 0.000000000000643999510347559324");
	m_poly.push_back("56750.359256550917 3.4083432903635602 -0.000138223740067740605 0.000000000000643658281303713045");
	m_poly.push_back("56750.359268125001 3.4082051014539312 -0.000138150446645195788 0.000000000000643316986125329429");
	m_poly.push_back("56750.359279699071 3.4080669894149409 -0.000138077254306785387 0.000000000000642976161654682765");
	m_poly.push_back("56750.359291273147 3.4079289468948590 -0.000138004150111679301 0.000000000000642635747632169896");
	m_poly.push_back("56750.359302847217 3.4077909811565390 -0.000137930990538348504 0.000000000000642295075732750740");
	m_poly.push_back("56750.359314421286 3.4076530848473392 -0.000137857978216916505 0.000000000000641955089527318621");
	m_poly.push_back("56750.359325995370 3.4075152652257219 -0.000137784905809888611 0.000000000000641614823524570187");
	m_poly.push_back("56750.359337569447 3.4073775149588381 -0.000137712050545898507 0.000000000000641275568671795017");
	m_poly.push_back("56750.359349143517 3.4072398412880949 -0.000137639079249650301 0.000000000000640935773499758091");
	m_poly.push_back("56750.359360717594 3.4071022368751440 -0.000137566184870070496 0.000000000000640596336497779127");
	m_poly.push_back("56750.359372291663 3.4069647089790140 -0.000137493309399911812 0.000000000000640256987547708306");
	m_poly.push_back("56750.359383865740 3.4068272502499899 -0.000137420511583613602 0.000000000000639918000200508318");
	m_poly.push_back("56750.359395439817 3.4066898679447228 -0.000137347745413284088 0.000000000000639579160214382543");
	m_poly.push_back("56750.359407013893 3.4065525547278850 -0.000137275024611354312 0.000000000000639240531490245192");
	m_poly.push_back("56750.359418587963 3.4064153178464749 -0.000137202370893904489 0.000000000000638902215150605659");
	m_poly.push_back("56750.359430162040 3.4062781499526369 -0.000137129707388675408 0.000000000000638563853231988684");
	m_poly.push_back("56750.359441736109 3.4061410583172802 -0.000137057195940416587 0.000000000000638226199381882043");
	m_poly.push_back("56750.359453310193 3.4060040355841652 -0.000136984683834902212 0.000000000000637888542470095510");
	m_poly.push_back("56750.359464884263 3.4058670890131921 -0.000136912145527982209 0.000000000000637550763546562785");
	m_poly.push_back("56750.359476458332 3.4057302112676040 -0.000136839733005572292 0.000000000000637213570351717226");
	m_poly.push_back("56750.359488032409 3.4055934095939011 -0.000136767357138484707 0.000000000000636876547844578901");
	m_poly.push_back("56750.359499606478 3.4054566766498779 -0.000136694951979113896 0.000000000000636539388933533569");
	m_poly.push_back("56750.359511180563 3.4053200196949338 -0.000136622603743406703 0.000000000000636202495092005811");
	m_poly.push_back("56750.359522754632 3.4051834313891920 -0.000136550374524253501 0.000000000000635866155463424139");
	m_poly.push_back("56750.359534328702 3.4050469189771451 -0.000136478160848306805 0.000000000000635529888211611258");
	m_poly.push_back("56750.359545902778 3.4049104751261301 -0.000136405966019608496 0.000000000000635193708723050273");
	m_poly.push_back("56750.359557476862 3.4047741070900019 -0.000136333851429383714 0.000000000000634857902872064167");
	m_poly.push_back("56750.359569050917 3.4046378075175370 -0.000136261692891852498 0.000000000000634521892374408223");
	m_poly.push_back("56750.359580625001 3.4045015836840520 -0.000136189712905797404 0.000000000000634186713319918919");
	m_poly.push_back("56750.359592199071 3.4043654282211291 -0.000136117627749965207 0.000000000000633851044529420794");
	m_poly.push_back("56750.359603773148 3.4042293484106070 -0.000136045730791279192 0.000000000000633516252098248906");
	m_poly.push_back("56750.359615347217 3.4040933368811959 -0.000135973744454713892 0.000000000000633181043467818250");
	m_poly.push_back("56750.359626921287 3.4039574009209712 -0.000135901872951359594 0.000000000000632846369569270192");
	m_poly.push_back("56750.359638495371 3.4038215331602899 -0.000135830083800354899 0.000000000000632512079152659816");
	m_poly.push_back("56750.359650069448 3.4036857408697130 -0.000135758245719335595 0.000000000000632177560885983060");
	m_poly.push_back("56750.359661643517 3.4035500166939441 -0.000135686516502935787 0.000000000000631843549558629765");
	m_poly.push_back("56750.359673217587 3.4034143679132369 -0.000135614816223545912 0.000000000000631509672977811202");
	m_poly.push_back("56750.359684791663 3.4032787871578138 -0.000135543123049621992 0.000000000000631175829483323116");
	m_poly.push_back("56750.359696365740 3.4031432817006420 -0.000135471551645619692 0.000000000000630842553022293315");
	m_poly.push_back("56750.359707939817 3.4030078441884219 -0.000135399935645937188 0.000000000000630509068895383429");
	m_poly.push_back("56750.359719513886 3.4028724818919902 -0.000135328446070256703 0.000000000000630176173474183329");
	m_poly.push_back("56750.359731087963 3.4027371874476962 -0.000135256957862378411 0.000000000000629843284421249562");
	m_poly.push_back("56750.359742662040 3.4026019681341921 -0.000135185465915248589 0.000000000000629510377954153505");
	m_poly.push_back("56750.359754236109 3.4024668165944330 -0.000135114010809959598 0.000000000000629177643044182499");
	m_poly.push_back("56750.359765810193 3.4023317400938051 -0.000135042677625583506 0.000000000000628845475870782890");
	m_poly.push_back("56750.359777384263 3.4021967312778560 -0.000134971347487676009 0.000000000000628513322882552448");
	m_poly.push_back("56750.359788958333 3.4020617974211080 -0.000134900038453002094 0.000000000000628181268161983314");
	m_poly.push_back("56750.359800532409 3.4019269311585032 -0.000134828867983795194 0.000000000000627849858686106492");
	m_poly.push_back("56750.359812106479 3.4017921397661248 -0.000134757710109026510 0.000000000000627518507855742434");
	m_poly.push_back("56750.359823680563 3.4016574158849422 -0.000134686546789694595 0.000000000000627187131671096664");
	m_poly.push_back("56750.359835254632 3.4015227667901189 -0.000134615417994045095 0.000000000000626855916247982800");
	m_poly.push_back("56750.359846828702 3.4013881851218160 -0.000134544356273993791 0.000000000000626525013168889026");
	m_poly.push_back("56750.359858402779 3.4012536781548892 -0.000134473420018706499 0.000000000000626194694328696409");
	m_poly.push_back("56750.359869976848 3.4011192385234539 -0.000134402349256949796 0.000000000000625863749142919176");
	m_poly.push_back("56750.359881550918 3.4009848735074830 -0.000134331452623516695 0.000000000000625533614803450848");
	m_poly.push_back("56750.359893125002 3.4008505757472371 -0.000134260570671752589 0.000000000000625203548829757670");
	m_poly.push_back("56750.359904699071 3.4007163525186770 -0.000134189730633355492 0.000000000000624873678028470935");
	m_poly.push_back("56750.359916273148 3.4005821964564582 -0.000134118934098173288 0.000000000000624544009803778415");
	m_poly.push_back("56750.359927847217 3.4004481148334720 -0.000134048120856264505 0.000000000000624214263780418396");
	m_poly.push_back("56750.359939421287 3.4003141002951600 -0.000133977402196988491 0.000000000000623884958190722628");
	m_poly.push_back("56750.359950995371 3.4001801601207240 -0.000133906708593361791 0.000000000000623555769273585841");
	m_poly.push_back("56750.359962569448 3.4000462869382990 -0.000133836076745115387 0.000000000000623226867926301251");
	m_poly.push_back("56750.359974143517 3.3999124880359490 -0.000133765458343963387 0.000000000000622898029195027163");
	m_poly.push_back("56750.359985717587 3.3997787560437138 -0.000133694941248991412 0.000000000000622569662206373550");
	m_poly.push_back("56750.359997291664 3.3996450982388269 -0.000133624405324534698 0.000000000000622241207534199892");
	m_poly.push_back("56750.360008865740 3.3995115072595019 -0.000133553942808422601 0.000000000000621913094695228425");
	m_poly.push_back("56750.360020439817 3.3993779903965611 -0.000133483540528897787 0.000000000000621585262352833716");
	m_poly.push_back("56750.360032013887 3.3992445402608960 -0.000133413181148623206 0.000000000000621257629774595005");
	m_poly.push_back("56750.360043587963 3.3991111641547280 -0.000133342884873767709 0.000000000000620930291051970929");
	m_poly.push_back("56750.360055162040 3.3989778547003651 -0.000133272564875537900 0.000000000000620602841857736528");
	m_poly.push_back("56750.360066736110 3.3988446191825230 -0.000133202304501048795 0.000000000000620275670306259323");
	m_poly.push_back("56750.360078310187 3.3987114502305960 -0.000133132041835047008 0.000000000000619948488083087343");
	m_poly.push_back("56750.360089884263 3.3985783551360780 -0.000133061873929314812 0.000000000000619621747119934458");
	m_poly.push_back("56750.360101458333 3.3984453265268142 -0.000132991805686623592 0.000000000000619295470247929079");
	m_poly.push_back("56750.360113032410 3.3983123716939758 -0.000132921760616633191 0.000000000000618969301280325578");
	m_poly.push_back("56750.360124606479 3.3981794832530561 -0.000132851668384112806 0.000000000000618642912694051142");
	m_poly.push_back("56750.360136180563 3.3980466684980981 -0.000132781669153259307 0.000000000000618316957178647254");
	m_poly.push_back("56750.360147754633 3.3979139200584099 -0.000132711745755490412 0.000000000000617991354787533116");
	m_poly.push_back("56750.360159328702 3.3977812452221250 -0.000132641818574061093 0.000000000000617665734775489201");
	m_poly.push_back("56750.360170902779 3.3976486366071472 -0.000132571865528735789 0.000000000000617339994324268090");
	m_poly.push_back("56750.360182476848 3.3975161015256048 -0.000132502098175879996 0.000000000000617015118569109413");
	m_poly.push_back("56750.360194050932 3.3973836325839639 -0.000132432336542924288 0.000000000000616690269448282027");
	m_poly.push_back("56750.360205625002 3.3972512370731121 -0.000132362571286180896 0.000000000000616365403451002663");
	m_poly.push_back("56750.360217199071 3.3971189076198880 -0.000132292818179784204 0.000000000000616040594032153617");
	m_poly.push_back("56750.360228773148 3.3969866515244518 -0.000132223147709955999 0.000000000000615716169417903750");
	m_poly.push_back("56750.360240347232 3.3968544614043958 -0.000132153577640360789 0.000000000000615392212327678175");
	m_poly.push_back("56750.360251921287 3.3967223445574071 -0.000132084034784700190 0.000000000000615068381960333776");
	m_poly.push_back("56750.360263495371 3.3965902935907848 -0.000132014434925439291 0.000000000000614744286148267846");
	m_poly.push_back("56750.360275069441 3.3964583158177342 -0.000131945004700294907 0.000000000000614420980254744084");
	m_poly.push_back("56750.360286643518 3.3963264038496890 -0.000131875447100792499 0.000000000000614097081227117743");
	m_poly.push_back("56750.360298217587 3.3961945649827059 -0.000131806043920827687 0.000000000000613773901269633331");
	m_poly.push_back("56750.360309791657 3.3960627918379092 -0.000131736696572061208 0.000000000000613450981295398541");
	m_poly.push_back("56750.360321365741 3.3959310917200551 -0.000131667362457225012 0.000000000000613128122944568624");
	m_poly.push_back("56750.360332939817 3.3957994572348049 -0.000131598131948963389 0.000000000000612805747048445052");
	m_poly.push_back("56750.360344513887 3.3956678956865969 -0.000131528845591739890 0.000000000000612483111083460886");
	m_poly.push_back("56750.360356087956 3.3955363996890759 -0.000131459654607632290 0.000000000000612160919233137207");
	m_poly.push_back("56750.360367662041 3.3954049765501919 -0.000131390489238729309 0.000000000000611838846661037382");
	m_poly.push_back("56750.360379236110 3.3952736188881021 -0.000131321388076592588 0.000000000000611517073073978133");
	m_poly.push_back("56750.360390810187 3.3951423339863118 -0.000131252314616890994 0.000000000000611195428484566231");
	m_poly.push_back("56750.360402384264 3.3950111144779731 -0.000131183303036915705 0.000000000000610874072044075262");
	m_poly.push_back("56750.360413958333 3.3948799676562542 -0.000131114216760729990 0.000000000000610552367770358361");
	m_poly.push_back("56750.360425532410 3.3947488861400279 -0.000131045391645943395 0.000000000000610231879623192403");
	m_poly.push_back("56750.360437106479 3.3946178772268918 -0.000130976436749822195 0.000000000000609910787132747483");
	m_poly.push_back("56750.360448680563 3.3944869335397332 -0.000130907545714075910 0.000000000000609589992014321051");
	m_poly.push_back("56750.360460254633 3.3943560623663820 -0.000130838706503821304 0.000000000000609269438225283277");
	m_poly.push_back("56750.360471828702 3.3942252563396451 -0.000130769907430567300 0.000000000000608949071337716016");
	m_poly.push_back("56750.360483402779 3.3940945227512169 -0.000130701155750770890 0.000000000000608628925141267735");
	m_poly.push_back("56750.360494976849 3.3939638542173340 -0.000130632450434139388 0.000000000000608308994839132905");
	m_poly.push_back("56750.360506550933 3.3938332580397512 -0.000130563829619261110 0.000000000000607989458027073051");
	m_poly.push_back("56750.360518125002 3.3937027268374349 -0.000130495156064062311 0.000000000000607669675622774801");
	m_poly.push_back("56750.360529699072 3.3935722679062579 -0.000130426603754757101 0.000000000000607350457811831191");
	m_poly.push_back("56750.360541273149 3.3934418738691918 -0.000130358043700494112 0.000000000000607031203934596337");
	m_poly.push_back("56750.360552847233 3.3933115520236461 -0.000130289574356385192 0.000000000000606712372457568606");
	m_poly.push_back("56750.360564421288 3.3931812949845699 -0.000130221103751056004 0.000000000000606393535106538182");
	m_poly.push_back("56750.360575995372 3.3930511100506058 -0.000130152683550125597 0.000000000000606074932467432128");
	m_poly.push_back("56750.360587569441 3.3929209898471808 -0.000130084333613370988 0.000000000000605756657020391134");
	m_poly.push_back("56750.360599143518 3.3927909416731050 -0.000130015943095524507 0.000000000000605438192601057701");
	m_poly.push_back("56750.360610717587 3.3926609581432552 -0.000129947718881107197 0.000000000000605120502591425730");
	m_poly.push_back("56750.360622291657 3.3925310465548399 -0.000129879433550801387 0.000000000000604802527986905118");
	m_poly.push_back("56750.360633865741 3.3924011995285270 -0.000129811243495911389 0.000000000000604484997042126251");
	m_poly.push_back("56750.360645439818 3.3922714243665348 -0.000129743105852714297 0.000000000000604167710156469119");
	m_poly.push_back("56750.360657013887 3.3921417136868359 -0.000129674943473614100 0.000000000000603850308084291054");
	m_poly.push_back("56750.360668587957 3.3920120747806481 -0.000129606794355089592 0.000000000000603532967759604547");
	m_poly.push_back("56750.360680162034 3.3918825002730171 -0.000129538728841253198 0.000000000000603216016748626766");
	m_poly.push_back("56750.360691736110 3.3917529974733029 -0.000129470813918884702 0.000000000000602899766981981668");
	m_poly.push_back("56750.360703310187 3.3916235589795818 -0.000129402802717453304 0.000000000000602583068880017300");
	m_poly.push_back("56750.360714884257 3.3914941921161290 -0.000129334933216775604 0.000000000000602267030621771592");
	m_poly.push_back("56750.360726458333 3.3913648894795578 -0.000129267040303587800 0.000000000000601950883339537672");
	m_poly.push_back("56750.360738032410 3.3912356583751331 -0.000129199081957940794 0.000000000000601634431361827372");
	m_poly.push_back("56750.360749606480 3.3911064914266831 -0.000129131362702339511 0.000000000000601319092732922961");
	m_poly.push_back("56750.360761180556 3.3909773959358391 -0.000129063624368562312 0.000000000000601003665262496033");
	m_poly.push_back("56750.360772754633 3.3908483645101501 -0.000128995930994620610 0.000000000000600688447151640570");
	m_poly.push_back("56750.360784328703 3.3907194044589759 -0.000128928202848305098 0.000000000000600373067117477120");
	m_poly.push_back("56750.360795902780 3.3905905083984158 -0.000128860581852415407 0.000000000000600058186040373705");
	m_poly.push_back("56750.360807476849 3.3904616836322559 -0.000128792941076684891 0.000000000000599743212854360826");
	m_poly.push_back("56750.360819050933 3.3903329227687462 -0.000128725405355267905 0.000000000000599428728864651307");
	m_poly.push_back("56750.360830625003 3.3902042331220410 -0.000128657856772957497 0.000000000000599114184984904692");
	m_poly.push_back("56750.360842199072 3.3900756073001528 -0.000128590397159961996 0.000000000000598800055399841563");
	m_poly.push_back("56750.360853773149 3.3899470526063840 -0.000128522962789134090 0.000000000000598486043355950569");
	m_poly.push_back("56750.360865347233 3.3898185616566470 -0.000128455556298378106 0.000000000000598172161137785296");
	m_poly.push_back("56750.360876921302 3.3896901417677658 -0.000128388185992207209 0.000000000000597858447415388704");
	m_poly.push_back("56750.360888495372 3.3895617855279969 -0.000128320924621710297 0.000000000000597545240963287883");
	m_poly.push_back("56750.360900069441 3.3894335002716018 -0.000128253641499606006 0.000000000000597231933220544829");
	m_poly.push_back("56750.360911643518 3.3893052785829689 -0.000128186387803012192 0.000000000000596918762500041538");
	m_poly.push_back("56750.360923217588 3.3891771277957781 -0.000128119200471469988 0.000000000000596605900814058385");
	m_poly.push_back("56750.360934791657 3.3890490405016669 -0.000128052016541669591 0.000000000000596293054967707405");
	m_poly.push_back("56750.360946365741 3.3889210240220802 -0.000127984929432723913 0.000000000000595980659976761908");
	m_poly.push_back("56750.360957939818 3.3887930709517700 -0.000127917828725692794 0.000000000000595668201663837953");
	m_poly.push_back("56750.360969513888 3.3886651886245271 -0.000127850834032212994 0.000000000000595356237013174673");
	m_poly.push_back("56750.360981087957 3.3885373696289411 -0.000127783795364781109 0.000000000000595044067591858167");
	m_poly.push_back("56750.360992662027 3.3884096212852040 -0.000127716825327268003 0.000000000000594732217751668125");
	m_poly.push_back("56750.361004236111 3.3882819361934380 -0.000127649937091431007 0.000000000000594420748829268049");
	m_poly.push_back("56750.361015810187 3.3881543216815180 -0.000127583045089697101 0.000000000000594109262368725930");
	m_poly.push_back("56750.361027384257 3.3880267703307720 -0.000127516217552302497 0.000000000000593798076092693224");
	m_poly.push_back("56750.361038958326 3.3878992894878510 -0.000127449539496146805 0.000000000000593487585891101144");
	m_poly.push_back("56750.361050532410 3.3877718717226708 -0.000127382684487244711 0.000000000000593176271688114607");
	m_poly.push_back("56750.361062106480 3.3876445243832589 -0.000127316006759716795 0.000000000000592865783014035121");
	m_poly.push_back("56750.361073680557 3.3875172400466278 -0.000127249318640444990 0.000000000000592555245948674979");
	m_poly.push_back("56750.361085254634 3.3873900260585650 -0.000127182736232742187 0.000000000000592245201139337777");
	m_poly.push_back("56750.361096828703 3.3872628749859621 -0.000127116093242060700 0.000000000000591934874217581218");
	m_poly.push_back("56750.361108402780 3.3871357941727909 -0.000127049620738151590 0.000000000000591625341184912630");
	m_poly.push_back("56750.361119976849 3.3870087762110090 -0.000126983043410856887 0.000000000000591315320029360704");
	m_poly.push_back("56750.361131550933 3.3868818284289208 -0.000126916558365800593 0.000000000000591005728594650555");
	m_poly.push_back("56750.361143125003 3.3867549434096542 -0.000126850137305464599 0.000000000000590696435111066505");
	m_poly.push_back("56750.361154699072 3.3866281284861541 -0.000126783701696687187 0.000000000000590387073879231460");
	m_poly.push_back("56750.361166273149 3.3865013762611280 -0.000126717393444504312 0.000000000000590078305696784472");
	m_poly.push_back("56750.361177847219 3.3863746940467019 -0.000126651025248834603 0.000000000000589769258378908033");
	m_poly.push_back("56750.361189421303 3.3862480744513692 -0.000126584811010847407 0.000000000000589460927981417849");
	m_poly.push_back("56750.361200995372 3.3861215247824239 -0.000126518603496407196 0.000000000000589152628891078674");
	m_poly.push_back("56750.361212569442 3.3859950376490562 -0.000126452374043841109 0.000000000000588844227642430949");
	m_poly.push_back("56750.361224143518 3.3858686203780879 -0.000126386223097791305 0.000000000000588536191966431142");
	m_poly.push_back("56750.361235717603 3.3857422655461882 -0.000126320094276385903 0.000000000000588228259315392018");
	m_poly.push_back("56750.361247291657 3.3856159805034660 -0.000126254013727739609 0.000000000000587920551450098704");
	m_poly.push_back("56750.361258865742 3.3854897578315071 -0.000126188014812014603 0.000000000000587613223716776198");
	m_poly.push_back("56750.361270439818 3.3853636048575280 -0.000126121977038272913 0.000000000000587305715034889128");
	m_poly.push_back("56750.361282013888 3.3852375141789262 -0.000126056077909934594 0.000000000000586998851970052975");
	m_poly.push_back("56750.361293587957 3.3851114931151862 -0.000125990101216554213 0.000000000000586691627713061559");
	m_poly.push_back("56750.361305162027 3.3849855342717992 -0.000125924255068345998 0.000000000000586385011353441578");
	m_poly.push_back("56750.361316736111 3.3848596449675341 -0.000125858476430945707 0.000000000000586078709363721712");
	m_poly.push_back("56750.361328310188 3.3847338177946962 -0.000125792612101626713 0.000000000000585772008338970203");
	m_poly.push_back("56750.361339884257 3.3846080600922099 -0.000125726878974735413 0.000000000000585465918271327579");
	m_poly.push_back("56750.361351458327 3.3844823644360331 -0.000125661152749027892 0.000000000000585159860338898485");
	m_poly.push_back("56750.361363032411 3.3843567381708102 -0.000125595498812554188 0.000000000000584854139027660261");
	m_poly.push_back("56750.361374606480 3.3842311738764890 -0.000125529797127349598 0.000000000000584548195368123339");
	m_poly.push_back("56750.361386180557 3.3841056788954762 -0.000125464242685162104 0.000000000000584242937360509219");
	m_poly.push_back("56750.361397754627 3.3839802458041550 -0.000125398683713240003 0.000000000000583937658258754898");
	m_poly.push_back("56750.361409328703 3.3838548819447518 -0.000125333125833783403 0.000000000000583632384242399647");
	m_poly.push_back("56750.361420902780 3.3837295799027580 -0.000125267657225623999 0.000000000000583327525926982785");
	m_poly.push_back("56750.361432476850 3.3836043470090509 -0.000125202229518302596 0.000000000000583022858069139709");
	m_poly.push_back("56750.361444050926 3.3834791758539522 -0.000125136796526290891 0.000000000000582718165601616008");
	m_poly.push_back("56750.361455625003 3.3833540737760370 -0.000125071472105798203 0.000000000000582413978707838824");
	m_poly.push_back("56750.361467199073 3.3832290333518631 -0.000125006117948930803 0.000000000000582109653342346585");
	m_poly.push_back("56750.361478773149 3.3831040619306592 -0.000124940847156941988 0.000000000000581805716173025505");
	m_poly.push_back("56750.361490347219 3.3829791520844070 -0.000124875654871469592 0.000000000000581502144577242431");
	m_poly.push_back("56750.361501921303 3.3828543111517519 -0.000124810407802855008 0.000000000000581198317875791894");
	m_poly.push_back("56750.361513495372 3.3827295317256092 -0.000124745258372223707 0.000000000000580894945835705105");
	m_poly.push_back("56750.361525069442 3.3826048211435511 -0.000124680123703197892 0.000000000000580591642532909239");
	m_poly.push_back("56750.361536643519 3.3824801719745330 -0.000124615013601320413 0.000000000000580288453628897649");
	m_poly.push_back("56750.361548217603 3.3823555915776100 -0.000124549927942245589 0.000000000000579985378543761895");
	m_poly.push_back("56750.361559791672 3.3822310725215510 -0.000124484911001907899 0.000000000000579682623454273692");
	m_poly.push_back("56750.361571365742 3.3821066221531391 -0.000124419968020588400 0.000000000000579380212761527128");
	m_poly.push_back("56750.361582939811 3.3819822330449858 -0.000124354958425526005 0.000000000000579077491873443703");
	m_poly.push_back("56750.361594513888 3.3818579125467161 -0.000124290105763559902 0.000000000000578775501760265800");
	m_poly.push_back("56750.361606087958 3.3817336532385029 -0.000124225314137545889 0.000000000000578473795866963423");
	m_poly.push_back("56750.361617662027 3.3816094624650850 -0.000124160476761213303 0.000000000000578171876930364012");
	m_poly.push_back("56750.361629236111 3.3814853327938001 -0.000124095680552173708 0.000000000000577870149692993638");
	m_poly.push_back("56750.361640810188 3.3813612715778110 -0.000124030893917749491 0.000000000000577568467039119838");
	m_poly.push_back("56750.361652384257 3.3812372713920320 -0.000123966181313398010 0.000000000000577267129113727998");
	m_poly.push_back("56750.361663958327 3.3811133395895951 -0.000123901546549425609 0.000000000000576966153659075458");
	m_poly.push_back("56750.361675532396 3.3809894687337469 -0.000123836895220835692 0.000000000000576665101068498242");
	m_poly.push_back("56750.361687106481 3.3808656661811338 -0.000123772330943084887 0.000000000000576364453838208943");
	m_poly.push_back("56750.361698680557 3.3807419245022272 -0.000123707827798986395 0.000000000000576064091282759755");
	m_poly.push_back("56750.361710254627 3.3806182510550928 -0.000123643304430955591 0.000000000000575763634550495010");
	m_poly.push_back("56750.361721828704 3.3804946383997709 -0.000123578844807835695 0.000000000000575463474652716460");
	m_poly.push_back("56750.361733402780 3.3803710938878782 -0.000123514407771215499 0.000000000000575163419929777152");
	m_poly.push_back("56750.361744976850 3.3802476100982699 -0.000123449983435615688 0.000000000000574863424349643374");
	m_poly.push_back("56750.361756550927 3.3801241943776130 -0.000123385684742416494 0.000000000000574564013835754640");
	m_poly.push_back("56750.361768125003 3.3800008392984711 -0.000123321278108308593 0.000000000000574264100681944138");
	m_poly.push_back("56750.361779699073 3.3798775522152988 -0.000123257018797154306 0.000000000000573964873552325176");
	m_poly.push_back("56750.361791273150 3.3797543257021321 -0.000123192785722932811 0.000000000000573665768597047417");
	m_poly.push_back("56750.361802847219 3.3796311670954990 -0.000123128563591128699 0.000000000000573366714594623694");
	m_poly.push_back("56750.361814421303 3.3795080689843950 -0.000123064455981671298 0.000000000000573068193877516443");
	m_poly.push_back("56750.361825995373 3.3793850387131168 -0.000123000265327081196 0.000000000000572769286449594707");
	m_poly.push_back("56750.361837569442 3.3792620688510979 -0.000122936230175612298 0.000000000000572471103138627491");
	m_poly.push_back("56750.361849143519 3.3791391667515431 -0.000122872170288240294 0.000000000000572172804640373452");
	m_poly.push_back("56750.361860717603 3.3790163249926328 -0.000122808189857740688 0.000000000000571874876141117923");
	m_poly.push_back("56750.361872291673 3.3788935509128630 -0.000122744171324180092 0.000000000000571576770208901765");
	m_poly.push_back("56750.361883865742 3.3787708370928171 -0.000122680269346905793 0.000000000000571279207033290170");
	m_poly.push_back("56750.361895439812 3.3786481908870329 -0.000122616322738423910 0.000000000000570981436025657367");
	m_poly.push_back("56750.361907013888 3.3785256048657519 -0.000122552546217031893 0.000000000000570684457046870174");
	m_poly.push_back("56750.361918587972 3.3784030863722032 -0.000122488735838094907 0.000000000000570387320404835120");
	m_poly.push_back("56750.361930162027 3.3782806279868178 -0.000122424934314344905 0.000000000000570090224997096564");
	m_poly.push_back("56750.361941736111 3.3781582370521059 -0.000122361095602362509 0.000000000000569792956416422467");
	m_poly.push_back("56750.361953310188 3.3780359061563479 -0.000122297439705136096 0.000000000000569496539132383552");
	m_poly.push_back("56750.361964884258 3.3779136426373162 -0.000122233778754146909 0.000000000000569200098313157593");
	m_poly.push_back("56750.361976458327 3.3777914390702111 -0.000122170195457018196 0.000000000000568904019097026629");
	m_poly.push_back("56750.361988032397 3.3776693028128340 -0.000122106573737082691 0.000000000000568607760958995495");
	m_poly.push_back("56750.361999606481 3.3775472264270050 -0.000122043050712068507 0.000000000000568311962404130101");
	m_poly.push_back("56750.362011180558 3.3774252172805310 -0.000121979484841094998 0.000000000000568015964330501954");
	m_poly.push_back("56750.362022754627 3.3773032679217092 -0.000121916046841862098 0.000000000000567720561705149154");
	m_poly.push_back("56750.362034328697 3.3771813857306689 -0.000121852603424711496 0.000000000000567425133848888548");
	m_poly.push_back("56750.362045902781 3.3770595632535620 -0.000121789156472591607 0.000000000000567129689530774700");
	m_poly.push_back("56750.362057476850 3.3769378078531589 -0.000121725736112677802 0.000000000000566834369040526709");
	m_poly.push_back("56750.362069050927 3.3768161121084770 -0.000121662499944203794 0.000000000000566539906257623121");
	m_poly.push_back("56750.362080624996 3.3766944833732389 -0.000121599204827008394 0.000000000000566245168971594813");
	m_poly.push_back("56750.362092199073 3.3765729141957248 -0.000121535921265076806 0.000000000000565950485493081418");
	m_poly.push_back("56750.362103773150 3.3764514119565270 -0.000121472821219565798 0.000000000000565656656577757215");
	m_poly.push_back("56750.362115347220 3.3763299692119029 -0.000121409652179980894 0.000000000000565362506382855141");
	m_poly.push_back("56750.362126921304 3.3762085933157389 -0.000121346475573074804 0.000000000000565068320948121884");
	m_poly.push_back("56750.362138495373 3.3760872768454879 -0.000121283380945481496 0.000000000000564774517258439278");
	m_poly.push_back("56750.362150069443 3.3759660271537548 -0.000121220327049971504 0.000000000000564480903240555021");
	m_poly.push_back("56750.362161643519 3.3758448367972349 -0.000121157271946532302 0.000000000000564187283596941405");
	m_poly.push_back("56750.362173217589 3.3757237131515501 -0.000121094325645539698 0.000000000000563894170602462012");
	m_poly.push_back("56750.362184791673 3.3756026487710882 -0.000121031367389600703 0.000000000000563601001937552156");
	m_poly.push_back("56750.362196365742 3.3754816510263219 -0.000120968368374932905 0.000000000000563307643473283551");
	m_poly.push_back("56750.362207939812 3.3753607124616520 -0.000120905522562991693 0.000000000000563014998413932773");
	m_poly.push_back("56750.362219513889 3.3752398404669299 -0.000120842756652014705 0.000000000000562722725420707581");
	m_poly.push_back("56750.362231087973 3.3751190275764000 -0.000120779902171405104 0.000000000000562430039992118056");
	m_poly.push_back("56750.362242662028 3.3749982811761172 -0.000120717161458186505 0.000000000000562137884332477804");
	m_poly.push_back("56750.362254236112 3.3748775938113700 -0.000120654445063424903 0.000000000000561845841913622990");
	m_poly.push_back("56750.362265810189 3.3747569728509381 -0.000120591727247569997 0.000000000000561553792875558147");
	m_poly.push_back("56750.362277384258 3.3746364108651048 -0.000120529111928060906 0.000000000000561262221122425832");
	m_poly.push_back("56750.362288958328 3.3745159152057940 -0.000120466520118762006 0.000000000000560970758845505873");
	m_poly.push_back("56750.362300532397 3.3743954784348378 -0.000120403844358383497 0.000000000000560678905640047462");
	m_poly.push_back("56750.362312106481 3.3742751079231321 -0.000120341338765031400 0.000000000000560387844834821590");
	m_poly.push_back("56750.362323680558 3.3741547962301088 -0.000120278900735778298 0.000000000000560097098648589078");
	m_poly.push_back("56750.362335254627 3.3740345507161020 -0.000120216460752521899 0.000000000000559806343361605002");
	m_poly.push_back("56750.362346828697 3.3739143639460689 -0.000120154017856024604 0.000000000000559515574507911135");
	m_poly.push_back("56750.362358402781 3.3737942432789390 -0.000120091608897008795 0.000000000000559224963686199971");
	m_poly.push_back("56750.362369976850 3.3736741812870150 -0.000120029283702554205 0.000000000000558934742922766916");
	m_poly.push_back("56750.362381550927 3.3735541853251418 -0.000119967000785621593 0.000000000000558644719027666243");
	m_poly.push_back("56750.362393124997 3.3734342479482762 -0.000119904666060960695 0.000000000000558354453883169278");
	m_poly.push_back("56750.362404699074 3.3733143765393478 -0.000119842401262966195 0.000000000000558064514358363358");
	m_poly.push_back("56750.362416273150 3.3731945636522549 -0.000119780252870266794 0.000000000000557775116887096952");
	m_poly.push_back("56750.362427847220 3.3730748166454738 -0.000119718066116933293 0.000000000000557485540783493673");
	m_poly.push_back("56750.362439421297 3.3729551280816299 -0.000119655922191795494 0.000000000000557196164113355902");
	m_poly.push_back("56750.362450995373 3.3728355053322110 -0.000119593826850280903 0.000000000000556907013676605744");
	m_poly.push_back("56750.362462569443 3.3727159409480310 -0.000119531801462077997 0.000000000000556618188984511691");
	m_poly.push_back("56750.362474143520 3.3725964423044581 -0.000119469780514791204 0.000000000000556329384970339377");
	m_poly.push_back("56750.362485717589 3.3724770019526331 -0.000119407820550165606 0.000000000000556040864927947274");
	m_poly.push_back("56750.362497291673 3.3723576272751830 -0.000119345847422664406 0.000000000000555752283589418167");
	m_poly.push_back("56750.362508865743 3.3722383108095491 -0.000119284013882040799 0.000000000000555464352252094562");
	m_poly.push_back("56750.362520439812 3.3721190599399300 -0.000119222130647565105 0.000000000000555176189508027268");
	m_poly.push_back("56750.362532013889 3.3719998672106688 -0.000119160291884424106 0.000000000000554888233848898047");
	m_poly.push_back("56750.362543587973 3.3718807400035549 -0.000119098450696542794 0.000000000000554600266896974199");
	m_poly.push_back("56750.362555162043 3.3717616708659768 -0.000119036712218171094 0.000000000000554312778222632128");
	m_poly.push_back("56750.362566736112 3.3716426671777899 -0.000118974970231475595 0.000000000000554025273209689351");
	m_poly.push_back("56750.362578310182 3.3715237214839848 -0.000118913307133215295 0.000000000000553738135548787116");
	m_poly.push_back("56750.362589884258 3.3714048411582480 -0.000118851681134368706 0.000000000000553451170643755188");
	m_poly.push_back("56750.362601458342 3.3712860187625471 -0.000118790075075235897 0.000000000000553164298589434302");
	m_poly.push_back("56750.362613032397 3.3711672616654340 -0.000118728491131865705 0.000000000000552877529517962508");
	m_poly.push_back("56750.362624606481 3.3710485624164681 -0.000118666953507251297 0.000000000000552590976134150477");
	m_poly.push_back("56750.362636180558 3.3709299283947471 -0.000118605455957464595 0.000000000000552304609361596195");
	m_poly.push_back("56750.362647754628 3.3708113521458150 -0.000118543937064634594 0.000000000000552018143201905141");
	m_poly.push_back("56750.362659328697 3.3706928410580521 -0.000118482609459828506 0.000000000000551732567794770962");
	m_poly.push_back("56750.362670902781 3.3705743876588210 -0.000118421117789814804 0.000000000000551446228398651866");
	m_poly.push_back("56750.362682476851 3.3704559993546410 -0.000118359810861914502 0.000000000000551160749273331601");
	m_poly.push_back("56750.362694050928 3.3703376686807109 -0.000118298505266289106 0.000000000000550875276351062882");
	m_poly.push_back("56750.362705624997 3.3702194030044810 -0.000118237252633030096 0.000000000000550590050052324837");
	m_poly.push_back("56750.362717199067 3.3701011948901969 -0.000118175974420093902 0.000000000000550304704638100927");
	m_poly.push_back("56750.362728773151 3.3699830517108000 -0.000118114765574268505 0.000000000000550019682238085448");
	m_poly.push_back("56750.362740347220 3.3698649660072921 -0.000118053611351712204 0.000000000000549734914196526262");
	m_poly.push_back("56750.362751921297 3.3697469451740578 -0.000117992481349912801 0.000000000000549450258939810197");
	m_poly.push_back("56750.362763495366 3.3696289817468679 -0.000117931375826443405 0.000000000000549165717668514082");
	m_poly.push_back("56750.362775069443 3.3695110831149830 -0.000117870329011886103 0.000000000000548881449780206346");
	m_poly.push_back("56750.362786643520 3.3693932418089401 -0.000117809263323434899 0.000000000000548597094002640208");
	m_poly.push_back("56750.362798217589 3.3692754652310350 -0.000117748304323554202 0.000000000000548313235030751330");
	m_poly.push_back("56750.362809791673 3.3691577459065658 -0.000117687328386019101 0.000000000000548029297185931393");
	m_poly.push_back("56750.362821365743 3.3690400912426148 -0.000117626412303152504 0.000000000000547745638059449218");
	m_poly.push_back("56750.362832939813 3.3689224937581121 -0.000117565542805496599 0.000000000000547462195861418525");
	m_poly.push_back("56750.362844513889 3.3688049608505710 -0.000117504700761584602 0.000000000000547178881503141054");
	m_poly.push_back("56750.362856087973 3.3686874850549700 -0.000117443857598561698 0.000000000000546895561932782287");
	m_poly.push_back("56750.362867662043 3.3685700737748170 -0.000117383094363148396 0.000000000000546612614552672847");
	m_poly.push_back("56750.362879236112 3.3684527195309379 -0.000117322406081518495 0.000000000000546330016202486233");
	m_poly.push_back("56750.362890810182 3.3683354297153030 -0.000117261605577936498 0.000000000000546046895276431771");
	m_poly.push_back("56750.362902384259 3.3682181968746558 -0.000117200992250089998 0.000000000000545764645954485248");
	m_poly.push_back("56750.362913958343 3.3681010283903730 -0.000117140367935416801 0.000000000000545482345469437432");
	m_poly.push_back("56750.362925532398 3.3679839168010659 -0.000117079734108301202 0.000000000000545200000687820510");
	m_poly.push_back("56750.362937106482 3.3678668695070568 -0.000117019246893592497 0.000000000000544918338621737805");
	m_poly.push_back("56750.362948680558 3.3677498790358031 -0.000116958679848974494 0.000000000000544636304817852464");
	m_poly.push_back("56750.362960254628 3.3676329527835480 -0.000116898191435218498 0.000000000000544354637165640560");
	m_poly.push_back("56750.362971828697 3.3675160832805231 -0.000116837716539611397 0.000000000000544073032461522285");
	m_poly.push_back("56750.362983402767 3.3673992779253230 -0.000116777357445334694 0.000000000000543791966997810303");
	m_poly.push_back("56750.362994976851 3.3672825292472659 -0.000116716921345571395 0.000000000000543510542948417138");
	m_poly.push_back("56750.363006550928 3.3671658446496129 -0.000116656680868166103 0.000000000000543230029834943193");
	m_poly.push_back("56750.363018124997 3.3670492166521480 -0.000116596335807184597 0.000000000000542949029715416456");
	m_poly.push_back("56750.363029699067 3.3669326526690662 -0.000116536075514411304 0.000000000000542668424326913733");
	m_poly.push_back("56750.363041273151 3.3668161452130150 -0.000116475879490577099 0.000000000000542388118213191386");
	m_poly.push_back("56750.363052847220 3.3666997016911702 -0.000116415627182571004 0.000000000000542107550004203151");
	m_poly.push_back("56750.363064421297 3.3665833146293029 -0.000116355433769995502 0.000000000000541827256047610912");
	m_poly.push_back("56750.363075995367 3.3664669914386010 -0.000116295394181874896 0.000000000000541547678390277612");
	m_poly.push_back("56750.363087569443 3.3663507246245450 -0.000116235207119809504 0.000000000000541267414003064717");
	m_poly.push_back("56750.363099143520 3.3662345216126281 -0.000116175200545459801 0.000000000000540987990075590768");
	m_poly.push_back("56750.363110717590 3.3661183749137100 -0.000116115147234908099 0.000000000000540708348514914019");
	m_poly.push_back("56750.363122291667 3.3660022919451440 -0.000116055181037368405 0.000000000000540429112604128774");
	m_poly.push_back("56750.363133865743 3.3658862652145092 -0.000115995226936884394 0.000000000000540149933023848370");
	m_poly.push_back("56750.363145439813 3.3657703021339471 -0.000115935305841289394 0.000000000000539870907132894962");
	m_poly.push_back("56750.363157013890 3.3656543952262168 -0.000115875470419849597 0.000000000000539592280192500912");
	m_poly.push_back("56750.363168587974 3.3655385519043630 -0.000115815629065345701 0.000000000000539313625622463431");
	m_poly.push_back("56750.363180162043 3.3654227646836699 -0.000115755799150642002 0.000000000000539035024322332131");
	m_poly.push_back("56750.363191736113 3.3653070409672741 -0.000115695955131588400 0.000000000000538756357342025059");
	m_poly.push_back("56750.363203310182 3.3651913732914549 -0.000115636342022412794 0.000000000000538478765618835160");
	m_poly.push_back("56750.363214884259 3.3650757690487110 -0.000115576658187206105 0.000000000000538200844550236508");
	m_poly.push_back("56750.363226458343 3.3649602207705618 -0.000115516966642568896 0.000000000000537922887580996900");
	m_poly.push_back("56750.363238032412 3.3648447358500140 -0.000115457333762434495 0.000000000000537645203787957184");
	m_poly.push_back("56750.363249606482 3.3647293068280049 -0.000115397751295580699 0.000000000000537367754749261804");
	m_poly.push_back("56750.363261180559 3.3646139410989999 -0.000115338196389052697 0.000000000000537090434046667998");
	m_poly.push_back("56750.363272754628 3.3644986311905818 -0.000115278671494236303 0.000000000000536813253096200111");
	m_poly.push_back("56750.363284328712 3.3643833845074349 -0.000115219157746119602 0.000000000000536536124049922622");
	m_poly.push_back("56750.363295902767 3.3642681935740661 -0.000115159690254585802 0.000000000000536259210401859838");
	m_poly.push_back("56750.363307476851 3.3641530657959771 -0.000115100319175342507 0.000000000000535982745706838792");
	m_poly.push_back("56750.363319050928 3.3640379936955820 -0.000115040921939102897 0.000000000000535706159208025890");
	m_poly.push_back("56750.363330624998 3.3639229846796992 -0.000114981560123610704 0.000000000000535429737648280527");
	m_poly.push_back("56750.363342199067 3.3638080312715761 -0.000114922287339606300 0.000000000000535153730672894333");
	m_poly.push_back("56750.363353773151 3.3636931408785000 -0.000114862952524884501 0.000000000000534877434842807060");
	m_poly.push_back("56750.363365347221 3.3635783060222000 -0.000114803757217103898 0.000000000000534601788641751064");
	m_poly.push_back("56750.363376921297 3.3634635341060051 -0.000114744544287767795 0.000000000000534326060382285411");
	m_poly.push_back("56750.363388495367 3.3633488176625832 -0.000114685365473549906 0.000000000000534050490982918498");
	m_poly.push_back("56750.363400069436 3.3632341640938028 -0.000114626261346660401 0.000000000000533775269371854299");
	m_poly.push_back("56750.363411643521 3.3631195659149560 -0.000114567128185060903 0.000000000000533499912556613050");
	m_poly.push_back("56750.363423217590 3.3630050305490480 -0.000114508071558210999 0.000000000000533224912132423163");
	m_poly.push_back("56750.363434791667 3.3628905505067261 -0.000114449103882912297 0.000000000000532950325920379040");
	m_poly.push_back("56750.363446365744 3.3627761331951218 -0.000114390013585035800 0.000000000000532675168700802086");
	m_poly.push_back("56750.363457939813 3.3626617711418909 -0.000114331138902522094 0.000000000000532401015517718970");
	m_poly.push_back("56750.363469513890 3.3625474717569932 -0.000114272197100001094 0.000000000000532126549781079869");
	m_poly.push_back("56750.363481087959 3.3624332275478190 -0.000114213259249895395 0.000000000000531852102448539635");
	m_poly.push_back("56750.363492662043 3.3623190459424488 -0.000114154413300109406 0.000000000000531578083058571992");
	m_poly.push_back("56750.363504236113 3.3622049194518788 -0.000114095572626131799 0.000000000000531304088235208170");
	m_poly.push_back("56750.363515810182 3.3620908554882458 -0.000114036784141801300 0.000000000000531030336437179646");
	m_poly.push_back("56750.363527384259 3.3619768465680679 -0.000113978108252459702 0.000000000000530757108949643776");
	m_poly.push_back("56750.363538958343 3.3618629001075502 -0.000113919333455032905 0.000000000000530483420883795827");
	m_poly.push_back("56750.363550532413 3.3617490086147690 -0.000113860652138891700 0.000000000000530210168123328610");
	m_poly.push_back("56750.363562106482 3.3616351795180321 -0.000113801982671115002 0.000000000000529936970534447522");
	m_poly.push_back("56750.363573680559 3.3615214053219979 -0.000113743428223067505 0.000000000000529664308547155028");
	m_poly.push_back("56750.363585254629 3.3614076934506398 -0.000113684848959180303 0.000000000000529391531000475298");
	m_poly.push_back("56750.363596828713 3.3612940364094661 -0.000113626330224980796 0.000000000000529119035316326609");
	m_poly.push_back("56750.363608402768 3.3611804416166349 -0.000113567829361027993 0.000000000000528846622845347565");
	m_poly.push_back("56750.363619976852 3.3610669015924128 -0.000113509312696295507 0.000000000000528574136795487893");
	m_poly.push_back("56750.363631550928 3.3609534237519552 -0.000113450904452090702 0.000000000000528302155616414904");
	m_poly.push_back("56750.363643124998 3.3608400006028321 -0.000113392473559213302 0.000000000000528030068970466213");
	m_poly.push_back("56750.363654699067 3.3607266395725590 -0.000113334141130329695 0.000000000000527758440831576921");
	m_poly.push_back("56750.363666273151 3.3606133331622869 -0.000113275781745084806 0.000000000000527486687166578675");
	m_poly.push_back("56750.363677847221 3.3605000888010719 -0.000113217481663834696 0.000000000000527215209655685960");
	m_poly.push_back("56750.363689421298 3.3603868989908681 -0.000113159267558721996 0.000000000000526944132501613060");
	m_poly.push_back("56750.363700995367 3.3602737711599291 -0.000113100960585169904 0.000000000000526672622893527638");
	m_poly.push_back("56750.363712569437 3.3601606978126748 -0.000113042805810697404 0.000000000000526401822016872582");
	m_poly.push_back("56750.363724143521 3.3600476863827740 -0.000112984720323399304 0.000000000000526131343782132785");
	m_poly.push_back("56750.363735717590 3.3599347293563819 -0.000112926465033229897 0.000000000000525860074840386641");
	m_poly.push_back("56750.363747291667 3.3598218341795429 -0.000112868428573646397 0.000000000000525589824906315722");
	m_poly.push_back("56750.363758865737 3.3597089933422799 -0.000112810444365882794 0.000000000000525319818287991379");
	m_poly.push_back("56750.363770439813 3.3595962142807498 -0.000112752314016449406 0.000000000000525049131142861504");
	m_poly.push_back("56750.363782013890 3.3594834894997399 -0.000112694347954269403 0.000000000000524779209019098836");
	m_poly.push_back("56750.363793587960 3.3593708264184090 -0.000112636399815627095 0.000000000000524509370356667167");
	m_poly.push_back("56750.363805162036 3.3592582175472860 -0.000112578466705045194 0.000000000000524239601673347067");
	m_poly.push_back("56750.363816736113 3.3591456703131208 -0.000112520543391124306 0.000000000000523969878607666924");
	m_poly.push_back("56750.363828310183 3.3590331772163911 -0.000112462709224155094 0.000000000000523700570664056866");
	m_poly.push_back("56750.363839884260 3.3589207456926342 -0.000112404850516683005 0.000000000000523431148443182756");
	m_poly.push_back("56750.363851458344 3.3588083682353620 -0.000112347179659965494 0.000000000000523162600969806513");
	m_poly.push_back("56750.363863032413 3.3586960522822511 -0.000112289367582113397 0.000000000000522893395882660892");
	m_poly.push_back("56750.363874606483 3.3585837903293649 -0.000112231696139195107 0.000000000000522624845677189380");
	m_poly.push_back("56750.363886180552 3.3584715898058959 -0.000112173982143417794 0.000000000000522356097317904404");
	m_poly.push_back("56750.363897754629 3.3583594432203339 -0.000112116342177713500 0.000000000000522087693687251132");
	m_poly.push_back("56750.363909328713 3.3582473579955550 -0.000112058654019186603 0.000000000000521819065639658179");
	m_poly.push_back("56750.363920902782 3.3581353266373428 -0.000112001135068448302 0.000000000000521551225526475352");
	m_poly.push_back("56750.363932476852 3.3580233565767070 -0.000111943601942305498 0.000000000000521283319402290552");
	m_poly.push_back("56750.363944050929 3.3579114403102892 -0.000111886115481310000 0.000000000000521015630578886758");
	m_poly.push_back("56750.363955624998 3.3577995852728679 -0.000111828621959257805 0.000000000000520747908873256376");
	m_poly.push_back("56750.363967199068 3.3576877839632049 -0.000111771178930422600 0.000000000000520480422294236946");
	m_poly.push_back("56750.363978773137 3.3575760438206550 -0.000111713683143503198 0.000000000000520212690039622597");
	m_poly.push_back("56750.363990347221 3.3574643573274021 -0.000111656330274148406 0.000000000000519945623296030778");
	m_poly.push_back("56750.364001921298 3.3573527319429188 -0.000111598963460316896 0.000000000000519678491616901030");
	m_poly.push_back("56750.364013495368 3.3572411601381109 -0.000111541672976952805 0.000000000000519411715378349611");
	m_poly.push_back("56750.364025069437 3.3571296493670522 -0.000111484414796813005 0.000000000000519145089561670589");
	m_poly.push_back("56750.364036643521 3.3570181921160489 -0.000111427187836313902 0.000000000000518878609122146792");
	m_poly.push_back("56750.364048217591 3.3569067958253851 -0.000111369949507069497 0.000000000000518612075741168101");
	m_poly.push_back("56750.364059791667 3.3567954529927229 -0.000111312895937702101 0.000000000000518346402714659297");
	m_poly.push_back("56750.364071365737 3.3566841710507300 -0.000111255657191011603 0.000000000000518079867387414695");
	m_poly.push_back("56750.364082939806 3.3565729424932309 -0.000111198561024376100 0.000000000000517813995999561070");
	m_poly.push_back("56750.364094513890 3.3564617747682921 -0.000111141519729701101 0.000000000000517548380127580659");
	m_poly.push_back("56750.364106087960 3.3563506603568340 -0.000111084436423958898 0.000000000000517282568625573499");
	m_poly.push_back("56750.364117662037 3.3562396067117160 -0.000111027521082548807 0.000000000000517017539266699203");
	m_poly.push_back("56750.364129236114 3.3561286063103970 -0.000110970499327904995 0.000000000000516752014381977000");
	m_poly.push_back("56750.364140810183 3.3560176666060659 -0.000110913562421405794 0.000000000000516486884600736168");
	m_poly.push_back("56750.364152384260 3.3559067800807090 -0.000110856639796893101 0.000000000000516221821324309593");
	m_poly.push_back("56750.364163958337 3.3557959541815210 -0.000110799765445139395 0.000000000000515956982833803616");
	m_poly.push_back("56750.364175532413 3.3556851814023880 -0.000110742941675420994 0.000000000000515692379883538884");
	m_poly.push_back("56750.364187106483 3.3555744691811231 -0.000110686168763074794 0.000000000000515428013754780906");
	m_poly.push_back("56750.364198680552 3.3554638100088048 -0.000110629379126238298 0.000000000000515163569745789787");
	m_poly.push_back("56750.364210254629 3.3553532113328459 -0.000110572680261728605 0.000000000000514899548426774167");
	m_poly.push_back("56750.364221828713 3.3552426656259020 -0.000110515951341098097 0.000000000000514635387147309735");
	m_poly.push_back("56750.364233402783 3.3551321803614091 -0.000110459268881332896 0.000000000000514371442216510828");
	m_poly.push_back("56750.364244976852 3.3550217479964370 -0.000110402629356345296 0.000000000000514107697215610500");
	m_poly.push_back("56750.364256550929 3.3549113760045191 -0.000110346052830194704 0.000000000000513844245574293815");
	m_poly.push_back("56750.364268124998 3.3548010568488160 -0.000110289441824770902 0.000000000000513580633375626760");
	m_poly.push_back("56750.364279699083 3.3546907979897829 -0.000110232904831656200 0.000000000000513317365822040210");
	m_poly.push_back("56750.364291273137 3.3545805919060649 -0.000110176430171241195 0.000000000000513054388526964651");
	m_poly.push_back("56750.364302847222 3.3544704460542718 -0.000110119924530994805 0.000000000000512791266969199497");
	m_poly.push_back("56750.364314421298 3.3543603529145489 -0.000110063512478616204 0.000000000000512528581213183624");
	m_poly.push_back("56750.364325995368 3.3542503199362610 -0.000110007101802921499 0.000000000000512265901866255932");
	m_poly.push_back("56750.364337569437 3.3541403396018450 -0.000109950678648252398 0.000000000000512003164408845630");
	m_poly.push_back("56750.364349143521 3.3540304193679038 -0.000109894433174639904 0.000000000000511741254341837626");
	m_poly.push_back("56750.364360717591 3.3539205517049990 -0.000109838083899052501 0.000000000000511478860908387278");
	m_poly.push_back("56750.364372291668 3.3538107440764349 -0.000109781815830058496 0.000000000000511216845620896362");
	m_poly.push_back("56750.364383865737 3.3537009889607758 -0.000109725641313404898 0.000000000000510955265969826949");
	m_poly.push_back("56750.364395439807 3.3535912938024039 -0.000109669358402868903 0.000000000000510693181568821618");
	m_poly.push_back("56750.364407013891 3.3534816510920229 -0.000109613217681587500 0.000000000000510431759287400425");
	m_poly.push_back("56750.364418587960 3.3533720682785289 -0.000109557108464165405 0.000000000000510170483705651257");
	m_poly.push_back("56750.364430162037 3.3532625378434582 -0.000109500960841700195 0.000000000000509909029285774556");
	m_poly.push_back("56750.364441736107 3.3531530672438450 -0.000109444924659585802 0.000000000000509648093798868446");
	m_poly.push_back("56750.364453310183 3.3530436489497162 -0.000109388890768984199 0.000000000000509387168981940449");
	m_poly.push_back("56750.364464884260 3.3529342904342991 -0.000109332921182848903 0.000000000000509126543604440342");
	m_poly.push_back("56750.364476458330 3.3528249841480480 -0.000109276916495712101 0.000000000000508865754774434092");
	m_poly.push_back("56750.364488032406 3.3527157375761401 -0.000109220943898635700 0.000000000000508605115373837692");
	m_poly.push_back("56750.364499606483 3.3526065431754559 -0.000109165056593795598 0.000000000000508344873145439680");
	m_poly.push_back("56750.364511180553 3.3524974084139640 -0.000109109194211377098 0.000000000000508084746969494587");
	m_poly.push_back("56750.364522754629 3.3523883257633669 -0.000109053365002602498 0.000000000000507824775269680853");
	m_poly.push_back("56750.364534328713 3.3522793026860049 -0.000108997590434860699 0.000000000000507565058010437345");
	m_poly.push_back("56750.364545902783 3.3521703316458882 -0.000108941752619590496 0.000000000000507305046231205445");
	m_poly.push_back("56750.364557476852 3.3520614201223680 -0.000108886012762049597 0.000000000000507045490601611112");
	m_poly.push_back("56750.364569050929 3.3519525605740390 -0.000108830325165210404 0.000000000000506786178329161766");
	m_poly.push_back("56750.364580624999 3.3518437604673692 -0.000108774590743350896 0.000000000000506526648009131916");
	m_poly.push_back("56750.364592199083 3.3517350122735450 -0.000108718934019761001 0.000000000000506267479499158620");
	m_poly.push_back("56750.364603773152 3.3516263234607488 -0.000108663427621184095 0.000000000000506009010992975997");
	m_poly.push_back("56750.364615347222 3.3515176864846179 -0.000108607810705966902 0.000000000000505750027852892270");
	m_poly.push_back("56750.364626921299 3.3514091088283151 -0.000108552229016096899 0.000000000000505491208742049202");
	m_poly.push_back("56750.364638495368 3.3513005829509090 -0.000108496760480771400 0.000000000000505232916547390723");
	m_poly.push_back("56750.364650069438 3.3511921163210059 -0.000108441274572581794 0.000000000000504974543452456627");
	m_poly.push_back("56750.364661643522 3.3510837014057180 -0.000108385833286718002 0.000000000000504716378145706701");
	m_poly.push_back("56750.364673217591 3.3509753456719058 -0.000108330373295715195 0.000000000000504458125734786983");
	m_poly.push_back("56750.364684791668 3.3508670415869699 -0.000108275033919996006 0.000000000000504200434981845891");
	m_poly.push_back("56750.364696365737 3.3507587966232930 -0.000108219636670257207 0.000000000000503942474730425944");
	m_poly.push_back("56750.364707939807 3.3506506032379670 -0.000108164369983454196 0.000000000000503685122459347727");
	m_poly.push_back("56750.364719513891 3.3505424689101542 -0.000108109001368742906 0.000000000000503427295547879343");
	m_poly.push_back("56750.364731087961 3.3504343860987742 -0.000108053814254276096 0.000000000000503170313812089240");
	m_poly.push_back("56750.364742662037 3.3503263622730759 -0.000107998564682764095 0.000000000000502913041236440219");
	m_poly.push_back("56750.364754236107 3.3502183899086599 -0.000107943402668355697 0.000000000000502656176379630647");
	m_poly.push_back("56750.364765810184 3.3501104764609790 -0.000107888262787473498 0.000000000000502399414588493342");
	m_poly.push_back("56750.364777384260 3.3500026144040000 -0.000107833146807602203 0.000000000000502142764094430514");
	m_poly.push_back("56750.364788958330 3.3498948112041158 -0.000107778006615855904 0.000000000000501886000853463598");
	m_poly.push_back("56750.364800532407 3.3497870593293531 -0.000107722961166615506 0.000000000000501629678790975986");
	m_poly.push_back("56750.364812106483 3.3496793662541888 -0.000107667994241655203 0.000000000000501373722383982185");
	m_poly.push_back("56750.364823680553 3.3495717244260939 -0.000107612960436497096 0.000000000000501117454541212204");
	m_poly.push_back("56750.364835254630 3.3494641413363921 -0.000107558007980041706 0.000000000000500861565506415241");
	m_poly.push_back("56750.364846828707 3.3493566094369709 -0.000107503051473470699 0.000000000000500605657610978216");
	m_poly.push_back("56750.364858402783 3.3492491362047341 -0.000107448191983154694 0.000000000000500350201481122824");
	m_poly.push_back("56750.364869976853 3.3491417140983049 -0.000107393295393424897 0.000000000000500094572592959507");
	m_poly.push_back("56750.364881550922 3.3490343506016180 -0.000107338514569498307 0.000000000000499839482779756061");
	m_poly.push_back("56750.364893124999 3.3489270381606162 -0.000107283698093896596 0.000000000000499584226949815350");
	m_poly.push_back("56750.364904699083 3.3488197842628020 -0.000107228938604131304 0.000000000000499329236479285698");
	m_poly.push_back("56750.364916273153 3.3487125813597962 -0.000107174211177780897 0.000000000000499074395314980811");
	m_poly.push_back("56750.364927847222 3.3486054369328029 -0.000107119448836947496 0.000000000000498819391565881031");
	m_poly.push_back("56750.364939421299 3.3484983434379409 -0.000107064794685714196 0.000000000000498564891613177853");
	m_poly.push_back("56750.364950995368 3.3483913083559580 -0.000107010119555592796 0.000000000000498310293968339222");
	m_poly.push_back("56750.364962569452 3.3482843241431892 -0.000106955537169565104 0.000000000000498056128196167307");
	m_poly.push_back("56750.364974143522 3.3481773982765022 -0.000106900955698366104 0.000000000000497801966682450835");
	m_poly.push_back("56750.364985717591 3.3480705232148522 -0.000106846424045364997 0.000000000000497548037152097158");
	m_poly.push_back("56750.364997291668 3.3479637064421381 -0.000106791905102265896 0.000000000000497294166805274427");
	m_poly.push_back("56750.365008865738 3.3478569404041059 -0.000106737399437506296 0.000000000000497040358289857455");
	m_poly.push_back("56750.365020439807 3.3477502325835271 -0.000106682966399489898 0.000000000000496786887967520894");
	m_poly.push_back("56750.365032013891 3.3476435754410718 -0.000106628500178947803 0.000000000000496533263126314534");
	m_poly.push_back("56750.365043587961 3.3475369764633820 -0.000106574211417416600 0.000000000000496280464641462509");
	m_poly.push_back("56750.365055162038 3.3474304280889631 -0.000106519856148724494 0.000000000000496027356457831498");
	m_poly.push_back("56750.365066736107 3.3473239378094219 -0.000106465515588346897 0.000000000000495774316763578453");
	m_poly.push_back("56750.365078310177 3.3472174980777289 -0.000106411227937044695 0.000000000000495521523445720525");
	m_poly.push_back("56750.365089884261 3.3471111163767282 -0.000106356984428449398 0.000000000000495268935681794472");
	m_poly.push_back("56750.365101458330 3.3470047851614471 -0.000106302731736039396 0.000000000000495016305151731877");
	m_poly.push_back("56750.365113032407 3.3468985119111880 -0.000106248606799906305 0.000000000000494764269531643921");
	m_poly.push_back("56750.365124606476 3.3467922890768111 -0.000106194413251618099 0.000000000000494511914410630577");
	m_poly.push_back("56750.365136180553 3.3466861241519021 -0.000106140266608286700 0.000000000000494259777706287403");
	m_poly.push_back("56750.365147754630 3.3465800095762610 -0.000106086166576811603 0.000000000000494007858054643046");
	m_poly.push_back("56750.365159328707 3.3464739528441991 -0.000106032046765495695 0.000000000000493755846294496858");
	m_poly.push_back("56750.365170902784 3.3463679464082809 -0.000105978035650044597 0.000000000000493504340688130126");
	m_poly.push_back("56750.365182476853 3.3462619977535439 -0.000105924141774780701 0.000000000000493253381022266814");
	m_poly.push_back("56750.365194050923 3.3461560993178749 -0.000105870152819501195 0.000000000000493001978605019867");
	m_poly.push_back("56750.365205624999 3.3460502585992131 -0.000105816155959390996 0.000000000000492750539376492645");
	m_poly.push_back("56750.365217199083 3.3459444680445030 -0.000105762262155181699 0.000000000000492499580038725696");
	m_poly.push_back("56750.365228773153 3.3458387351434800 -0.000105708401187106795 0.000000000000492248773604571901");
	m_poly.push_back("56750.365240347222 3.3457330523448570 -0.000105654539819349406 0.000000000000491997965308588432");
	m_poly.push_back("56750.365251921299 3.3456274271316548 -0.000105600682626053196 0.000000000000491747176449920472");
	m_poly.push_back("56750.365263495369 3.3455218519594450 -0.000105546937690236393 0.000000000000491496910330197525");
	m_poly.push_back("56750.365275069453 3.3454163343197920 -0.000105493234996414405 0.000000000000491246840913541283");
	m_poly.push_back("56750.365286643508 3.3453108666430320 -0.000105439474020008295 0.000000000000490996500096793690");
	m_poly.push_back("56750.365298217592 3.3452054564414779 -0.000105385799783577305 0.000000000000490746563192994326");
	m_poly.push_back("56750.365309791669 3.3451000961453330 -0.000105332166323638605 0.000000000000490496816168862730");
	m_poly.push_back("56750.365321365738 3.3449947932503719 -0.000105278473763987301 0.000000000000490246793938690705");
	m_poly.push_back("56750.365332939808 3.3448895402015029 -0.000105224944949797698 0.000000000000489997534207096697");
	m_poly.push_back("56750.365344513892 3.3447843444983700 -0.000105171360242236906 0.000000000000489748014200188437");
	m_poly.push_back("56750.365356087961 3.3446791985718649 -0.000105117796327045703 0.000000000000489498591014616486");
	m_poly.push_back("56750.365367662038 3.3445741099311128 -0.000105064315714560393 0.000000000000489249555735954916");
	m_poly.push_back("56750.365379236107 3.3444690710093239 -0.000105010878614171895 0.000000000000489000723075626254");
	m_poly.push_back("56750.365390810177 3.3443640893015809 -0.000104957439861762497 0.000000000000488751882720812844");
	m_poly.push_back("56750.365402384261 3.3442591572532061 -0.000104904019423691493 0.000000000000488503127648249806");
	m_poly.push_back("56750.365413958330 3.3441542823594279 -0.000104850644442838599 0.000000000000488254584250944906");
	m_poly.push_back("56750.365425532407 3.3440494570606938 -0.000104797282349524505 0.000000000000488006100865133913");
	m_poly.push_back("56750.365437106477 3.3439446888516580 -0.000104743962587023405 0.000000000000487757814596012647");
	m_poly.push_back("56750.365448680554 3.3438399701738231 -0.000104690740435860196 0.000000000000487509982864431988");
	m_poly.push_back("56750.365460254630 3.3437353085317199 -0.000104637466823360195 0.000000000000487261911495780501");
	m_poly.push_back("56750.365471828707 3.3436306963534870 -0.000104584234937708597 0.000000000000487014034432268089");
	m_poly.push_back("56750.365483402777 3.3435261411486970 -0.000104531075367936304 0.000000000000486766494114277993");
	m_poly.push_back("56750.365494976853 3.3434216353448840 -0.000104477833143831901 0.000000000000486518568906406922");
	m_poly.push_back("56750.365506550923 3.3433171864562050 -0.000104424786204575904 0.000000000000486271553062962358");
	m_poly.push_back("56750.365518125000 3.3432127869055090 -0.000104371699572410994 0.000000000000486024352384337079");
	m_poly.push_back("56750.365529699076 3.3431084442024641 -0.000104318644968133696 0.000000000000485777300845638324");
	m_poly.push_back("56750.365541273153 3.3430041507741768 -0.000104265615588260304 0.000000000000485530366766590187");
	m_poly.push_back("56750.365552847223 3.3428999141351992 -0.000104212565060744002 0.000000000000485283334209625002");
	m_poly.push_back("56750.365564421299 3.3427957267160910 -0.000104159605802937305 0.000000000000485036726659717148");
	m_poly.push_back("56750.365575995369 3.3426915960226689 -0.000104106685598547399 0.000000000000484790300964930488");
	m_poly.push_back("56750.365587569453 3.3425875144769819 -0.000104053713746302297 0.000000000000484543634765424151");
	m_poly.push_back("56750.365599143523 3.3424834895989188 -0.000104000854364700494 0.000000000000484297492296636326");
	m_poly.push_back("56750.365610717592 3.3423795138104571 -0.000103948021797350696 0.000000000000484051474690794279");
	m_poly.push_back("56750.365622291669 3.3422755946285858 -0.000103895228878500700 0.000000000000483805641711258026");
	m_poly.push_back("56750.365633865738 3.3421717244737810 -0.000103842407804241899 0.000000000000483559677622122481");
	m_poly.push_back("56750.365645439822 3.3420679108615059 -0.000103789707140984703 0.000000000000483314274238761527");
	m_poly.push_back("56750.365657013892 3.3419641462120082 -0.000103736970852697806 0.000000000000483068704962907109");
	m_poly.push_back("56750.365668587961 3.3418604380482342 -0.000103684292953576997 0.000000000000482823407581273691");
	m_poly.push_back("56750.365680162038 3.3417567787878690 -0.000103631540713519204 0.000000000000482577764022033199");
	m_poly.push_back("56750.365691736108 3.3416531759512300 -0.000103578946383559196 0.000000000000482332855787476648");
	m_poly.push_back("56750.365703310177 3.3415496219499321 -0.000103526424813569899 0.000000000000482088286367285546");
	m_poly.push_back("56750.365714884261 3.3414461243095488 -0.000103473782566124197 0.000000000000481843154997375316");
	m_poly.push_back("56750.365726458331 3.3413426754567479 -0.000103421251217237603 0.000000000000481598540038485581");
	m_poly.push_back("56750.365738032408 3.3412392828956290 -0.000103368787414686097 0.000000000000481354239615216024");
	m_poly.push_back("56750.365749606477 3.3411359390528679 -0.000103316270427722395 0.000000000000481109691532085411");
	m_poly.push_back("56750.365761180547 3.3410326514525348 -0.000103263829709053298 0.000000000000480865498599365399");
	m_poly.push_back("56750.365772754631 3.3409294125098450 -0.000103211422270610106 0.000000000000480621460639132269");
	m_poly.push_back("56750.365784328700 3.3408262297336289 -0.000103158993791105998 0.000000000000480377324697305301");
	m_poly.push_back("56750.365795902777 3.3407230955631180 -0.000103106663997641401 0.000000000000480133648297528288");
	m_poly.push_back("56750.365807476846 3.3406200175052381 -0.000103054317515213794 0.000000000000479889894182209168");
	m_poly.push_back("56750.365819050923 3.3405169879836589 -0.000103002024510298902 0.000000000000479646389090318155");
	m_poly.push_back("56750.365830625000 3.3404140145197578 -0.000102949846400767595 0.000000000000479403419020229982");
	m_poly.push_back("56750.365842199077 3.3403110895229191 -0.000102897643066832302 0.000000000000479160331489231538");
	m_poly.push_back("56750.365853773154 3.3402082205262018 -0.000102845397650775300 0.000000000000478917047996572249");
	m_poly.push_back("56750.365865347223 3.3401053999354442 -0.000102793176162374594 0.000000000000478673875925113904");
	m_poly.push_back("56750.365876921300 3.3400026352830059 -0.000102741066904807904 0.000000000000478431226467687717");
	m_poly.push_back("56750.365888495369 3.3398999189810352 -0.000102688944137974104 0.000000000000478188514102208209");
	m_poly.push_back("56750.365900069453 3.3397972585541091 -0.000102636892381390203 0.000000000000477946132402471798");
	m_poly.push_back("56750.365911643523 3.3396946464190411 -0.000102584816199767203 0.000000000000477703636964146960");
	m_poly.push_back("56750.365923217592 3.3395920900910441 -0.000102532778716287905 0.000000000000477461321726564118");
	m_poly.push_back("56750.365934791669 3.3394895819955419 -0.000102480769912245294 0.000000000000477219140037412645");
	m_poly.push_back("56750.365946365739 3.3393871296578528 -0.000102428840645011693 0.000000000000476977328718891068");
	m_poly.push_back("56750.365957939823 3.3392847254805931 -0.000102376902913391702 0.000000000000476735477984365524");
	m_poly.push_back("56750.365969513892 3.3391823769989490 -0.000102325046317310495 0.000000000000476494005065016125");
	m_poly.push_back("56750.365981087962 3.3390800766213848 -0.000102273140942205906 0.000000000000476252305000126796");
	m_poly.push_back("56750.365992662038 3.3389778318806500 -0.000102221281397356394 0.000000000000476010818347603871");
	m_poly.push_back("56750.366004236108 3.3388756351895932 -0.000102169525281444904 0.000000000000475769813322867494");
	m_poly.push_back("56750.366015810177 3.3387734940686919 -0.000102117724809662496 0.000000000000475528601748515517");
	m_poly.push_back("56750.366027384262 3.3386714009320340 -0.000102065983952738900 0.000000000000475287667776854578");
	m_poly.push_back("56750.366038958331 3.3385693633095892 -0.000102014246284393205 0.000000000000475046748651673208");
	m_poly.push_back("56750.366050532408 3.3384673736095452 -0.000101962559668820007 0.000000000000474806067258860660");
	m_poly.push_back("56750.366062106477 3.3383654393662270 -0.000101910916663043802 0.000000000000474565588938487627");
	m_poly.push_back("56750.366073680547 3.3382635529888161 -0.000101859339436118104 0.000000000000474325416924140303");
	m_poly.push_back("56750.366085254631 3.3381617219987580 -0.000101807675611326795 0.000000000000474084841655563043");
	m_poly.push_back("56750.366096828700 3.3380599388183061 -0.000101756137428936198 0.000000000000473844851454384053");
	m_poly.push_back("56750.366108402777 3.3379582109751040 -0.000101704587318244806 0.000000000000473604805706228883");
	m_poly.push_back("56750.366119976847 3.3378565308644439 -0.000101653027828338304 0.000000000000473364716282083128");
	m_poly.push_back("56750.366131550923 3.3377549060389722 -0.000101601616354168406 0.000000000000473125316108429358");
	m_poly.push_back("56750.366143125000 3.3376533288941701 -0.000101550134455949604 0.000000000000472885587996624008");
	m_poly.push_back("56750.366154699077 3.3375518069610579 -0.000101498724198590700 0.000000000000472646193487068378");
	m_poly.push_back("56750.366166273147 3.3374503326549099 -0.000101447381221112098 0.000000000000472407112273261285");
	m_poly.push_back("56750.366177847223 3.3373489135010201 -0.000101395969889051505 0.000000000000472167712757134111");
	m_poly.push_back("56750.366189421293 3.3372475419127809 -0.000101344597725871603 0.000000000000471928495634709042");
	m_poly.push_back("56750.366200995370 3.3371462254237318 -0.000101293374217920304 0.000000000000471689970740752583");
	m_poly.push_back("56750.366212569454 3.3370449564330409 -0.000101242058241212001 0.000000000000471451015254871064");
	m_poly.push_back("56750.366224143523 3.3369437424794288 -0.000101190857843788502 0.000000000000471212597975388063");
	m_poly.push_back("56750.366235717593 3.3368425759707332 -0.000101139574516696093 0.000000000000470973794524026716");
	m_poly.push_back("56750.366247291669 3.3367414644389020 -0.000101088388063749394 0.000000000000470735442176556534");
	m_poly.push_back("56750.366258865739 3.3366404002940548 -0.000101037321924103994 0.000000000000470497650080958910");
	m_poly.push_back("56750.366270439823 3.3365393910638939 -0.000100986147452749201 0.000000000000470259353525030469");
	m_poly.push_back("56750.366282013892 3.3364384291555531 -0.000100935094289460905 0.000000000000470021621853208635");
	m_poly.push_back("56750.366293587962 3.3363375221107021 -0.000100883996192982600 0.000000000000469783680943446227");
	m_poly.push_back("56750.366305162039 3.3362366623310589 -0.000100832965012229495 0.000000000000469546051633687360");
	m_poly.push_back("56750.366316736108 3.3361358573484319 -0.000100781926281918798 0.000000000000469308387167072955");
	m_poly.push_back("56750.366328310192 3.3360350995722139 -0.000100731040487879794 0.000000000000469071434864868193");
	m_poly.push_back("56750.366339884262 3.3359343965390020 -0.000100680059711525402 0.000000000000468834040265299968");
	m_poly.push_back("56750.366351458331 3.3358337406445142 -0.000100629122660431794 0.000000000000468596849276746717");
	m_poly.push_back("56750.366363032408 3.3357331394451508 -0.000100578291409725396 0.000000000000468360150958141137");
	m_poly.push_back("56750.366374606478 3.3356325853183959 -0.000100527425617572905 0.000000000000468123291792805056");
	m_poly.push_back("56750.366386180547 3.3355320858221309 -0.000100476580369098705 0.000000000000467886528289961396");
	m_poly.push_back("56750.366397754631 3.3354316333492009 -0.000100425832358925194 0.000000000000467650217587622677");
	m_poly.push_back("56750.366409328701 3.3353312354483191 -0.000100375033873298494 0.000000000000467413671839161556");
	m_poly.push_back("56750.366420902777 3.3352308845071001 -0.000100324304239636302 0.000000000000467177446707041315");
	m_poly.push_back("56750.366432476847 3.3351305880761908 -0.000100273611136950596 0.000000000000466941391684053336");
	m_poly.push_back("56750.366444050916 3.3350303385522748 -0.000100222950914808501 0.000000000000466705489772432919");
	m_poly.push_back("56750.366455625001 3.3349301434774330 -0.000100172240261622001 0.000000000000466469353021480710");
	m_poly.push_back("56750.366467199077 3.3348299952506770 -0.000100121667978502195 0.000000000000466233860605768647");
	m_poly.push_back("56750.366478773147 3.3347299014177172 -0.000100071044891301096 0.000000000000465998131613649545");
	m_poly.push_back("56750.366490347224 3.3346298543686199 -0.000100020453894160506 0.000000000000465762552051914693");
	m_poly.push_back("56750.366501921293 3.3345298616534960 -0.000099969995831404932 0.000000000000465527591512676736");
	m_poly.push_back("56750.366513495370 3.3344299156661199 -0.000099919481750932620 0.000000000000465292370120019994");
	m_poly.push_back("56750.366525069447 3.3343300239588780 -0.000099869021041391057 0.000000000000465057397253644703");
	m_poly.push_back("56750.366536643523 3.3342301789164628 -0.000099818579712007739 0.000000000000464822514632578015");
	m_poly.push_back("56750.366548217593 3.3341303880963769 -0.000099768096264975291 0.000000000000464587435884462321");
	m_poly.push_back("56750.366559791670 3.3340306438840970 -0.000099717825822046821 0.000000000000464353349013396329");
	m_poly.push_back("56750.366571365739 3.3339309538344168 -0.000099667394546897763 0.000000000000464118513207462067");
	m_poly.push_back("56750.366582939823 3.3338313103318300 -0.000099617176604480462 0.000000000000463884670808889913");
	m_poly.push_back("56750.366594513893 3.3337317209335282 -0.000099566783316928453 0.000000000000463650011894431146");
	m_poly.push_back("56750.366606087962 3.3336321780265221 -0.000099516544315686600 0.000000000000463416071431005669");
	m_poly.push_back("56750.366617662039 3.3335326891636869 -0.000099466276803762826 0.000000000000463181998202906291");
	m_poly.push_back("56750.366629236109 3.3334332467348320 -0.000099416018875336202 0.000000000000462947969600850883");
	m_poly.push_back("56750.366640810193 3.3333338582949450 -0.000099365847509247889 0.000000000000462714344084605198");
	m_poly.push_back("56750.366652384262 3.3332345162278050 -0.000099315740580853431 0.000000000000462481018629189067");
	m_poly.push_back("56750.366663958332 3.3331352280922060 -0.000099265594963197191 0.000000000000462247513011285115");
	m_poly.push_back("56750.366675532408 3.3330359862720149 -0.000099215521892347756 0.000000000000462014345215296586");
	m_poly.push_back("56750.366687106478 3.3329367983266192 -0.000099165493088550883 0.000000000000461781383552492442");
	m_poly.push_back("56750.366698680547 3.3328376566355140 -0.000099115396534136665 0.000000000000461548106400583934");
	m_poly.push_back("56750.366710254631 3.3327385687574291 -0.000099065363911151865 0.000000000000461315126951139745");
	m_poly.push_back("56750.366721828701 3.3326395270782849 -0.000099015460936274229 0.000000000000461082751222146392");
	m_poly.push_back("56750.366733402778 3.3325405391624350 -0.000098965515710520009 0.000000000000460850178745706458");
	m_poly.push_back("56750.366744976847 3.3324415973838351 -0.000098915545580107795 0.000000000000460617490297341436");
	m_poly.push_back("56750.366756550917 3.3323427093022171 -0.000098865664871983922 0.000000000000460385218252489521");
	m_poly.push_back("56750.366768125001 3.3322438673102019 -0.000098815846940651287 0.000000000000460153238534275724");
	m_poly.push_back("56750.366779699078 3.3321450789560489 -0.000098766037376004753 0.000000000000459921297775132616");
	m_poly.push_back("56750.366791273147 3.3320463366284820 -0.000098716191502427490 0.000000000000459689187938663076");
	m_poly.push_back("56750.366802847217 3.3319476478840242 -0.000098666393253235335 0.000000000000459457299868928223");
	m_poly.push_back("56750.366814421293 3.3318490051103469 -0.000098616694462802171 0.000000000000459225874939683766");
	m_poly.push_back("56750.366825995370 3.3317504158587452 -0.000098566923285435158 0.000000000000458994112931060354");
	m_poly.push_back("56750.366837569447 3.3316518725180670 -0.000098517263832637546 0.000000000000458762871179921502");
	m_poly.push_back("56750.366849143516 3.3315533826506378 -0.000098467574119437081 0.000000000000458531488516349626");
	m_poly.push_back("56750.366860717593 3.3314549386318091 -0.000098417919223019523 0.000000000000458300267980476693");
	m_poly.push_back("56750.366872291670 3.3313565480316041 -0.000098368345595368164 0.000000000000458069425880332174");
	m_poly.push_back("56750.366883865740 3.3312582032187779 -0.000098318804990369559 0.000000000000457838737553322427");
	m_poly.push_back("56750.366895439824 3.3311599117671800 -0.000098269238423774949 0.000000000000457607928331758218");
	m_poly.push_back("56750.366907013893 3.3310616660460730 -0.000098219724757373937 0.000000000000457377365445341672");
	m_poly.push_back("56750.366918587963 3.3309634736317228 -0.000098170276452377406 0.000000000000457147106920240430");
	m_poly.push_back("56750.366930162039 3.3308653268903261 -0.000098120804201960778 0.000000000000456916736890005362");
	m_poly.push_back("56750.366941736109 3.3307672333945400 -0.000098071371742153163 0.000000000000456686552147768577");
	m_poly.push_back("56750.366953310193 3.3306691855182020 -0.000098021941982422622 0.000000000000456456379978187278");
	m_poly.push_back("56750.366964884262 3.3305711908275688 -0.000097972583126359892 0.000000000000456226537978073727");
	m_poly.push_back("56750.366976458332 3.3304732416982890 -0.000097923158509210292 0.000000000000455996389753481327");
	m_poly.push_back("56750.366988032409 3.3303753457073300 -0.000097873952438428096 0.000000000000455767259213283988");
	m_poly.push_back("56750.366999606478 3.3302774952136960 -0.000097824636481615748 0.000000000000455537616975799605");
	m_poly.push_back("56750.367011180548 3.3301796978000651 -0.000097775366799150100 0.000000000000455308190218580852");
	m_poly.push_back("56750.367022754632 3.3300819458294182 -0.000097726108600893665 0.000000000000455078816938316777");
	m_poly.push_back("56750.367034328701 3.3299842468758780 -0.000097676901464291565 0.000000000000454849681430962161");
	m_poly.push_back("56750.367045902778 3.3298865933157531 -0.000097627744012659782 0.000000000000454620777286715185");
	m_poly.push_back("56750.367057476848 3.3297889927166580 -0.000097578604013828598 0.000000000000454391954411957790");
	m_poly.push_back("56750.367069050917 3.3296914374470181 -0.000097529417136686162 0.000000000000454162913242489284");
	m_poly.push_back("56750.367080625001 3.3295939350921602 -0.000097480374838011116 0.000000000000453934545317535496");
	m_poly.push_back("56750.367092199071 3.3294964780005452 -0.000097431354308707193 0.000000000000453706278763546289");
	m_poly.push_back("56750.367103773147 3.3293990737644270 -0.000097382259749330426 0.000000000000453477667478729391");
	m_poly.push_back("56750.367115347217 3.3293017147510211 -0.000097333296365736054 0.000000000000453249667028181775");
	m_poly.push_back("56750.367126921286 3.3292044085231711 -0.000097284318939964634 0.000000000000453021601187189605");
	m_poly.push_back("56750.367138495370 3.3291071474587559 -0.000097235314318022537 0.000000000000452793408703618817");
	m_poly.push_back("56750.367150069447 3.3290099391328640 -0.000097186448137201624 0.000000000000452565860885309612");
	m_poly.push_back("56750.367161643517 3.3289127759121619 -0.000097137559307708155 0.000000000000452338207600345888");
	m_poly.push_back("56750.367173217594 3.3288156653713901 -0.000097088709860259430 0.000000000000452110737700892404");
	m_poly.push_back("56750.367184791663 3.3287185998793030 -0.000097039830054707657 0.000000000000451883126434934478");
	m_poly.push_back("56750.367196365740 3.3286215870156739 -0.000096991107066788290 0.000000000000451656245406542287");
	m_poly.push_back("56750.367207939817 3.3285246191405431 -0.000096942266820917961 0.000000000000451428818352711120");
	m_poly.push_back("56750.367219513893 3.3284277038336909 -0.000096893518821758595 0.000000000000451201820854650780");
	m_poly.push_back("56750.367231087963 3.3283308334642849 -0.000096844845101363424 0.000000000000450975169243498509");
	m_poly.push_back("56750.367242662040 3.3282340156013621 -0.000096796111464127004 0.000000000000450748238621385103");
	m_poly.push_back("56750.367254236109 3.3281372426238631 -0.000096747411773254004 0.000000000000450521466073781299");
	m_poly.push_back("56750.367265810193 3.3280405221044909 -0.000096698821328919157 0.000000000000450295202243591920");
	m_poly.push_back("56750.367277384263 3.3279438463996178 -0.000096650173934275109 0.000000000000450068673217293617");
	m_poly.push_back("56750.367288958332 3.3278472231039138 -0.000096601632455482129 0.000000000000449842637398669807");
	m_poly.push_back("56750.367300532409 3.3277506445666289 -0.000096553030331537719 0.000000000000449616319178508892");
	m_poly.push_back("56750.367312106478 3.3276541183835970 -0.000096504563833173655 0.000000000000449390632512713700");
	m_poly.push_back("56750.367323680563 3.3275576369037450 -0.000096456022212271369 0.000000000000449164596029815087");
	m_poly.push_back("56750.367335254632 3.3274612077192449 -0.000096407592244883872 0.000000000000448939079472695702");
	m_poly.push_back("56750.367346828702 3.3273648231852380 -0.000096359067987963895 0.000000000000448713123845160091");
	m_poly.push_back("56750.367358402778 3.3272684908882080 -0.000096310718303459211 0.000000000000448487981132350205");
	m_poly.push_back("56750.367369976862 3.3271722031870961 -0.000096262268041084738 0.000000000000448262370066789902");
	m_poly.push_back("56750.367381550917 3.3270759676632422 -0.000096213912041597243 0.000000000000448037197945563084");
	m_poly.push_back("56750.367393125001 3.3269797766848050 -0.000096165518978223242 0.000000000000447811853231629303");
	m_poly.push_back("56750.367404699071 3.3268836378328590 -0.000096117322087880917 0.000000000000447587422018124299");
	m_poly.push_back("56750.367416273148 3.3267875434554601 -0.000096069009937999836 0.000000000000447362454085030521");
	m_poly.push_back("56750.367427847217 3.3266915011587712 -0.000096020749267219224 0.000000000000447137725868786223");
	m_poly.push_back("56750.367439421287 3.3265955032856538 -0.000095972558354350096 0.000000000000446913322487551993");
	m_poly.push_back("56750.367450995371 3.3264995574279270 -0.000095924347279721431 0.000000000000446688825219372111");
	m_poly.push_back("56750.367462569448 3.3264036559413910 -0.000095876152192391083 0.000000000000446464402397319880");
	m_poly.push_back("56750.367474143517 3.3263078064174119 -0.000095828094471480190 0.000000000000446240619236076883");
	m_poly.push_back("56750.367485717587 3.3262120012065619 -0.000095779870997813579 0.000000000000446016064227880084");
	m_poly.push_back("56750.367497291663 3.3261162479058952 -0.000095731835943338919 0.000000000000445792386613501618");
	m_poly.push_back("56750.367508865740 3.3260205388667350 -0.000095683835412546742 0.000000000000445568869762000491");
	m_poly.push_back("56750.367520439817 3.3259248816744451 -0.000095635806344427103 0.000000000000445345220021783005");
	m_poly.push_back("56750.367532013886 3.3258292686859119 -0.000095587845240088808 0.000000000000445121886762053924");
	m_poly.push_back("56750.367543587963 3.3257337074938880 -0.000095539813089973329 0.000000000000444898222668208285");
	m_poly.push_back("56750.367555162040 3.3256381904540220 -0.000095491925305161293 0.000000000000444675230827170306");
	m_poly.push_back("56750.367566736109 3.3255427251541732 -0.000095443996432992677 0.000000000000444452047656808221");
	m_poly.push_back("56750.367578310193 3.3254473039465640 -0.000095396068138143192 0.000000000000444228867174252204");
	m_poly.push_back("56750.367589884263 3.3253519344274309 -0.000095348221236405548 0.000000000000444006065706475214");
	m_poly.push_back("56750.367601458333 3.3252566089374311 -0.000095300373135620518 0.000000000000443783258654678578");
	m_poly.push_back("56750.367613032409 3.3251613350904461 -0.000095252524413107274 0.000000000000443560448706325810");
	m_poly.push_back("56750.367624606479 3.3250661052187440 -0.000095204682218740788 0.000000000000443337669156482202");
	m_poly.push_back("56750.367636180563 3.3249709269353040 -0.000095156993839947399 0.000000000000443115605864799419");
	m_poly.push_back("56750.367647754632 3.3248757925700199 -0.000095109335783729706 0.000000000000442893683773081106");
	m_poly.push_back("56750.367659328702 3.3247807097320670 -0.000095061615501394403 0.000000000000442671471917017799");
	m_poly.push_back("56750.367670902779 3.3246856707623769 -0.000095013974294012716 0.000000000000442449628281859612");
	m_poly.push_back("56750.367682476848 3.3245906832653720 -0.000094966250787572280 0.000000000000442227401410409576");
	m_poly.push_back("56750.367694050918 3.3244957395835129 -0.000094918689658791299 0.000000000000442005930668349884");
	m_poly.push_back("56750.367705625002 3.3244008473197950 -0.000094871077104200879 0.000000000000441784220454807705");
	m_poly.push_back("56750.367717199071 3.3243059988139811 -0.000094823512351632281 0.000000000000441562732836230199");
	m_poly.push_back("56750.367728773148 3.3242112016717020 -0.000094776003314798159 0.000000000000441341504662846722");
	m_poly.push_back("56750.367740347217 3.3241164482351002 -0.000094728456468003660 0.000000000000441120100422587915");
	m_poly.push_back("56750.367751921287 3.3240217461034711 -0.000094680996707575545 0.000000000000440899101708412717");
	m_poly.push_back("56750.367763495371 3.3239270876253029 -0.000094633575618645603 0.000000000000440678283071908076");
	m_poly.push_back("56750.367775069448 3.3238324804018209 -0.000094586177080688077 0.000000000000440457569445141386");
	m_poly.push_back("56750.367786643517 3.3237379167695851 -0.000094538727294557731 0.000000000000440236617174949075");
	m_poly.push_back("56750.367798217587 3.3236434043438412 -0.000094491430355880706 0.000000000000440016376654762501");
	m_poly.push_back("56750.367809791664 3.3235489354551548 -0.000094444028487236041 0.000000000000439795647515818411");
	m_poly.push_back("56750.367821365740 3.3234545177109189 -0.000094396737916832845 0.000000000000439575436648183978");
	m_poly.push_back("56750.367832939817 3.3233601434505280 -0.000094349406765337519 0.000000000000439355036809698110");
	m_poly.push_back("56750.367844513887 3.3232658202906360 -0.000094302181414229457 0.000000000000439135129641340202");
	m_poly.push_back("56750.367856087963 3.3231715405581430 -0.000094254942465036022 0.000000000000438915159151352601");
	m_poly.push_back("56750.367867662040 3.3230773118596590 -0.000094207765439977882 0.000000000000438695477016692488");
	m_poly.push_back("56750.367879236110 3.3229831265406351 -0.000094160514047337296 0.000000000000438475448580463420");
	m_poly.push_back("56750.367890810187 3.3228889922149931 -0.000094113435868191690 0.000000000000438256226731030310");
	m_poly.push_back("56750.367902384263 3.3227949011981242 -0.000094066298269587445 0.000000000000438036728187653124");
	m_poly.push_back("56750.367913958333 3.3227008611199569 -0.000094019119175062392 0.000000000000437817036412433293");
	m_poly.push_back("56750.367925532410 3.3226068643101989 -0.000093972129796476817 0.000000000000437598228070382513");
	m_poly.push_back("56750.367937106479 3.3225129183806019 -0.000093925048410989741 0.000000000000437378991286411501");
	m_poly.push_back("56750.367948680563 3.3224190156587210 -0.000093878065347386978 0.000000000000437160212348872778");
	m_poly.push_back("56750.367960254633 3.3223251637663731 -0.000093831015696686955 0.000000000000436941123339594507");
	m_poly.push_back("56750.367971828702 3.3222313550265321 -0.000093784089254765602 0.000000000000436722608065390502");
	m_poly.push_back("56750.367983402779 3.3221375970633860 -0.000093737150849016045 0.000000000000436504037078891575");
	m_poly.push_back("56750.367994976848 3.3220438822006231 -0.000093690215178870875 0.000000000000436285478830508087");
	m_poly.push_back("56750.368006550932 3.3219502180591749 -0.000093643328962768346 0.000000000000436067150869036183");
	m_poly.push_back("56750.368018125002 3.3218565969675442 -0.000093596495362640816 0.000000000000435849067919299905");
	m_poly.push_back("56750.368029699071 3.3217630265418512 -0.000093549706180556965 0.000000000000435631191805807305");
	m_poly.push_back("56750.368041273148 3.3216694991087099 -0.000093502828393313364 0.000000000000435412903091868518");
	m_poly.push_back("56750.368052847232 3.3215760222952491 -0.000093456096499048357 0.000000000000435195293743762383");
#endif /* ENABLE_LBCSIIFSERVICEWORKERTEST */
}

/*!*******************************
* @brief print the most important parameters of a DDstruct structure.
* @param os The stream to print this to.
* @param DD The structure as defined in the ICE interface
* @since 2017-01-30
*/
std::ostream & lbto::operator << (std::ostream & os , const DDstruct & DD)
{
	os << "("
		<< DD.DDname << "," 
		<< DD.DDkey << ","
		<< DD.dataType << ")"  ;
	return os;
}

/*!*******************************
* @brief print the most important parameters of a position structure.
* @param os The stream to print this to.
* @param pos The structure as defined in the ICE interface
* @since 2017-01-30
*/
std::ostream & lbto::operator << (std::ostream & os , const position & pos)
{
	os << "("
		<< pos.coord1 << "," 
		<< pos.coord2 << "," 
		<< pos.system << "," 
		<< pos.epoch << "," 
		<< pos.equinox << "," 
		<< pos.pmcoord1 << "," 
		<< pos.pmcoord2 << "," 
		<< pos.apparentMagnitude << "," 
		<< pos.filter << "," 
		<< pos.color << "," 
		<< pos.colorType << "," 
		<< pos.wavelength << "," 
		<< pos.targname << ")" ;
	return os;
}

/*!*******************************
* @brief print the most important parameters of a position2 structure.
* @param os The stream to print this to.
* @param pos2 The structure as defined in the ICE interface
* @since 2017-01-30
*/
std::ostream & lbto::operator << (std::ostream & os , const position2 & pos2)
{
	os << "("
		<< pos2.coord1 << "," 
		<< pos2.coord2 << "," 
		<< pos2.system << "," 
		<< pos2.epoch << "," 
		<< pos2.equinox << "," 
		// << pos2.pmcoord1 << "," 
		// << pos2.pmcoord2 << "," 
		<< pos2.apparentMagnitude << "," 
		<< pos2.filter << "," 
		<< pos2.color << "," 
		<< pos2.colorType << "," 
		<< pos2.wavelength << "," 
		// << pos2.file << "," 
		<< pos2.time << "," 
		<< pos2.RARate << "," 
		<< pos2.DECRate << ")"  ;
	return os;
}

/*!*******************************
* @brief print the most important parameters of a position2 structure.
* @param os The stream to print this to.
* @param pos2 The structure as defined in the ICE interface
* @since 2017-05-27
*/
std::istream & lbto::operator >> (std::istream & is , position2 & pos2)
{
	is >> pos2.type >> pos2.coord1 
		>> pos2.coord2
		>> pos2.system 
		>> pos2.epoch 
		>> pos2.equinox
		>> pos2.pmcoord1 
		>> pos2.pmcoord2 
		>> pos2.apparentMagnitude 
		>> pos2.filter 
		>> pos2.color 
		>> pos2.colorType 
		>> pos2.wavelength
		>> pos2.time 
		>> pos2.RARate 
		>> pos2.DECRate ;
	return is;
}

/*!*******************************
* @brief print the most important parameters of a nonsidereal structure.
* @param target The structure as defined in the ICE interface
* @since 2017-01-30
*/
std::ostream & lbto::operator << (std::ostream & os , const nonsidereal & target)
{
	os << "("
		<< target.type << "," 
		<< target.wavelength << "," 
		<< target.value0 << "," 
		<< target.value1 << ")" ;
	return os;
}
